#!/usr/bin/python
'''Application main window

Demonstrates a typical application window, with menubar, toolbar, statusbar.'''
# pygtk version: Maik Hertha <maik.hertha@berlin.de>

'''
This is the alignment GUI for CHAMP
Started working from IRCam_GUI
Jan 2010: RMG & FB added basic new buttons and text area with alignment steps instructions.
Jan 2010: FB added auto select of proper beam and quadrant for the GUI plots.
Feb 2011: RMG started adding control of CHARA shutters. but should use CHARA libraries instead...
Apr 2011: FB+RMG added shutter control using CHARA libs - main code in champ server. 
        : FB merged the picogui controls into this align gui, fixed the ensuing cosmetic issues
May 2011: FB fixed the cross when aligning, added "START" disabling when exposure stopped,        
          and added new scanning modes
July 2011: SK+FB added automatic alignment
May 2012: FB automatic calibration and spiral partially working
'''  

import gobject
#from matplotlib.numerix import arange, sin, pi
#from numpy import sin, pi
import numpy as np
import matplotlib 
matplotlib.use('GTK')

from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
from matplotlib.backends.backend_gtk import NavigationToolbar2GTK as NavigationToolbar
from matplotlib.backends.backend_gtk import new_figure_manager
from matplotlib.axes import Subplot
from matplotlib.figure import Figure

import gtk
from matplotlib.lines import Line2D
from matplotlib.patches import Rectangle
import time
import os
import common
import IRCamDisplay
import IRCamCommands
import FTCommands
import childWindows
import alignCommands
import PicoGui
from PicoGui import *

(
  COLOR_RED,
  COLOR_GREEN,
  COLOR_BLUE
) = range(3)

(
  SHAPE_SQUARE,
  SHAPE_RECTANGLE,
  SHAPE_OVAL,
) = range(3)

ui_info = \
'''<ui>
<menubar name='MenuBar'>
  </menubar>
  <toolbar  name='ToolBar'>
    <toolitem action='Quit'/>
    <separator/>
    <toolitem action='Wide Field Mode'/>
    <separator/>
    <toolitem action='Fast Mode'/>
    <separator/>
    <toolitem action='Slow Mode'/>
    <separator/>
    <toolitem action='Normal Mode'/>
    <separator/>    
    <toolitem action='Start exposure'/>
    <separator/>
    <toolitem action='Take BG'/>
    <separator/>
    <toolitem action='Use BG'/>
    <separator/>
    <toolitem action='Reset BG'/>
    <separator/>
    <toolitem action='Stop exposure'/>
    <separator/>
    <toolitem action='Kill Server'/>
    <separator/>    
 </toolbar>
</ui>'''

class ApplicationMainWindow(gtk.Window):
    
    def __init__(self, parent=None):        
    	self.picos = home()
        self.anim = IRCamDisplay.AnimatedImage()
        self.ircc = IRCamCommands.IRCamCommands()
        self.ft = FTCommands.FTCommands()
        self.tag1 = -1
        self.tag2 = -1
        self.alignc = alignCommands.alignCommands()
        self.anim.align_counter = self.alignc.counter

        common.shuttercontrol = 0

        # Create the toplevel window
        gtk.Window.__init__(self)
        try:
            self.set_screen(parent.get_screen())
        except AttributeError:
            self.connect('destroy', lambda *w: gtk.main_quit())
        self.set_title("IRCam - CHAMP")
        self.set_default_size(1700, 710)

        merge = gtk.UIManager()
        self.set_data("ui-manager", merge)
        merge.insert_action_group(self.__create_action_group(), 0)
        self.add_accel_group(merge.get_accel_group())

        try:
            mergeid = merge.add_ui_from_string(ui_info)
        except gobject.GError, msg:
            print "building menus failed: %s" % msg

        bigVBox = gtk.VBox(False, 0)

        table1 = gtk.Table(47, 2, False)
        bigVBox.pack_start(table1, False, False, 0)
        bar = merge.get_widget("/ToolBar")
        bar.set_style(gtk.TOOLBAR_BOTH)
        bar.set_tooltips(True)
        table1.attach(bar,
            # X direction #       # Y direction
            0, 47,                   1, 2,
            gtk.EXPAND | gtk.FILL,  gtk.EXPAND | gtk.FILL,
            0,                      0)


        hpane = gtk.HPaned()
        hpane.set_position(465)

        bigVBox.pack_start(hpane, True, True, 0)

        #left pannel
        cameraParametersVBox = gtk.VBox(False, 2)

	notebook = gtk.Notebook()
	hpane.add1(notebook)

	frame=gtk.Frame()
	frame.add(cameraParametersVBox)

        picosbox = gtk.VBox(False, 2)

	frame4 = gtk.Frame()
        frame4.add(picosbox)
        
        notebook.append_page(frame4, gtk.Label("PICOS"))
#        table.attach(self.picos.frame10,
#            # X direction #          # Y direction
#            0, 1,                      1,2,
#            gtk.EXPAND | gtk.FILL,     gtk.EXPAND | gtk.FILL,
#            0,                         0);
        picosbox.pack_start(self.picos.PT_table, False, False, 0)

                


	notebook.append_page(frame, gtk.Label("FPA"))
        
        hbox = gtk.HBox()
        frame = gtk.Frame("rows")
        self.entry1 = gtk.Entry()
        self.entry1.set_size_request(30, 20)
        self.entry1.set_text(str(self.ircc.rows))
        common.rows = self.ircc.rows
        self.entry1.connect("activate", self.cameraSetEntry, 'rows')
        frame.add(self.entry1)
        hbox.add(frame)
        
        frame = gtk.Frame("cols")
        self.entry2 = gtk.Entry()
        self.entry2.set_size_request(30, 20)
        self.entry2.set_text(str(self.ircc.cols))
        common.cols = self.ircc.cols
        self.entry2.connect("activate", self.cameraSetEntry, 'cols')
        frame.add(self.entry2)
        hbox.add(frame)
        cameraParametersVBox.add(hbox)
        
        hbox = gtk.HBox()
        frame = gtk.Frame("row off")
        self.entry3 = gtk.Entry()
        self.entry3.set_size_request(30, 20)
        self.entry3.set_text(str(self.ircc.rowOffset))
        self.entry3.connect("activate", self.cameraSetEntry, 'rowOffset')
        frame.add(self.entry3)
        hbox.add(frame)
        
        frame = gtk.Frame("col off")
        self.entry4 = gtk.Entry()
        self.entry4.set_size_request(30, 20)
        self.entry4.set_text(str(self.ircc.colOffset))
        self.entry4.connect("activate", self.cameraSetEntry, 'colOffset')
        frame.add(self.entry4)
        hbox.add(frame)
        cameraParametersVBox.add(hbox)
        
        frame = gtk.Frame("NReads")
        self.entry5 = gtk.Entry()
        self.entry5.set_size_request(30, 20)
        self.entry5.set_text(str(self.ircc.nreads))
        self.entry5.connect("activate", self.cameraSetEntry, 'nreads')
        cameraParametersVBox.add(frame)
        frame.add(self.entry5)

        frame = gtk.Frame("Number of frames")
        self.entry6 = gtk.Entry()
        self.entry6.set_size_request(30, 20)
        self.entry6.set_text(str(self.ircc.numberOfFrames))
        self.entry6.connect("activate", self.cameraSetEntry, 'nframes')
        cameraParametersVBox.add(frame)
        frame.add(self.entry6)

        frame = gtk.Frame("Frames per reset")
        self.entry7 = gtk.Entry()
        self.entry7.set_size_request(30, 20)
        self.entry7.set_text(str(self.ircc.framesPerReset))
        self.entry7.connect("activate", self.cameraSetEntry, 'framesPerReset')
        cameraParametersVBox.add(frame)
        frame.add(self.entry7)

        frame = gtk.Frame("image directory")
        self.dirEntry  = gtk.Entry()
        self.dirEntry.set_size_request(30, 20)
        self.dirEntry.set_text(self.ircc.imageDirectory)
        self.dirEntry.connect("activate", self.cameraSetEntry, 'dirname')
        frame.add(self.dirEntry)
        cameraParametersVBox.add(frame)

        frame = gtk.Frame("Image Coadds:")
        entry = gtk.Entry()
        entry.set_size_request(30, 20)
        entry.set_text(str(self.ircc.numberOfCoadds))
        entry.connect("activate", self.cameraSetEntry, 'ncoadds')
        cameraParametersVBox.add(frame)
        frame.add(entry)

        frame = gtk.Frame("Number Backgrounds:")
        entry = gtk.Entry()
        entry.set_size_request(30, 20)
        entry.set_text(str(self.ircc.numBkgnd))
        entry.connect("activate", self.cameraSetEntry, 'numBkgnd')
        cameraParametersVBox.add(frame)
        frame.add(entry)

        button = gtk.Button("Reconfig Camera")
        button.connect("clicked", self.configCamera)
        cameraParametersVBox.add(button)



        frame = gtk.Frame("Files and saved dithers per file:")
        hbox = gtk.HBox()
        entry = gtk.Entry()
        entry.set_size_request(20, 20)
        entry.set_text(str(self.ircc.savenum))
        entry.connect("activate", self.cameraSetEntry, 'savenum')
        hbox.add(entry)
        entry = gtk.Entry()
        entry.set_size_request(20, 20)
        entry.set_text(str(self.ircc.savesPerFile))
        entry.connect("activate", self.cameraSetEntry, 'savesPerFile')
        hbox.add(entry)
	frame.add(hbox)
        cameraParametersVBox.add(frame)


        button = gtk.Button("Save data")
        button.connect("clicked", self.updateDoImageSave)
        cameraParametersVBox.add(button)


#### FT config page ####

        ditherParametersVBox = gtk.VBox(False, 2)

	frame2=gtk.Frame()
	frame2.add(ditherParametersVBox)
	
	notebook.append_page(frame2, gtk.Label("FT"))

	### MODULATION ###
	frame=gtk.Frame("Modulation")
	#
	vbox=gtk.VBox()
	frame.add(vbox)
	#
	hbox=gtk.HBox(True,0)
	label = gtk.Label("Freq (Hz)")
	hbox.pack_start(label,True,True,0)
	self.entry8 = gtk.Entry()
	self.entry8.set_size_request(20, 20)
	self.entry8.set_text(str(self.ft.dither_freq))
	self.entry8.connect("activate", self.ftModSetEntry, 'freq')
	hbox.pack_start(self.entry8,True,True,0)
	vbox.pack_start(hbox,True,True,0)
	#
	hbox=gtk.HBox(True,0)
	label = gtk.Label("Amp (V)")
	hbox.pack_start(label,True,True,0)
	self.entry9 = gtk.Entry()
	self.entry9.set_size_request(20, 20)
	self.entry9.set_text(str(self.ft.dither_amp))
	self.entry9.connect("activate", self.ftModSetEntry, 'amp')
	hbox.pack_start(self.entry9,True,True,0)
	vbox.pack_start(hbox,True,True,0)
	#
	hbox=gtk.HBox(True,0)
	label = gtk.Label("OPD (micron)")
	hbox.pack_start(label,True,True,0)
	self.entry10 = gtk.Entry()
	self.entry10.set_size_request(20, 20)
	self.entry10.set_text(str(self.ft.opdamp))
	self.entry10.connect("activate", self.ftModSetEntry, 'opdamp')
	hbox.pack_start(self.entry10,True,True,0)
	vbox.pack_start(hbox,True,True,0)
	#
        hbox=gtk.HBox(True,0)
        label = gtk.Label("Smooth")
        hbox.pack_start(label,True,True,0)
        self.entry11 = gtk.Entry()
        self.entry11.set_size_request(20, 20)
        self.entry11.set_text(str(self.ft.dither_smoothfrac))
        self.entry11.connect("activate", self.ftModSetEntry, 'smoothfrac')
        hbox.pack_start(self.entry11,True,True,0)
        vbox.pack_start(hbox,True,True,0)
        #
        #
        hbox=gtk.HBox(True,0)
        label = gtk.Label("Delay")
        hbox.pack_start(label,True,True,0)
        self.entry12 = gtk.Entry()
        self.entry12.set_size_request(20, 20)
        self.entry12.set_text(str(self.ft.dither_delay))
        self.entry12.connect("activate", self.ftModSetEntry, 'ditherdelay')
        hbox.pack_start(self.entry12,True,True,0)
        vbox.pack_start(hbox,True,True,0)
        #

	ditherParametersVBox.pack_start(frame, False, False, 0)

	### BEAMS ###
	frame=gtk.Frame("Tracking Modes")
	vbox=gtk.VBox()
	frame.add(vbox)
	#
	hbox=gtk.HBox()
#	label = gtk.Label("Slow")
#	hbox.add(label)
#	label = gtk.Label("1")
#	hbox.add(label)
#	label = gtk.Label("2")
#	hbox.add(label)
#	vbox.pack_start(hbox,False,False,0)
        
        	
	for j in range(3):
            hbox=gtk.HBox(True,0)
            for i in range(3):
                button=gtk.ToggleButton(str(common.rate[j])+'Hz\n'+str(common.stroke[i])+'um')
#                button.set_active(self.ft.active[i])
                button.connect("toggled", self.ftSetMode, [i,j])
                hbox.pack_start(button,True,True,0)
                                        #
            vbox.pack_start(hbox,False,False,0)


	ditherParametersVBox.pack_start(frame, False, False, 0)
	vbox=gtk.VBox()
        button = gtk.Button("Update Server")
        button.connect("clicked", self.ftUpdateServer, '')
	vbox.pack_start(button,True,True,0)
	ditherParametersVBox.pack_start(vbox, False, False, 0)

        serverbox = gtk.VBox(False, 2)
	frame3=gtk.Frame()
	frame3.add(serverbox)
	notebook.append_page(frame3, gtk.Label("SERV"))
        self.host = common.host
        self.port = common.port
        table = gtk.Table(1, 3, False)
        frame = gtk.Frame("server IP")
        hostEntry = gtk.Entry()
        hostEntry.set_size_request(20, 20)
        hostEntry.set_text(self.host)
        hostEntry.connect("activate", self.serverSetEntry, 'host')
        frame.add(hostEntry)
        table.attach(frame,
            # X direction #          # Y direction
            0, 1,                      0,1,
            gtk.EXPAND | gtk.FILL,     gtk.EXPAND | gtk.FILL,
            0,                         0);

        frame = gtk.Frame("server port")
        hostEntry = gtk.Entry()
        hostEntry.set_size_request(20, 20)
        hostEntry.set_text(str(self.port))
        hostEntry.connect("activate", self.serverSetEntry, 'port')
        frame.add(hostEntry)
        table.attach(frame,
            # X direction #          # Y direction
            0, 1,                      1,2,
            gtk.EXPAND | gtk.FILL,     gtk.EXPAND | gtk.FILL,
            0,                         0);
        serverbox.pack_start(table, False, False, 0)


        #right pannel
        table = gtk.Table(47, 20, False)
        hpane.add2(table)       

        
        #some buttons
        self.rb1 = {"2D image":1, "cross section":0, "Option 3":0, "Option 4":0}
        self.rb2  = {"Option 1":1, "Option 2":0, "Option 3":0, "Option 4":0}

        button1 = gtk.RadioButton(None, "2D image")
        button1.connect("toggled", self.radio_button_cb1, "2D image")
        table.attach(button1,
                         # X direction           Y direction
                         0, 2,                   3, 4,
                         gtk.FILL,                      0,
                         0,                      0)

        button1 = gtk.RadioButton(button1, "cross section")
        button1.connect("toggled", self.radio_button_cb1, "cross section")
        table.attach(button1,
                     # X direction           Y direction
                     0, 2,                   4, 5,
                     gtk.FILL,                      0,
                     0,                      0)

        bigFrame = gtk.Frame()
        vbox = gtk.VBox()
        frame = gtk.Frame("Row #")
        self.row_entry1 = gtk.Entry()
        self.row_entry1.set_size_request(10, 20)
        self.row_entry1.set_text(str(0))
        self.row_entry1.connect("activate", self.row_entry_cb, 1)
        frame.add(self.row_entry1)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        vbox.add(frame)

        hbox = gtk.HBox()
        frame = gtk.Frame("cmin")
        self.row_entry1 = gtk.Entry()
        self.row_entry1.set_size_request(10, 20)
        self.row_entry1.set_text(str(0))
        self.row_entry1.connect("activate", self.col_entry_cb, 1, 'cmin')
        frame.add(self.row_entry1)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        hbox.add(frame)

        frame = gtk.Frame("cmax")
        self.row_entry1 = gtk.Entry()
        self.row_entry1.set_size_request(10, 20)
        self.row_entry1.set_text(str(0))
        self.row_entry1.connect("activate", self.col_entry_cb, 1, 'cmax')
        self.row_entry1.set_text(str(common.cols))
        frame.add(self.row_entry1)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        hbox.add(frame)
        vbox.add(hbox)
        bigFrame.add(vbox)
        
        table.attach(bigFrame,
                     # X direction           Y direction
                     0, 2,                   5,6,
                     0,                      0,
                     0,                      0)

        button1 = gtk.RadioButton(button1, "quadrant mode")
        button1.connect("toggled", self.radio_button_cb1, "quadrant mode")
        table.attach(button1,
                         # X direction           Y direction
                         0, 2,                   6,7,
                         0,                      0,
                         0,                      0)

        bbox1 = gtk.HBox()
        bboxv11 = gtk.VBox()
        bboxv12 = gtk.VBox()

        quadButton1 = gtk.ToggleButton('Q3')
        quadButton1.connect("toggled", self.quadrant1, 2) 
        bboxv11.add(quadButton1)
        quadButton1 = gtk.ToggleButton('Q4')
        quadButton1.connect("toggled", self.quadrant1, 3) 
        bboxv12.add(quadButton1)
        quadButton1 = gtk.ToggleButton('Q1')
        quadButton1.connect("toggled", self.quadrant1, 0) 
        bboxv11.add(quadButton1)
        quadButton1 = gtk.ToggleButton('Q2')
        quadButton1.connect("toggled", self.quadrant1, 1) 
        bboxv12.add(quadButton1)

        bbox1.add(bboxv11)
        bbox1.add(bboxv12)
        table.attach(bbox1,
                         # X direction           Y direction
                         0, 2,                   7,9,
                         0,                      0,
                         0,                      0)

 
        button1 = gtk.RadioButton(button1, "power spectrum")
        button1.connect("toggled", self.radio_button_cb1, "power spectrum")
        table.attach(button1,
                         # X direction           Y direction
                         0, 2,                   9,10,
                         gtk.FILL,                      0,
                         0,                      0)

        self.channel = gtk.RadioButton(button1, "channel")
        self.channel.connect("toggled", self.radio_button_cb1, "channel")
        table.attach(self.channel,
                         # X direction           Y direction
                         0, 1,                   10, 11,
                         gtk.FILL,                      0,
                         0,                      0)


#	self.combo = gtk.Combo()
#        self.combo.set_size_request(60, 20)
#	slist=["L1","R1","L2","R2","L3","R3","L4","R4","L5","R5","L6","R6"]
#	self.combo.set_popdown_strings(slist)
#	self.combo.entry.connect("activate", self.psrow_entry_cb, 1)

	self.combo = gtk.combo_box_new_text()
        self.combo.set_size_request(60, 30)
	self.combo.append_text('L1')
        self.combo.append_text('R1')
        self.combo.append_text('L2')
        self.combo.append_text('R2')
        self.combo.append_text('L3')
        self.combo.append_text('R3')
        self.combo.append_text('L4')
        self.combo.append_text('R4')
        self.combo.append_text('L5')
        self.combo.append_text('R5')
        self.combo.append_text('L6')
        self.combo.append_text('R6')        
	self.combo.connect("changed", self.new_psrow_entry_cb)



        table.attach(self.combo,
                         # X direction           Y direction
                         1, 2,                   10,11,
                         0,                      0,
                         0,                      0)

	
	
        frame = gtk.Frame()
        vbox = gtk.VBox()
        self.start_button1 = gtk.Button('   START   ')
        self.start_button1.connect('clicked', self.control_display, "start1")
        self.start_button1.set_sensitive(False)
        vbox.add(self.start_button1)
        
        self.stop_button1 = gtk.Button('   STOP   ')
        self.stop_button1.connect('clicked', self.control_display, "stop1")
        vbox.add(self.stop_button1)

        frame.add(vbox)
        table.attach(frame,
                         # X direction           Y direction
                         0, 2,                   13,15,
                         0,                      0,
                         0,                      0)
        


## Right




        button2 = gtk.RadioButton(None, "2D image")        
        button2.connect("toggled", self.radio_button_cb2, "2D image")
        table.attach(button2,
                         # X direction           Y direction
                         24, 25,                   3, 4,
                         gtk.FILL,                      0,
                         0,                      0)

        button2 = gtk.RadioButton(button2, "cross section")
        button2.connect("toggled", self.radio_button_cb2, "cross section")
        table.attach(button2,
                         # X direction           Y direction
                         24, 25,                   4, 5,
                         gtk.FILL,                      0,
                         0,                      0)

        bigFrame = gtk.Frame()
        vbox = gtk.VBox()
        frame = gtk.Frame("Row #")
        self.row_entry2 = gtk.Entry()
        self.row_entry2.set_size_request(10, 20)
        self.row_entry2.set_text(str(0))
        self.row_entry2.connect("activate", self.row_entry_cb, 2)
        frame.add(self.row_entry2)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        vbox.add(frame)

        hbox = gtk.HBox()
        frame = gtk.Frame("cmin")
        self.row_entry2 = gtk.Entry()
        self.row_entry2.set_size_request(10, 20)
        self.row_entry2.set_text(str(0))
        self.row_entry2.connect("activate", self.col_entry_cb, 2, 'cmin')
        frame.add(self.row_entry2)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        hbox.add(frame)

        frame = gtk.Frame("cmax")
        self.row_entry2 = gtk.Entry()
        self.row_entry2.set_size_request(10, 20)
        self.row_entry2.set_text(str(0))
        self.row_entry2.connect("activate", self.col_entry_cb, 2, 'cmax')
        self.row_entry2.set_text(str(common.cols))
        frame.add(self.row_entry2)
        frame.set_shadow_type(gtk.SHADOW_NONE)
        hbox.add(frame)
        vbox.add(hbox)
        bigFrame.add(vbox)
        table.attach(bigFrame,
                     # X direction           Y direction
                     24, 25,                   5,6,
                     0,                      0,
                     0,                      0)

        self.quadbutton2 = gtk.RadioButton(button2, "quadrant mode")
        self.quadbutton2.connect("toggled", self.radio_button_cb2, "quadrant mode")
        table.attach(self.quadbutton2,
                         # X direction           Y direction
                         24, 25,                   6,7,
                         gtk.FILL,                      0,
                         0,                      0)
        self.quad2 = {'q1':0, 'q2':0, 'q3':0, 'q4':0}
        bbox2 = gtk.HBox()
        bboxv21 = gtk.VBox()
        bboxv22 = gtk.VBox()
        self.q2_button3 = gtk.ToggleButton('Q3')
        self.q2_button3.connect('clicked', self.quadrant2, 2)
        bboxv21.add(self.q2_button3)
        self.q2_button4 = gtk.ToggleButton('Q4')
        self.q2_button4.connect('clicked', self.quadrant2, 3)
        bboxv22.add(self.q2_button4)
        self.q2_button1 = gtk.ToggleButton('Q1')
        self.q2_button1.connect('clicked', self.quadrant2, 0)
        bboxv21.add(self.q2_button1)
        self.q2_button2 = gtk.ToggleButton('Q2')
        self.q2_button2.connect('clicked', self.quadrant2, 1)
        bboxv22.add(self.q2_button2)
        bbox2.add(bboxv21)
        bbox2.add(bboxv22)
        table.attach(bbox2,
                         # X direction           Y direction
                         24, 25,                   7,9,
                         0,                      0,
                         0,                      0)

        button2 = gtk.RadioButton(button2, "focus mode")
        button2.connect("toggled", self.radio_button_cb2, "focus mode")
        table.attach(button2,
                         # X direction           Y direction
                         24, 25,                   9,10,
                         gtk.FILL,                      0,
                         0,                      0)

        button2 = gtk.RadioButton(button2, "future")
        button2.connect("toggled", self.radio_button_cb2, "future")
        table.attach(button2,
                         # X direction           Y direction
                         24, 25,                   10, 11,
                         gtk.FILL,                      0,
                         0,                      0)
        frame = gtk.Frame()
        bbox2 = gtk.HBox()
        bboxv21 = gtk.VBox()
        bboxv22 = gtk.VBox()

        buttonps2 = gtk.RadioButton(None, "row #")
        buttonps2.connect("toggled", self.radio_button_cb_ps2, "row")
        bboxv21.add(buttonps2)

        buttonps2 = gtk.RadioButton(buttonps2, "col #")
        buttonps2.connect("toggled", self.radio_button_cb_ps2, "col")
        bboxv22.add(buttonps2)

        frame = gtk.Frame()
        vbox = gtk.VBox()
        self.start_button2 = gtk.Button('   START   ')
        self.start_button2.connect('clicked', self.control_display, "start2")
        self.start_button2.set_sensitive(False)
        vbox.add(self.start_button2)
	    
        self.stop_button2 = gtk.Button('   STOP   ')
        self.stop_button2.connect('clicked', self.control_display, "stop2")
        vbox.add(self.stop_button2)

        frame.add(vbox)
        table.attach(frame,
                         # X direction           Y direction
                         24, 25,                   13,15,
                         0,                      0,
                         0,                      0)
        
        table.attach(self.anim.canvas1,
            2, 24,                   3, 19,
            gtk.EXPAND|gtk.FILL,               gtk.EXPAND|gtk.FILL,
            0,                      0)
        toolbar1 = NavigationToolbar(self.anim.canvas1, self)
        
	
        table.attach(self.anim.canvas2,
            25, 47,                   3, 19,
            gtk.EXPAND|gtk.FILL,               gtk.EXPAND|gtk.FILL,
            0,                      0)
        toolbar2 = NavigationToolbar(self.anim.canvas2, self)

        table.attach(toolbar1,
            # X direction           Y direction
            2, 24,                   19, 20,
            gtk.EXPAND|gtk.FILL,    gtk.EXPAND|gtk.FILL,
            0,                      0)

        table.attach(toolbar2,
            # X direction           Y direction
            25, 47,                   19, 20,
            gtk.EXPAND|gtk.FILL,    gtk.EXPAND|gtk.FILL,
            0,                      0)

# jan-apr2011 RMG/FB alignment stuff
# may 2012 modified
        frame = gtk.Frame()
#        frame.show()
        vbox = gtk.VBox()

        start_button = gtk.Button('   ALIGN START   ')
        start_button.connect('clicked', self.align_commands, "start_align")
        vbox.add(start_button)
	    
        next_button = gtk.Button('   ALIGN NEXT   ')
        next_button.connect('clicked', self.align_commands, "next_align")
        vbox.add(next_button)

        next_button = gtk.Button('   ALIGN BACK   ')
        next_button.connect('clicked', self.align_commands, "back_align")
        vbox.add(next_button)

        self.autoalign_button = gtk.Button('   AUTO FIND   ')
        self.autoalign_button.connect('clicked', self.align_commands, "auto_find")
        vbox.add(self.autoalign_button)

        self.calib_button = gtk.Button('   CALIBRATE   ')
        self.calib_button.connect('clicked', self.align_commands, "calib")
        vbox.add(self.calib_button)

        self.spiral_button = gtk.Button('   SPIRAL SEARCH   ')
        self.spiral_button.connect('clicked', self.align_commands, "spiral")
        vbox.add(self.spiral_button)

        frame.add(vbox)
        
        table.attach(frame,
                         # X direction           Y direction
                         0, 2,                   21,22,
                         0,                      0,
                         0,                      0)


        frame=gtk.Frame()

# FB may 2012 -- in progress -- buttons to click to access directly the steps
#        hbox = gtk.HBox(False, 2)        
#        aligntable = gtk.Table(1,3, True)
#        self.alignstep = []
#        box1 = gtk.VBox(False, 2)
#        box2 = gtk.VBox(False, 2)
#        box3 = gtk.VBox(False, 2)
#        for i in range(0,18):
#            self.alignstep.append(gtk.Button()) 
#            self.alignstep[i].set_label(' ALIGN STEP '+str(i+1))
#            if(i<6):            
#                box1.pack_start(self.alignstep[i], False, False, 0)
#            if ((i>5)and(i<12)):
#                box2.pack_start(self.alignstep[i], False, False, 0)
#            if(i>11):
#                box3.pack_start(self.alignstep[i], False, False, 0)

#        hbox.add(box1)
#        hbox.add(box2)
#        hbox.add(box3)

#            self.alignstep[i].connect('clicked', self.align_commands, "spiral")

#        frame=gtk.Frame()              
        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        textview = gtk.TextView()
        self.textbuffer = textview.get_buffer()
        textview.set_editable(0)
        textview.set_cursor_visible(0)
        sw.add(textview)
        string = 'Please click ALIGN START to begin the alignment steps.' + '\nCurrent CHAMP server config: ' + str(common.number_beams) + ' beams'
        self.textbuffer.set_text(string)
        frame.add(sw)
#        hbox.add(sw)
#         frame.add(hbox)        

#        table.attach(frame,
#                     # X direction           Y direction
#                     2, 47,                   21,22,
#                     0,    0,
#                     0,                      0)


        table.attach(frame,
                    # X direction           Y direction
                    2, 47,                   21,22,
                    gtk.EXPAND|gtk.FILL,    gtk.EXPAND|gtk.FILL,
                    0,                      0)    

        self.add(bigVBox)
        self.show_all()


    def __create_action_group(self):
        # GtkActionEntry
        entries = (
          ( "Quit", gtk.STOCK_QUIT,                    # name, stock id
            "_Quit", "<control>Q",                     # label, accelerator
            "Quit",                                    # tooltip
            self.quit_cb ),
          ( "Take BG", None,                   # name, stock id
             "_Take BG", None,                              # label, accelerator
            "Take BG",                                    # tooltip
            self.takeBackgroundFrames ),
          ( "Kill Server", gtk.STOCK_DELETE,                   # name, stock id
             "_Kill Server", None,                              # label, accelerator
            "Kill Server",                                    # tooltip
            self.killServer ),
          ( "Reset BG", None,                   # name, stock id
             "_Reset BG", None,                 # label, accelerator
            "Reset BG",                         # tooltip
            self.resetBackgroundFrames ),
          ( "Use BG", None,                     # name, stock id
             "_Use BG", None,                   # label, accelerator
            "Use BG",                           # tooltip
            self.useBackgroundFrames ),
          ( "Wide Field Mode", None,                   # name, stock id
            "_Wide Field Mode", None,                              # label, accelerator
            "Wide Field Mode",                                    # tooltip
            self.widefieldMode ),
          ( "Fast Mode", None,                   # name, stock id
            "_Fast Mode", None,                              # label, accelerator
            "Fast Mode",                                    # tooltip
            self.fastMode ),
          ( "Slow Mode", None,                   # name, stock id
            "_Slow Mode", None,                              # label, accelerator
            "Slow Mode",                                    # tooltip
            self.slowMode ),
          ( "Normal Mode", None,                   # name, stock id
            "_Normal Mode", None,                              # label, accelerator
            "Normal Mode",                                    # tooltip
            self.normalMode ),
          ( "Start exposure", None,                   # name, stock id
             "_Start exposure", None,                              # label, accelerator
            "Start exposure",                                    # tooltip
            self.startExposure ),
          ( "Stop exposure", None,                   # name, stock id
             "_Stop exposure", None,                              # label, accelerator
            "Stop exposure",                                    # tooltip
            self.stopExposure ),
        );
        
        # GtkToggleActionEntry
#        toggle_entries = (
#          ( "Bold", gtk.STOCK_BOLD,                    # name, stock id
#             "_Bold", "<control>B",                    # label, accelerator
#            "Bold",                                    # tooltip
#            self.activate_action,
#            True ),                                    # is_active
#        )

        # GtkRadioActionEntry
#        color_entries = (
#          ( "Red", None,                               # name, stock id
#            "_Red", "<control><shift>R",               # label, accelerator
#            "Blood", COLOR_RED ),                      # tooltip, value
#          ( "Green", None,                             # name, stock id
#            "_Green", "<control><shift>G",             # label, accelerator
#            "Grass", COLOR_GREEN ),                    # tooltip, value
#          ( "Blue", None,                              # name, stock id
#            "_Blue", "<control><shift>B",              # label, accelerator
#            "Sky", COLOR_BLUE ),                       # tooltip, value
#        )

        # GtkRadioActionEntry
#        shape_entries = (
#          ( "Square", None,                            # name, stock id
#            "_Square", "<control><shift>S",            # label, accelerator
#            "Square",  SHAPE_SQUARE ),                 # tooltip, value
#          ( "Rectangle", None,                         # name, stock id
#            "_Rectangle", "<control><shift>R",         # label, accelerator
#            "Rectangle", SHAPE_RECTANGLE ),            # tooltip, value
#          ( "Oval", None,                              # name, stock id
#            "_Oval", "<control><shift>O",              # label, accelerator
#            "Egg", SHAPE_OVAL ),                       # tooltip, value
#        )

        # Create the menubar and toolbar
        action_group = gtk.ActionGroup("AppWindowActions")
        action_group.add_actions(entries)
#        action_group.add_toggle_actions(toggle_entries)
#        action_group.add_radio_actions(color_entries, COLOR_RED, self.activate_radio_action) #FB
#        action_group.add_radio_actions(shape_entries, SHAPE_OVAL, self.activate_radio_action)
        action_group.set_visible(True)
        action_group.set_sensitive(True)

        return action_group

    def quit_cb(self, b):
        print 'Quitting program'
        gtk.main_quit()

    def radio_button_cb_ps1(self, widget, data=None):
	active = widget.get_active()
        if active:
            self.anim.ps1XSec = data
            print "ps1XSec = ", self.anim.ps1XSec

    def radio_button_cb_ps2(self, widget, data=None):
	active = widget.get_active()
        if active:
            self.anim.ps2XSec = data
            print "ps2XSec = ", self.anim.ps2XSec

    def radio_button_cb1(self, widget, data=None):
	active = widget.get_active()
	if active:
            self.anim.plot1Type = data
            print "plot1 type = ", self.anim.plot1Type
	        
    def radio_button_cb2(self, widget, data=None):
	active = widget.get_active()
	if active:
            self.anim.plot2Type = data
            print "plot2 type = ", self.anim.plot2Type
		        
    def setServerParameters(self, action):
        cmd = 'childWindows.main'
	window = eval(cmd)('server')        

    def serverSetEntry(self, widget, data):
        entry = widget.get_text()
        if data == 'host':
            self.host = entry
            common.host = self.host
            print "New IP is: ", self.host

        elif data == 'port':
            self.port = int(entry)
            common.port = self.port
            print "New port is: ", self.port
        
    def delayLinesControl(self, action):
	#cmd = 'childWindows.main'
	#window = eval(cmd)('delayLines')
        #import childWindows_S1E1W1W2 as cw
        #cw.main('delayLines')
        os.system('/home/observe/mirc_python/IRCam/delayLines_S1E1W1W2 &')

    def gdtControl(self, action):
	#cmd = 'childWindows.main'
	#window = eval(cmd)('gdt')
        #import childWindows_S1E1W1W2 as cw
        #cw.main('gdt')
        os.system('/home/observe/mirc_python/IRCam/gdt_S1E1W1W2 &')
        
    def create_shutterMatrix(self, action):
        pass
##         import shutterMatrix
##         shutterMatrix.main()

    def create_fiberExplorer(self, action):
        pass
##         import mircFiberExplorer
##         mircFiberExplorer.main()

    def create_gerbil(self, action):
    	import gerbil
	cmd = 'gerbil.main'
	window1 = eval(cmd)()
    
    def activate_action(self, action):
        dialog = gtk.MessageDialog(self, gtk.DIALOG_DESTROY_WITH_PARENT,
            gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE,
            'You activated action: "%s" of type "%s"' % (action.get_name(), type(action)))
        # Close dialog on user response
        dialog.connect ("response", lambda d, r: d.destroy())
        dialog.show()

    def activate_radio_action(self, action, current):
        active = current.get_active()
        value = current.get_current_value()

        if active:
            dialog = gtk.MessageDialog(self, gtk.DIALOG_DESTROY_WITH_PARENT,
                gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE,
                "You activated radio action: \"%s\" of type \"%s\".\nCurrent value: %d" %
                (current.get_name(), type(current), value))

            # Close dialog on user response
            dialog.connect("response", lambda d, r: d.destroy())
            dialog.show()


    def update_resize_grip(self, widget, event):
        mask = gtk.gdk.WINDOW_STATE_MAXIMIZED | gtk.gdk.WINDOW_STATE_FULLSCREEN
        if (event.changed_mask & mask):
            self.statusbar.set_has_resize_grip(not (event.new_window_state & mask))	

    def quadrant1(self, widget, status):
        if widget.get_active():
            self.anim.quad1[status] = 1
            print "quad1 =", self.anim.quad1
        else:
            self.anim.quad1[status] = 0
            print "quad1 = ", self.anim.quad1

    def quadrant2(self, widget, status):
        if widget.get_active():
            self.anim.quad2[status] = 1
#            print "quad2 =", self.anim.quad2
        else:
            self.anim.quad2[status] = 0
#            print "quad2 = ", self.anim.quad2

    def control_display(self, widget, status):
        common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.float32)
        common.myArrayQuadrantframes = 0
        common.autoalignfind = 1
        common.calib = 0
        self.autoalign_button.set_label('   AUTO FIND   ')
	if (status == "start1"):
	    common.status1 = 1            
            if self.tag1 == -1:
                self.tag1 = gobject.timeout_add(100, self.anim.updatefig1)
	if (status == "stop1"):
	    common.status1 = 0
            if self.tag1 != -1:
                gobject.source_remove(self.tag1)
                self.tag1 = -1
	if (status == "start2"):
	    common.status2 = 1
            if self.tag2 == -1:
                self.tag2 = gobject.timeout_add(100, self.anim.updatefig2)
	if (status == "stop2"):
	    common.status2 = 0
            if self.tag2 != -1:
                gobject.source_remove(self.tag2)
                self.tag2 = -1

    def align_commands(self,widget,status):

        if (status == "start_align"):
            #FB TBD -- start both live windows if not currently running
            common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.float32)
            common.myArrayQuadrantframes = 0
            common.autoalignfind = 1
            common.status1 = 1
            if self.tag1 == -1:
                self.tag1 = gobject.timeout_add(100, self.anim.updatefig1)
	    common.status2 = 1
            if self.tag2 == -1:
                self.tag2 = gobject.timeout_add(100, self.anim.updatefig2)                            
            self.alignc.start_align()
            
        if (status == "next_align"):
            common.autoalignfind = 1
            self.autoalign_button.set_label('   AUTO FIND   ')
            common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.float32)
            common.myArrayQuadrantframes = 0
            self.alignc.next_align()

        if (status == "back_align"):
            common.autoalignfind = 1
            self.autoalign_button.set_label('   AUTO FIND   ')
            common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.float32)
            common.myArrayQuadrantframes = 0
            self.alignc.back_align()

        i = self.alignc.counter

        if (status == "spiral"): # should be disabled unless working on alignment
            #currently supposes we are in quadrant mode
            # and that the background is being removed
            # we also supposed there are no spots anywhere
             print "Starting SPIRAL Search to find the spot"
             print "WARNING -- HIGHLY EXPERIMENTAL"
             spot_detected = 0
             move_stepsize = 1
             move_direction = 1
             spiral_countdown = 8
             startimage =  common.myArrayQuadrant # store initial image
             threshold = startimage.max()*10. # we want SNR > 10
             print "Threshold", threshold 
             while ((spot_detected == 0) and  (spiral_countdown > 0)):             
                 if(spiral_countdown % 2)==0:
                     print "X move"
                     self.picos.MoveRelativeXY(move_direction*move_stepsize, 0)
                     sleep(2*move_stepsize)                     
                 else:
                    #Y move
                    print "Y move"
                    self.picos.MoveRelativeXY(0, move_direction*move_stepsize)
                    sleep(2*move_stepsize)

                 self.anim.updatefig1()
                 self.anim.updatefig2()
                 newimage =  common.myArrayQuadrant # update image
                 # test spot detection
                 diff = np.fabs(newimage - startimage)
                 if ( diff.max > threshold): # spot was detected
                     print "Diff ", diff.max()
                     spot_detected = 1                  
                 else: #no spot found
                     spiral_countdown = spiral_countdown - 1
                     if(spiral_countdown % 2)==0:
                         move_direction = -move_direction
                         move_stepsize = 2*move_stepsize
                     print "Move intensity", move_stepsize*move_direction 

             if(spot_detected == 1):
                 print "Spot detection ?"
             else:
                 print "No spot detected in the allowed spiral moves"
                                           
        if (status == "auto_find"):

            if( common.autoalignfind == 1):
                #find the peak position and move automatically to the target position (this is done in IRCamDisplay.py)
                print "AUTO FIND"
                common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.float32)
                common.myArrayQuadrantframes = 0
                common.autoalignfind = 0                
                self.autoalign_button.set_label('   AUTO MOVE   ')
            else:
                print "AUTO MOVE"
                #compute difference between peak position and target position
                delta_pix = [common.xtarget[i]-common.xmove, common.ytarget[i]-common.ymove]

                # safety check, don't try to move by more than 2 pixels
                # TBD: change for widescreen mode alignment
                if ( abs(delta_pix[0]) > 1.5):
                   delta_pix[0] = 1 
                if ( abs(delta_pix[1]) > 1.5):
                   delta_pix[1] = 1 
                  
                print "MOVE by ", delta_pix[0], delta_pix[1], " pixels"
                #do move
                self.picos.MoveRelativeXY(delta_pix[0], -delta_pix[1])  	#Command function in PicoLowLevel that sends telnet command to move the picomotor
                # put back the button into FIND mode
                common.autoalignfind = 1
                self.autoalign_button.set_label('   AUTO FIND   ')

        if (status == "calib"):
            #2012 May FB: undergoing code revision
            
            print "Calibrating the pico"
            common.calib = 1
            move_stepsize = 6
            safety_factor_movement = 5
            safety_factor_flux = 3

            picoIDX = picoData.picoId.field('picoId').search(picoData.label)[0][0]
            picoIDY = picoData.picoId.field('picoId').search(picoData.label)[0][1]

            print "picoIDs: X=", picoIDX, "  Y=", picoIDY

            #start with default values
            common.plus_scales[picoIDX] = common.def_plus_scales[picoIDX]
            common.minus_scales[picoIDX] = common.def_minus_scales[picoIDX]
            common.angle_scales[picoIDX] = common.def_angle_scales[picoIDX]
            common.plus_scales[picoIDY] = common.def_plus_scales[picoIDY]
            common.minus_scales[picoIDY] = common.def_minus_scales[picoIDY]
            common.angle_scales[picoIDY] = common.def_angle_scales[picoIDY]


            #*** calibrate X-axis
            print
            print "*** Calibrate X-axis for", picoData.label

            common.plus_scales[picoIDX] = common.def_plus_scales[picoIDX]
            common.minus_scales[picoIDX] = common.def_minus_scales[picoIDX]
            common.angle_scales[picoIDX] = common.def_angle_scales[picoIDX]
            common.plus_scales[picoIDY] = common.def_plus_scales[picoIDY]
            common.minus_scales[picoIDY] = common.def_minus_scales[picoIDY]
            common.angle_scales[picoIDY] = common.def_angle_scales[picoIDY]

            moveX = move_stepsize
            moveY = 0

            common.dataready2 = 0
            while common.dataready2 == 0:
                sleep(0.1)
            common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.float32)
            common.myArrayQuadrantframes = 0
            self.anim.updatefig1()
            self.anim.updatefig2()

            pos0X = common.xmove
            pos0Y = common.ymove
            flux0 = common.peakval
            print "POS0: ", pos0X, pos0Y, "   flux=", flux0

            #move predefined number of steps (plus direction)
            self.picos.MoveRelativeXY(moveX, moveY)
            sleep(5)

            common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.Float32)
            common.myArrayQuadrantframes = 0
            common.dataready2 = 0
            while common.dataready2 == 0:
                sleep(0.1)
            self.anim.updatefig1()
            self.anim.updatefig2()

            pos1X = common.xmove
            pos1Y = common.ymove
            flux1 = common.peakval
            amp1 = common.stepsS / math.sqrt(math.pow(pos1Y-pos0Y, 2.) + math.pow(pos1X-pos0X, 2.))
            print "POS-: ", pos1X, pos1Y, "   flux=", flux1
            print "Measured movement: ", pos1X-pos0X, pos1Y-pos0Y


            #move predefined number of steps (minus direction)
            self.picos.MoveRelativeXY(-moveX, -moveY)
            sleep(5)

            common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.float32)
            common.myArrayQuadrantframes = 0
            common.dataready2 = 0
            while common.dataready2 == 0:
                sleep(0.1)
            self.anim.updatefig1()
            self.anim.updatefig2()

            pos2X = common.xmove
            pos2Y = common.ymove
            flux2 = common.peakval
            amp2 = common.stepsS / math.sqrt(math.pow(pos2Y-pos1Y, 2.) + math.pow(pos2X-pos1X, 2.))
            print "POS+: ", pos2X, pos2Y, "   flux=", flux2
            print "Measured movement: ", pos2X-pos1X, pos2Y-pos1Y

            
            angle = math.atan2((pos1Y-pos0Y)-(pos2Y-pos1Y), (pos1X-pos0X)-(pos2X-pos1X))*180./math.pi
            print pos2Y-pos1Y, pos2X-pos1X, math.atan2((pos2Y-pos1Y), pos2X-pos1X)*180./math.pi-180.
            print pos1Y-pos0Y, pos1X-pos0X, math.atan2((pos1Y-pos0Y), pos1X-pos0X)*180./math.pi
            print (pos1Y-pos0Y)-(pos2Y-pos1Y), (pos1X-pos0X)-(pos2X-pos1X), math.atan2((pos1Y-pos0Y)-(pos2Y-pos1Y), (pos1X-pos0X)-(pos2X-pos1X))*180./math.pi
                        
            print "Old picoScales for entry #", picoIDX, ":"
            print common.plus_scales[picoIDX], common.minus_scales[picoIDX], common.angle_scales[picoIDX]

            new_plus = int(amp1)
            new_minus = -int(amp2)

            if (abs(new_plus/common.plus_scales[picoIDX]) > safety_factor_movement) and (flux0/flux1 < safety_factor_flux) and (flux0/flux2 < safety_factor_flux):
                print "WARNING: Skipping this value, because step size changed by more than factor ", safety_factor, " -> please redo calibration!"
            else:
                common.plus_scales[picoIDX] = new_plus
                common.minus_scales[picoIDX] = new_minus
                common.angle_scales[picoIDX] = angle

            print "New picoScales for entry #", picoIDX, ":"
            print common.plus_scales[picoIDX], common.minus_scales[picoIDX], common.angle_scales[picoIDX]

            if (abs(new_plus/common.plus_scales[picoIDX]) <= safety_factor_movement):
                print "WARNING: Please check that the new values make sense! If not, please repeat calibration or restore values with 'cp picoScales-default.dat picoScales.dat'!"


            #*** calibrate Y-axis
            print
            print "*** Calibrate Y-axis for", picoData.label

            moveX = 0
            moveY = move_stepsize

            common.dataready2 = 0
            while common.dataready2 == 0:
                sleep(0.1)
            common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.float32)
            common.myArrayQuadrantframes = 0
            self.anim.updatefig1()
            self.anim.updatefig2()

            pos0X = common.xmove
            pos0Y = common.ymove
            flux0 = common.peakval
            print "POS0: ", pos0X, pos0Y, "   flux=", flux0

            #move predefined number of steps (plus direction)
            self.picos.MoveRelativeXY(moveX, moveY)
            sleep(5)

            common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.float32)
            common.myArrayQuadrantframes = 0
            common.dataready2 = 0
            while common.dataready2 == 0:
                sleep(0.1)
            self.anim.updatefig1()
            self.anim.updatefig2()

            pos1X = common.xmove
            pos1Y = common.ymove
            flux1 = common.peakval
            amp1 = common.stepsT / math.sqrt(math.pow(pos1Y-pos0Y, 2.) + math.pow(pos1X-pos0X, 2.))
            print "POS-: ", pos1X, pos1Y, "   flux=", flux1
            print "Measured movement: ", pos1X-pos0X, pos1Y-pos0Y


            #move predefined number of steps (minus direction)
            self.picos.MoveRelativeXY(-moveX, -moveY)
            sleep(5)

            common.myArrayQuadrant = np.zeros((common.rows/2, common.cols/2), dtype=np.float32)
            common.myArrayQuadrantframes = 0
            common.dataready2 = 0
            while common.dataready2 == 0:
                sleep(0.1)
            self.anim.updatefig1()
            self.anim.updatefig2()

            pos2X = common.xmove
            pos2Y = common.ymove
            flux2 = common.peakval
            amp2 = common.stepsT / math.sqrt(math.pow(pos2Y-pos1Y, 2.) + math.pow(pos2X-pos1X, 2.))
            print "POS+: ", pos2X, pos2Y, "   flux=", flux2
            print "Measured movement: ", pos2X-pos1X, pos2Y-pos1Y

            
            angle = math.atan2((pos1Y-pos0Y)-(pos2Y-pos1Y), (pos1X-pos0X)-(pos2X-pos1X))*180./math.pi
            print pos2Y-pos1Y, pos2X-pos1X, math.atan2((pos2Y-pos1Y), pos2X-pos1X)*180./math.pi-90.
            print pos1Y-pos0Y, pos1X-pos0X, math.atan2((pos1Y-pos0Y), pos1X-pos0X)*180./math.pi+90.
            print (pos1Y-pos0Y)-(pos2Y-pos1Y), (pos1X-pos0X)-(pos2X-pos1X), math.atan2((pos1Y-pos0Y)-(pos2Y-pos1Y), (pos1X-pos0X)-(pos2X-pos1X))*180./math.pi+90.
                        
            print "Old picoScales for entry #", picoIDY, ":"
            print common.plus_scales[picoIDY], common.minus_scales[picoIDY], common.angle_scales[picoIDY]

            new_plus = int(amp1)
            new_minus = -int(amp2)

            if (abs(new_plus/common.plus_scales[picoIDY]) > safety_factor_movement) and (flux0/flux1 < safety_factor_flux) and (flux0/flux2 < safety_factor_flux):
                print "WARNING: Skipping this value, because step size changed by more than factor ", safety_factor, " -> please redo calibration!"
            else:
                common.plus_scales[picoIDY] = new_plus
                common.minus_scales[picoIDY] = new_minus
                common.angle_scales[picoIDY] = (angle+90.)

            print "New picoScales for entry #", picoIDY, ":"
            print common.plus_scales[picoIDY], common.minus_scales[picoIDY], common.angle_scales[picoIDY]

            if (abs(new_plus/common.plus_scales[picoIDY]) <= safety_factor_movement):
                print "WARNING: Please check that the new values make sense! If not, please repeat calibration or restore default values with 'cp picoScales-default.dat picoScales.dat'!"

            #write new matrix to file
            file = open("picoScales.dat", 'w')
            for n in range(48):
                file.write('%d,%d,%d\n' %(common.plus_scales[n], common.minus_scales[n], common.angle_scales[n]))

        self.anim.align_counter = i

        if i < len(common.step):
            string = 'step: ' + str(common.step[i]) + '\nbeam: '+ str(common.beam[i]) + ' | spot: ' + common.spot_name[i] +\
                     ' | quadrant: ' + str(common.spot_quadrant[i]) + ' | position: ' + common.position_name[common.position_index[i]-1] +\
                     ' | pico: ' + common.pico_name[i] + '\nComment: ' + common.additional_info[i]
            self.textbuffer.set_text(string)

            ## rmg/fb apr2011 - new shutter code
            if (common.shuttercontrol == 1):
                self.openShutter(common.beam[i])

            #FB april 2011 - selection of the picos
#            for p in range(24):
#                self.picos.mount_buttons[p].set_sensitive(False)

            self.picos.mount_buttons[common.pico_allowed[i]].set_sensitive(True)
            self.picos.mount_buttons[common.pico_allowed[i]].activate()

            # Quadrant selection

            #FB set quadrant mode button near display2
            self.quadbutton2.set_active(1)

            # FB toggle quadrant button
            # q2active = common.spot_quadrant[i]-1
            q2active = common.spot_quadrant[i]-1
            if (q2active == 0):
                self.q2_button1.set_active(1)
                self.q2_button2.set_active(0)
                self.q2_button3.set_active(0)
                self.q2_button4.set_active(0)
            
            if (q2active == 1):
                self.q2_button1.set_active(0)
                self.q2_button2.set_active(1)
                self.q2_button3.set_active(0)
                self.q2_button4.set_active(0)

            if (q2active == 2):
                self.q2_button1.set_active(0)
                self.q2_button2.set_active(0)
                self.q2_button3.set_active(1)
                self.q2_button4.set_active(0)

            if (q2active == 3):
                self.q2_button1.set_active(0)
                self.q2_button2.set_active(0)
                self.q2_button3.set_active(0)
                self.q2_button4.set_active(1)

            # FB: select active quadrant on display2
            self.anim.quad2[0] = 0
            self.anim.quad2[1] = 0
            self.anim.quad2[2] = 0
            self.anim.quad2[3] = 0
            self.anim.quad2[q2active] = 1
        
            # Cross section profile

            # FB set channel mode 
            self.channel.set_active(1)

            # FB set active drop down selection
            # self.combo.set_active(common.spot_number[i])
            self.combo.set_active(common.spot_number[i])

            # FB set display to the right channel
            # self.anim.psxrow1 = common.spot_number[i]
            self.anim.psxrow1 = common.spot_number[i]
            
        else:
            string = 'DONE ... no more steps!'
            self.textbuffer.set_text(string)
            for p in range(24):
                self.picos.mount_buttons[p].set_sensitive(True)    

            if (common.shuttercontrol == 1):
                self.openShutter(0) # this means open all shutters


    def row_entry_cb(self, widget, n):
        entry_text = widget.get_text()
        if n == 1:
            self.anim.xrow1 = int(entry_text)
            print "New row 1: %d\n" % self.anim.xrow1
            if self.anim.xrow1 > common.rows:
                self.anim.xrow1 = common.rows
                widget.set_text(str(self.anim.xrow1))
        if n == 2:
            self.anim.xrow2 = int(entry_text)
            print "New row 2: %d\n" % self.anim.xrow2
            if self.anim.xrow2 > common.rows:
                self.anim.xrow2 = common.rows
                widget.set_text(str(self.anim.xrow2))

    def col_entry_cb(self, widget, n, m):
        entry_text = widget.get_text()
        if n == 1:
            if m == 'cmin':
                self.anim.cmin1 = int(entry_text)
                if self.anim.cmin1 < 0:
                    self.anim.cmin1 = 0
                    widget.set_text(str(self.anim.cmin1))
             
            if m == 'cmax':
                self.anim.cmax1 = int(entry_text)
                if self.anim.cmax1 > common.cols:
                    self.anim.cmax1 = common.cols
                    widget.set_text(str(self.anim.cmax1))
        if n == 2:
            if m == 'cmin':
                self.anim.cmin2 = int(entry_text)
                if self.anim.cmin2 < 0:
                    self.anim.cmin2 = 0
                widget.set_text(str(self.anim.cmin2))
             
            if m == 'cmax':
                self.anim.cmax2 = int(entry_text)
                if self.anim.cmax2 > common.cols:
                    self.anim.cmax2 = common.cols
                    widget.set_text(str(self.anim.cmax2))

    def new_psrow_entry_cb(self, widget):
        self.anim.psxrow1=widget.get_active()

#    def psrow_entry_cb(self, widget, n):
#        slist=["L1","R1","L2","R2","L3","R3","L4","R4","L5","R5","L6","R6"]
#        entry_text = widget.get_text()
	#print n,entry_text
 #       if n == 1:
 #           for tempi in range(12):
#		if (slist[tempi] in entry_text): self.anim.psxrow1=tempi
            #widget.set_text(str(self.anim.psxrow1))
#        if n == 2:
#            self.anim.psxrow2 = int(entry_text)
#            print "New ps row 2: %d\n" % self.anim.psxrow2
#            if self.anim.psxrow2 > common.rows/2:
#                self.anim.psxrow2 = common.rows/2
 #               widget.set_text(str(self.anim.psxrow2))

    def pscol_entry_cb(self, widget, n):
        entry_text = widget.get_text()
        if n == 1:
            self.anim.psxcol1 = int(entry_text)
            print "New ps col 1: %d\n" % self.anim.psxcol1
            if self.anim.psxcol1 > 1+common.cols/2:
                self.anim.psxcol1 = 1+common.cols/2
                widget.set_text(str(self.anim.psxcol1))
        if n == 2:
            self.anim.psxcol2 = int(entry_text)
            print "New ps col 2: %d\n" % self.anim.psxcol2
            if self.anim.psxcol2 > 1+common.cols/2:
                self.anim.psxcol2 = 1+common.cols/2
                widget.set_text(str(self.anim.psxcol2))
            
    def checkButton(self, widget, data=None):
        if data == "doReset":
            self.ircc.doReset = widget.get_active()
            print "doReset = ", self.ircc.doReset
            
        elif data == "doImageSave":
            self.ircc.doImageSave = widget.get_active()
            print "doImageSave = ", self.ircc.doImageSave
            
        elif data == "read":
            self.ircc.read = widget.get_active()
            print "read = ", self.ircc.read

        elif data == "continuousReadout":
            self.ircc.continuousReadout = widget.get_active()
            print "continuousReadout= ", self.ircc.continuousReadout

        elif data == "useBackground":
            self.ircc.useBackground = widget.get_active()
            print "useBackground = ", self.ircc.useBackground

	       
    def getDirectory(self, widget):
        dirChooser = gtk.FileChooserDialog("Open..",
                                            None,
                                            gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                             gtk.STOCK_APPLY, gtk.RESPONSE_OK))
        dirChooser.set_default_response(gtk.RESPONSE_OK)
        response = dirChooser.run()
        if response == gtk.RESPONSE_OK:
            self.imageDirectory = dirChooser.get_current_folder()
            print self.imageDirectory
        elif response == gtk.RESPONSE_CANCEL:
            print 'Closed, no directory selected'
        self.dirEntry.set_text(self.imageDirectory)
        dirChooser.destroy()

    def cameraSetEntry(self, widget, data):
        entry = widget.get_text()
        if data == 'rows':
            self.ircc.rows = int(entry)
            common.rows = self.ircc.rows
            print "new rows = ",  self.ircc.rows

        elif data == 'cols':
            self.ircc.cols = int(entry)
            common.cols = self.ircc.cols
            print "new cols = ",  self.ircc.cols

        elif data == 'rowOffset':
            self.ircc.rowOffset = int(entry)
            print "new rowOffset = ",  self.ircc.rowOffset
            
        elif data == 'colOffset':
            self.ircc.colOffset = int(entry)
            print "new colOffset = ",  self.ircc.colOffset

        elif data == 'nreads':
            self.ircc.nreads = int(entry)
            print "new nreads = ",  self.ircc.nreads

        elif data == 'nframes':
            self.ircc.numberOfFrames = int(entry)
            print "new nframes = ",  self.ircc.numberOfFrames
	    common.numberOfFrames = self.ircc.numberOfFrames

        elif data == 'numberCoherent':
            self.ircc.numberCoherent = int(entry)
            print "new numberCoherent = ",  self.ircc.numberCoherent

        elif data == 'ncoadds':
            self.ircc.numberOfCoadds = int(entry)
            print "new ncoadds = ",  self.ircc.numberOfCoadds

        elif data == 'framesPerBuffer':
            self.ircc.framesPerBuffer = int(entry)
#	    common.numberOfFrames = self.ircc.framesPerBuffer
            print "new framesPerBuffer = ",  self.ircc.framesPerBuffer

        elif data == 'framesPerReset':
            self.ircc.framesPerReset = int(entry)
            common.framesPerReset = self.ircc.framesPerReset
            print "new framesPerReset = ",  self.ircc.framesPerReset

        elif data == 'savenum':
            self.ircc.savenum = int(entry)
            print "new savenum = ",  self.ircc.savenum
            
        elif data == 'savesPerFile':
            self.ircc.savesPerFile = int(entry)
            print "new savesPerFile = ",  self.ircc.savesPerFile
            
        elif data == 'numBkgnd':
            self.ircc.numBkgnd = int(entry)
            print "new numBkgnd = ",  self.ircc.numBkgnd
            
        elif data == 'dirname':
            self.ircc.imageDirectory = entry
            print "new imageDirectory = ",  self.ircc.imageDirectory

        elif data == 'filename':
            self.ircc.imageFilename = entry
            print "new imageFilename = ",  self.ircc.imageFilename

    def updateParameters(self, widget):
        self.ircc.sendStruct()
                
    def updateDaqStructure(self, widget):
        print "Updating DAQ..."
        self.ircc.updateDaqStructure()
        print "Done"
        
    def updateDoImageSave(self, widget):
        self.ircc.updateDoImageSave()
        
    def resetController(self, widget):
        self.ircc.resetCont()
        
    def setController(self, widget):
        self.ircc.setCont()

    def configCamera(self, widget):
        self.ircc.updateDaqStructure()
        time.sleep(0.2)
        self.ircc.sendStruct()
        time.sleep(0.4)
        self.ircc.resetCont()
        time.sleep(0.5)
        self.ircc.setCont()
        time.sleep(0.5)
        self.ircc.setBias()

        
    def setPicnicBias(self, widget):
        self.ircc.setBias()

    def reOPLE(self, widget):
        self.ircc.reOPLE()


    def widefieldMode(self, widget): #FB apr 2011
        print "Entering widefield mode"
        common.widefield = 1
        self.ircc.rows = 100
        self.ircc.cols = 100
        self.ircc.rowOffset = 1 
        self.ircc.colOffset = 1
        self.ircc.nreads = 4 
        self.ircc.numberOfCoadds = 1
        self.ircc.framesPerReset = 10
        self.ircc.expTime = 0
        self.ircc.numberOfFrames = 20
        self.ircc.byteSwapping  =False
        self.ircc.doReset = True
        self.ircc.doDeinterlace = False
        self.ircc.doImageSave = False
        self.ircc.daqRunning = False
        self.ircc.read = False
        self.ircc.voffset = 4.6 
        self.ircc.vreset = 0.5
        self.ircc.vd = 5.0
        self.ircc.ictl = 3.3
        self.ircc.continuousReadout = True
	self.ircc.savenum = 0 
	self.ircc.savesPerFile = 100 
	self.ircc.numBkgnd = 100 
	self.ircc.numberCoherent = 1
        self.ircc.useBackground = False
        self.ircc.numberOfBackgroundFrames = 1000
        common.rows = self.ircc.rows
        common.cols = self.ircc.cols
        common.roff = (7-1)
        common.coff = (20-1)
        common.numberOfFrames = self.ircc.numberOfFrames
        common.framesPerReset = self.ircc.framesPerReset
            
        # 1- Update DAQ -- warning, some values are reset e.g. gains which is annoying !
        self.ircc.updateDoImageSave()
        # 2- Update Server
	self.ft.dither_freq = 3.0
	self.ft.dither_amp = 1.05
        self.ft.opdamp = 13.2
	self.ft.dither_smoothfrac = 0.05
        self.ft.dither_delay = 0.0
	self.ft.active=[0,0,0,0,0,0]
	self.ft.offset=[0.0,0.0,0.0,0.0,0.0,0.0]
	self.ft.gain=[1,1,1,1,1,1]
	self.ft.search_state=[0,0,0,0,0,0]
        self.ft.search_step=[0.0,0.0,0.0,0.0,0.0,0.0]
 	self.ft.search_threshold=[1000.,1000.,1000.,1000.,1000.,1000.]
        self.ft.opdmapsfile = 'jdm_opds6Hz.txt'
        print " Switching to Wide Field mode"
        print "     dither frequency = ",  self.ft.dither_freq
	print "     dither amp = ",  self.ft.dither_amp
        print "     opd amp = ",  self.ft.opdamp
	print "     dither smoothing frac = ",  self.ft.dither_smoothfrac
#	print "     Beam params...."
#	for i in range(6):
#		print i,self.ft.active[i], self.ft.offset[i], self.ft.gain[i]

        #update gui entries to reflect the changes
        self.entry1.set_text(str(self.ircc.rows))
        self.entry2.set_text(str(self.ircc.cols))
        self.entry3.set_text(str(self.ircc.rowOffset))
        self.entry4.set_text(str(self.ircc.colOffset))
        self.entry5.set_text(str(self.ircc.nreads))
	self.entry6.set_text(str(self.ircc.numberOfFrames))
        self.entry7.set_text(str(self.ircc.framesPerReset))
        self.entry8.set_text(str(self.ft.dither_freq))
        self.entry9.set_text(str(self.ft.dither_amp))
        self.entry10.set_text(str(self.ft.opdamp))
        self.entry11.set_text(str(self.ft.dither_smoothfrac))
        self.entry12.set_text(str(self.ft.dither_delay))
        
        time.sleep(0.5)      
        self.ft.updateFTStructure()
        # 3- Config Camera -- some values are reset, but this is expected
        self.ircc.updateDaqStructure()
        time.sleep(0.2)
        self.ircc.sendStruct()
        time.sleep(0.4)
        self.ircc.resetCont()
        time.sleep(0.5)
        self.ircc.setCont()
        time.sleep(0.5)
        self.ircc.setBias()
        print "Camera now in widefield mode"

    def fastMode(self, widget): #FB jul 2011 #may 2012, new pzt mode
        print "Entering fast mode"
        common.widefield = 0         
        self.ircc.rows = 16
        self.ircc.cols = 16
        self.ircc.rowOffset = 7 
        self.ircc.colOffset = 18
        self.ircc.nreads = 25 # 10
        self.ircc.numberOfCoadds = 1
        self.ircc.framesPerReset = 105 #21
        self.ircc.expTime = 0
        self.ircc.numberOfFrames = 210 #42
        self.ircc.byteSwapping  =False
        self.ircc.doReset = True
        self.ircc.doDeinterlace = False
        self.ircc.doImageSave = False
        self.ircc.daqRunning = False
        self.ircc.read = False
        self.ircc.voffset = 4.6 
        self.ircc.vreset = 0.5
        self.ircc.vd = 5.0
        self.ircc.ictl = 3.3
        self.ircc.continuousReadout = True
	self.ircc.savenum = 0 
	self.ircc.savesPerFile = 100 
	self.ircc.numBkgnd = 30 
	self.ircc.numberCoherent = 1
        self.ircc.useBackground = False
        self.ircc.numberOfBackgroundFrames = 1000
        self.ircc.updateDaqStructure()
        common.rows = self.ircc.rows
        common.cols = self.ircc.cols
        common.roff = 0
        common.coff = 0
        common.numberOfFrames = self.ircc.numberOfFrames
        common.framesPerReset = self.ircc.framesPerReset

        # 1- Update DAQ -- warning, some values are reset e.g. gains which is annoying !
        self.ircc.updateDoImageSave()
        # 2- Update Server
	self.ft.dither_freq = 2.6 # 29.257127
	self.ft.dither_amp = 5.12 #1.05
        self.ft.opdamp = 58.0 # 13.2
	self.ft.dither_smoothfrac = 0.05
        self.ft.dither_delay = 0.025
	self.ft.active=[0,0,0,0,0,0]
	self.ft.offset=[0.0,0.0,0.0,0.0,0.0,0.0]
	self.ft.gain=[1,1,1,1,1,1]
	self.ft.search_state=[0,0,0,0,0,0]
        self.ft.search_step=[0.0,0.0,0.0,0.0,0.0,0.0]
 	self.ft.search_threshold=[1000.,1000.,1000.,1000.,1000.,1000.]
        self.ft.opdmapsfile='jdm_opds30Hz.txt'
        print " Switching to alignement mode"
        print "     dither frequency = ",  self.ft.dither_freq
	print "     dither amp = ",  self.ft.dither_amp
        print "     opd amp = ",  self.ft.opdamp
	print "     dither smoothing frac = ",  self.ft.dither_smoothfrac
#	print "     Beam params...."
#	for i in range(6):
#		print i,self.ft.active[i], self.ft.offset[i], self.ft.gain[i]

        #update gui entries to reflect the changes
        self.entry1.set_text(str(self.ircc.rows))
        self.entry2.set_text(str(self.ircc.cols))
        self.entry3.set_text(str(self.ircc.rowOffset))
        self.entry4.set_text(str(self.ircc.colOffset))
        self.entry5.set_text(str(self.ircc.nreads))
	self.entry6.set_text(str(self.ircc.numberOfFrames))
        self.entry7.set_text(str(self.ircc.framesPerReset))
        self.entry8.set_text(str(self.ft.dither_freq))
        self.entry9.set_text(str(self.ft.dither_amp))
        self.entry10.set_text(str(self.ft.opdamp))
        self.entry11.set_text(str(self.ft.dither_smoothfrac))
        self.entry12.set_text(str(self.ft.dither_delay))

        time.sleep(0.5) 
        self.ft.updateFTStructure()
        # 3- Config Camera -- some values are reset, but this is expected

        time.sleep(0.2)
        self.ircc.sendStruct()
        time.sleep(0.4)
        self.ircc.resetCont()
        time.sleep(0.5)
        self.ircc.setCont()
        time.sleep(0.5)
        self.ircc.setBias()
        print "Camera now in fast mode"

    def normalMode(self, widget): 
        print "Entering normal mode"
        common.widefield = 0
        self.ircc.rows = 16
        self.ircc.cols = 16
        self.ircc.rowOffset = 7
        self.ircc.colOffset = 18
        self.ircc.nreads = 31 
        self.ircc.numberOfCoadds = 1
        self.ircc.framesPerReset = 231
        self.ircc.expTime = 0
        self.ircc.numberOfFrames = 462
        self.ircc.byteSwapping  =False
        self.ircc.doReset = True
        self.ircc.doDeinterlace = False
        self.ircc.doImageSave = False
        self.ircc.daqRunning = False
        self.ircc.read = False
        self.ircc.voffset = 4.6 
        self.ircc.vreset = 0.5
        self.ircc.vd = 5.0
        self.ircc.ictl = 3.3
        self.ircc.continuousReadout = True
	self.ircc.savenum = 0 
	self.ircc.savesPerFile = 100 
	self.ircc.numBkgnd = 30
	self.ircc.numberCoherent = 1
        self.ircc.useBackground = False
        self.ircc.numberOfBackgroundFrames = 1000
        self.ircc.updateDaqStructure()
        common.rows = self.ircc.rows
        common.cols = self.ircc.cols
        common.roff = 0
        common.coff = 0
        common.numberOfFrames = self.ircc.numberOfFrames
        common.framesPerReset = self.ircc.framesPerReset

        # 1- Update DAQ -- warning, some values are reset e.g. gains which is annoying !
        self.ircc.updateDoImageSave()
        # 2- Update Server
	self.ft.dither_freq = 1.0
	self.ft.dither_amp = 6.74
        self.ft.opdamp = 69.1
	self.ft.dither_smoothfrac = 0.05
        self.ft.dither_delay = 0.025
	self.ft.active=[0,0,0,0,0,0]
	self.ft.offset=[0.0,0.0,0.0,0.0,0.0,0.0]
	self.ft.gain=[1,1,1,1,1,1]
	self.ft.search_state=[0,0,0,0,0,0]
        self.ft.search_step=[0.0,0.0,0.0,0.0,0.0,0.0]
 	self.ft.search_threshold=[1000.,1000.,1000.,1000.,1000.,1000.]
        self.ft.opdmapsfile = 'jdm_opds12Hz.txt'
        print " Switching to alignement mode"
        print "     dither frequency = ",  self.ft.dither_freq
	print "     dither amp = ",  self.ft.dither_amp
        print "     opd amp = ",  self.ft.opdamp
	print "     dither smoothing frac = ",  self.ft.dither_smoothfrac
#	print "     Beam params...."
#	for i in range(6):
#		print i,self.ft.active[i], self.ft.offset[i], self.ft.gain[i]

        #update gui entries to reflect the changes
        self.entry1.set_text(str(self.ircc.rows))
        self.entry2.set_text(str(self.ircc.cols))
        self.entry3.set_text(str(self.ircc.rowOffset))
        self.entry4.set_text(str(self.ircc.colOffset))
        self.entry5.set_text(str(self.ircc.nreads))
	self.entry6.set_text(str(self.ircc.numberOfFrames))
        self.entry7.set_text(str(self.ircc.framesPerReset))
        self.entry8.set_text(str(self.ft.dither_freq))
        self.entry9.set_text(str(self.ft.dither_amp))
        self.entry10.set_text(str(self.ft.opdamp))
        self.entry11.set_text(str(self.ft.dither_smoothfrac))
        self.entry12.set_text(str(self.ft.dither_delay))
        

        time.sleep(0.5) 
        self.ft.updateFTStructure()
        # 3- Config Camera -- some values are reset, but this is expected

        time.sleep(0.2)
        self.ircc.sendStruct()
        time.sleep(0.4)
        self.ircc.resetCont()
        time.sleep(0.5)
        self.ircc.setCont()
        time.sleep(0.5)
        self.ircc.setBias()
        print "Camera now in normal mode"


    def slowMode(self, widget):
        print "Entering slow mode"
        common.widefield = 0
        self.ircc.rows = 16
        self.ircc.cols = 16
        self.ircc.rowOffset = 7
        self.ircc.colOffset = 18
        self.ircc.nreads = 122
        self.ircc.numberOfCoadds = 1
        self.ircc.framesPerReset = 128
        self.ircc.expTime = 0
        self.ircc.numberOfFrames = 256
        self.ircc.byteSwapping  =False
        self.ircc.doReset = True
        self.ircc.doDeinterlace = False
        self.ircc.doImageSave = False
        self.ircc.daqRunning = False
        self.ircc.read = False
        self.ircc.voffset = 4.6 
        self.ircc.vreset = 0.5
        self.ircc.vd = 5.0
        self.ircc.ictl = 3.3
        self.ircc.continuousReadout = True
	self.ircc.savenum = 0 
	self.ircc.savesPerFile = 100 
	self.ircc.numBkgnd = 30
	self.ircc.numberCoherent = 1
        self.ircc.useBackground = False
        self.ircc.numberOfBackgroundFrames = 1000
        self.ircc.updateDaqStructure()
        common.rows = self.ircc.rows
        common.cols = self.ircc.cols
        common.roff = 0
        common.coff = 0
        common.numberOfFrames = self.ircc.numberOfFrames
        common.framesPerReset = self.ircc.framesPerReset

        # 1- Update DAQ -- warning, some values are reset e.g. gains which is annoying !
        self.ircc.updateDoImageSave()
        # 2- Update Server
	self.ft.dither_freq = .4375
	self.ft.dither_amp =  6.734
        self.ft.opdamp = 68.8
	self.ft.dither_smoothfrac = 0.05
        self.ft.dither_delay = 0.025
	self.ft.active=[0,0,0,0,0,0]
	self.ft.offset=[0.0,0.0,0.0,0.0,0.0,0.0]
	self.ft.gain=[1,1,1,1,1,1]
	self.ft.search_state=[0,0,0,0,0,0]
        self.ft.search_step=[0.0,0.0,0.0,0.0,0.0,0.0]
 	self.ft.search_threshold=[1000.,1000.,1000.,1000.,1000.,1000.]
        self.ft.opdmapsfile = 'jdm_opds6Hz.txt'
        print " Switching to alignement mode"
        print "     dither frequency = ",  self.ft.dither_freq
	print "     dither amp = ",  self.ft.dither_amp
        print "     opd amp = ",  self.ft.opdamp
	print "     dither smoothing frac = ",  self.ft.dither_smoothfrac
#	print "     Beam params...."
#	for i in range(6):
#		print i,self.ft.active[i], self.ft.offset[i], self.ft.gain[i]

        #update gui entries to reflect the changes
        self.entry1.set_text(str(self.ircc.rows))
        self.entry2.set_text(str(self.ircc.cols))
        self.entry3.set_text(str(self.ircc.rowOffset))
        self.entry4.set_text(str(self.ircc.colOffset))
        self.entry5.set_text(str(self.ircc.nreads))
	self.entry6.set_text(str(self.ircc.numberOfFrames))
        self.entry7.set_text(str(self.ircc.framesPerReset))
        self.entry8.set_text(str(self.ft.dither_freq))
        self.entry9.set_text(str(self.ft.dither_amp))
        self.entry10.set_text(str(self.ft.opdamp))
        self.entry11.set_text(str(self.ft.dither_smoothfrac))
        self.entry12.set_text(str(self.ft.dither_delay))
        
        time.sleep(0.5) 
        self.ft.updateFTStructure()
        # 3- Config Camera -- some values are reset, but this is expected

        time.sleep(0.2)
        self.ircc.sendStruct()
        time.sleep(0.4)
        self.ircc.resetCont()
        time.sleep(0.5)
        self.ircc.setCont()
        time.sleep(0.5)
        self.ircc.setBias()
        print "Camera now in slow mode"


    def killServer(self, widget):
        #stop camera update
        common.status1 = 0
        if self.tag1 != -1:
            gobject.source_remove(self.tag1)
            self.tag1 = -1
        common.status2 = 0
        if self.tag2 != -1:
            gobject.source_remove(self.tag2)
            self.tag2 = -1
        
        self.ircc.killServer()
        
    def startExposure(self, action):
        self.ircc.sendStruct()
        #added a sleep between the two commands
        #without sleep startExp fails
	#the whoel process is suser fragile..
	# use 2 seconds for large arrays, 1 for small
        time.sleep(1)
        self.ircc.startExposure()
        self.start_button1.set_sensitive(True)
        self.start_button2.set_sensitive(True)

    def openShutter(self, number):
        self.ircc.openShutter(number)

    def stopExposure(self, action):
        #FB may 2011 -- stop live updating + disable start buttons
        common.status1 = 0
        if self.tag1 != -1:
            gobject.source_remove(self.tag1)
            self.tag1 = -1
        common.status2 = 0
        if self.tag2 != -1:
            gobject.source_remove(self.tag2)
            self.tag2 = -1

        self.start_button1.set_sensitive(False)
        self.start_button2.set_sensitive(False)
        self.ircc.stopExposure()

#    def abortExposure(self, action):
#        self.ircc.abortExposure()

    def takeBackgroundFrames(self, action):
        self.ircc.getBackgroundFrame()

    def resetBackgroundFrames(self, action):
        self.ircc.resetBackgroundFrame()

    def useBackgroundFrames(self, action):
        self.ircc.useBackgroundFrame()



### FRINGE TRACKER ###


    def ftSetMode(self, widget, data):
        i = int(data[0])
	j = int(data[1])
        print "Stroke index", i, "Speed index", j

        # Common values for non-wide field modes
        common.widefield = 0         
        self.ircc.rows = 16
        self.ircc.cols = 16
        self.ircc.rowOffset = 7 
        self.ircc.colOffset = 18
        self.ircc.voffset = 4.6 
        self.ircc.vreset = 0.5
        self.ircc.vd = 5.0
        self.ircc.ictl = 3.3
        self.ircc.continuousReadout = True
	self.ircc.savenum = 0 
	self.ircc.savesPerFile = 100 
        self.ircc.numberOfCoadds = 1	
        common.rows = self.ircc.rows
        common.cols = self.ircc.cols
        common.roff = 0
        common.coff = 0
        self.ircc.expTime = 0
        self.ircc.byteSwapping  =False
        self.ircc.doReset = True
        self.ircc.doDeinterlace = False
        self.ircc.doImageSave = False
        self.ircc.daqRunning = False
        self.ircc.read = False
        self.ircc.numberCoherent = 1
        self.ircc.useBackground = False
        self.ircc.numberOfBackgroundFrames = 1000
        #Variables for PZT modulation function
        self.ft.dither_delay = 0.025 # used for Triangle, for Sawtooth this was 0.0
	self.ft.dither_smoothfrac = 0.05

        self.ft.opdmapsfile='jdm_opds30Hz.txt' #old opd map, unused at the moment
        
        #following 6 = leftover junk currently doing nothing afaik
	self.ft.active=[0,0,0,0,0,0]
	self.ft.offset=[0.0,0.0,0.0,0.0,0.0,0.0]
	self.ft.gain=[1,1,1,1,1,1]
	self.ft.search_state=[0,0,0,0,0,0]
        self.ft.search_step=[0.0,0.0,0.0,0.0,0.0,0.0]
 	self.ft.search_threshold=[1000.,1000.,1000.,1000.,1000.,1000.]


        print str(common.rate[j])+'Hz, '+str(common.stroke[i])+' mm'


        #Mode definition using JDM's IDL code values
        # i = horizontal index of button (= stroke length)
        # j = vertical index of button (= fringe rate)
        
        if (i==0) and (j==0):  #250Hz, short stroke
            self.ircc.nreads = 61
            self.ircc.framesPerReset = 54
            self.ircc.numberOfFrames = 108
            self.ircc.numBkgnd = 30     
            self.ft.dither_freq = 2.051
            self.ft.dither_amp = 2.873
            self.ft.opdamp = 29.03                     
        elif (i==1) and (j==0): #250Hz, medium stroke
            self.ircc.nreads = 61
            self.ircc.framesPerReset = 91
            self.ircc.numberOfFrames = 182
            self.ircc.numBkgnd = 30    
            self.ft.dither_freq = 1.2268
            self.ft.dither_amp = 4.803
            self.ft.opdamp = 48.913
        elif (i==2) and (j==0): # 250Hz, long stroke 70um
            self.ircc.nreads = 61
            self.ircc.framesPerReset = 128
            self.ircc.numberOfFrames = 256
            self.ircc.numBkgnd = 30     
            self.ft.dither_freq = .8750
            self.ft.dither_amp = 6.734
            self.ft.opdamp = 68.8
        elif (i==0) and (j==1): #500 Hz, short stroke
            self.ircc.nreads = 31
            self.ircc.framesPerReset = 54
            self.ircc.numberOfFrames = 108
            self.ircc.numBkgnd = 30    
            self.ft.dither_freq = 4.037
            self.ft.dither_amp = 2.873
            self.ft.opdamp = 29.03
        elif (i==1) and (j==1): #500 Hz, medium stroke
            self.ircc.nreads = 31
            self.ircc.framesPerReset = 91
            self.ircc.numberOfFrames = 182
            self.ircc.numBkgnd = 30     
            self.ft.dither_freq = 2.414
            self.ft.dither_amp = 4.803
            self.ft.opdamp = 48.91
        elif (i==2) and (j==1): #500 Hz, long stroke
            self.ircc.nreads = 31
            self.ircc.framesPerReset = 128
            self.ircc.numberOfFrames = 256
            self.ircc.numBkgnd = 30     
            self.ft.dither_freq = 1.722
            self.ft.dither_amp = 6.734
            self.ft.opdamp = 68.80
        elif (i==0) and (j==2): #1000 Hz, short stroke
            self.ircc.nreads = 15
            self.ircc.framesPerReset = 54
            self.ircc.numberOfFrames = 108
            self.ircc.numBkgnd = 30    
            self.ft.dither_freq = 8.342
            self.ft.dither_amp = 2.873
            self.ft.opdamp = 29.03
        elif (i==1) and (j==2): #1000 Hz, medium stroke
            self.ircc.nreads = 15
            self.ircc.framesPerReset = 91
            self.ircc.numberOfFrames = 182
            self.ircc.numBkgnd = 30     
            self.ft.dither_freq = 4.989
            self.ft.dither_amp = 4.803
            self.ft.opdamp = 48.91
        elif (i==2) and (j==2):  #1000 Hz, long stroke
            self.ircc.nreads = 15
            self.ircc.framesPerReset = 128
            self.ircc.numberOfFrames = 256
            self.ircc.numBkgnd = 30     
            self.ft.dither_freq = 3.559
            self.ft.dither_amp = 6.734
            self.ft.opdamp = 68.8                      

        #common part again        
        print "     dither frequency = ",  self.ft.dither_freq
	print "     dither amp = ",  self.ft.dither_amp
        print "     opd amp = ",  self.ft.opdamp
	print "     dither smoothing frac = ",  self.ft.dither_smoothfrac
        common.numberOfFrames = self.ircc.numberOfFrames
        common.framesPerReset = self.ircc.framesPerReset

        # 1- Update DAQ -- warning, some values are reset
        self.ircc.updateDaqStructure()
        time.sleep(0.5) 
        self.ircc.updateDoImageSave()

        # 2- Update FT structure
        time.sleep(0.5) 
        self.ft.updateFTStructure()

        # 3- Config Camera -- some values are reset, but this is expected
        time.sleep(0.2)
        self.ircc.sendStruct()
        time.sleep(0.4)
        self.ircc.resetCont()
        time.sleep(0.5)
        self.ircc.setCont()
        time.sleep(0.5)
        self.ircc.setBias()

        # 4- Update gui entries to reflect the changes
        self.entry1.set_text(str(self.ircc.rows))
        self.entry2.set_text(str(self.ircc.cols))
        self.entry3.set_text(str(self.ircc.rowOffset))
        self.entry4.set_text(str(self.ircc.colOffset))
        self.entry5.set_text(str(self.ircc.nreads))
	self.entry6.set_text(str(self.ircc.numberOfFrames))
        self.entry7.set_text(str(self.ircc.framesPerReset))
        self.entry8.set_text(str(self.ft.dither_freq))
        self.entry9.set_text(str(self.ft.dither_amp))
        self.entry10.set_text(str(self.ft.opdamp))
        self.entry11.set_text(str(self.ft.dither_smoothfrac))
        self.entry12.set_text(str(self.ft.dither_delay))






    def ftTrackingUpdate(self, widget, data):
	type=str(data[0])
	beamnum=int(data[1])

	if type == 'active':
	   if widget.get_active():
	   	print "Tracking beam: ", beamnum
	   	self.ft.active[beamnum]=1
	   else:
	   	print "NOT Tracking beam: ", beamnum
	   	self.ft.active[beamnum]=0

	elif type == 'offset':
	   self.ft.offset[beamnum]=float(widget.get_text())

	elif type == 'gain':
	   self.ft.gain[beamnum]=float(widget.get_text())

 	elif type == 'searchState':
	   self.ft.search_state[beamnum]=float(widget.get_text())

	elif type == 'step':
	   self.ft.search_step[beamnum]=float(widget.get_text())

	elif type == 'threshold':
	   self.ft.search_threshold[beamnum]=float(widget.get_text())

        
    def ftModSetEntry(self, widget, data):
        entry = widget.get_text()
        if data == 'freq':
            self.ft.dither_freq = float(entry)
            print "new dither frequency = ",  self.ft.dither_freq

        elif data == 'amp':
            self.ft.dither_amp = float(entry)
            print "new dither amp = ",  self.ft.dither_amp

        elif data == 'opdamp':
            self.ft.opdamp = float(entry)
            print "new opdamp = ",  self.ft.opdamp


        elif data == 'smoothfrac':
            self.ft.dither_smoothfrac = float(entry)
            print "new dither smoothing frac = ",  self.ft.dither_smoothfrac

        elif data == 'ditherdelay':
            self.ft.dither_delay = float(entry)
            print "new dither delay frac = ",  self.ft.dither_delay


	else:
	   print "unknown request for ftModSetEntry"
        

    def ftUpdateServer(self,widget,data):

	print "Sending the following modulation params to server...."
	print "     dither frequency = ",  self.ft.dither_freq
	print "     dither amp = ",  self.ft.dither_amp
        print "     opd amp = ",  self.ft.opdamp
	print "     dither smoothing frac = ",  self.ft.dither_smoothfrac
	
#	print "Sending the following beam params to server...."
#	for i in range(6):
#		print i,self.ft.active[i], self.ft.offset[i], self.ft.gain[i]
	
        self.ft.updateFTStructure()

        
def main():
    ApplicationMainWindow()
    gtk.main()

if __name__ == '__main__':
    main()
