#!/usr/bin/local/python2.4

from random import randint  #random integer function, useful for tests
from time import sleep
import signal #for interupting mapping sequence 
from matplotlib import rcParams
import numpy as np
import pylab as pl

import common
import IRCamSocket

exit_index = 0 #declare signal index globally

def z_prism(stage, z_length, z_spacing, side, spacing, dat, ij_range = [0,256,0,256], seconds = .5):
	
	if((z_length * 10)%1 != 0) or ( (z_spacing*10)%1 != 0):
		print 'The range and spacing of z must be divisible by 100 nm (z resolution)'
	
	zn = (z_length/z_spacing) + 1 # zn is currently float
	
	z = [None]*int(zn)
	z0 = common.lum.absolute_position(stage,1)
	z_steps = long(z_spacing*10)
	
	if ( zn % 2 == 0) or (zn % 2 == 1): #n is even or odd
		for i in range(int(zn)):
			z[i] = long(z0 - z_steps * (zn-1)/2.0 + z_steps * i)
	else:
		print 'The range divided by the spacing must be an integer value'
		return [0],[0],[0]
	
	zn = int(zn)
	n = int(spacing/side)+1
	x = [None]*int(n)
	y = [None]*int(n)
	
	#intensities = [[intensity],[intensity],[intensity],...]
	intensities = [[[None]*2 for i in range(n)] for j in range(zn)]
	max_info = [[None]*3 for i in range(zn)]
	max_intensity_z = 0
	z_max = 0
	
	for i in range(zn):
		common.lum.move_absolute(stage,1,z[i])
		intensities[i],x,y = xy_square(stage, side, spacing, dat, ij_range, seconds)
		max_info[i] = get_max(intensities[i],x,y)
		if max_info[i][2] > max_intensity_z:
			max_intensity_z = max_info[i][2]
			max_index = i
		
	common.lum.move_absolute(stage,1,z0)
	print 'Maximum intensity at z = ',z[max_index]
	print '                     x = ',max_info[max_index][0]
	print '                     y = ',max_info[max_index][1]
	print 'intensity = ',max_info[max_index][2]
	
	return [intensities, x, y, z] 
	
def xy_total(stage, spacing, dat, range_i = [0,256], range_j = [0,256], seconds = .5):
	if ((spacing * 1000) % 4 !=0):
		print 'The spacing must be divisible by 4 nm (xy resolution)'
		return 0
	
	common.lum.move_absolute(stage, 2, 62500)
	common.lum.move_absolute(stage, 3, 62500)
	return xy_square(stage, 500, spacing, dat, range_i, range_j, seconds)
	 

def xy_square(stage, side, spacing, dat, ij_range= [0,256,0,256], disp_img = 1):
	
	#take dummy exposure to kill buffer
	#dat.Exposure()
	#sleep(0.001*dat.exptime+0.040)

	rcParams['interactive'] = True

	if ((spacing * 1000) % 4 != 0) or ( (side * 1000) % 4 != 0):
		print 'The range and spacing must be divisible by 4 nm (xy resolution)'
		return 0,0,0

	side = float(side)
	spacing = float(spacing)
	
	n = ( side / spacing ) + 1 # n is currently a float
	
	x = []
	y = []
	x0 = common.lum.absolute_position(stage,2)
	y0 = common.lum.absolute_position(stage,3)
	steps = long(spacing * 250)
	
	if ( n % 2 == 0) or (n % 2 == 1): #n is even or odd
		for i in range(int(n)):
			x_temp = long(x0 - steps * (n-1)/2.0 + steps * i)
			y_temp = long(y0 - steps * (n-1)/2.0 + steps * i)
			if (x_temp > 0) and (x_temp < 128000):
				x.append(x_temp)
			if (y_temp > 0) and (y_temp < 128000):
				y.append(y_temp)
	else:
		print 'The range divided by the spacing must be an integer value'
		return (0,0,0,1)
	
	del(x_temp)
	del(y_temp)

	#switch n to an integer for the rest of the program
	n = int(n)
			
	intensity = np.array(np.zeros((len(y),len(x))))
	x = np.array(x)
	y = np.array(y)
	
	#initiate window for plotting
	if (disp_img):
		#use figure 3 active figure
		figure(3)
		clf()
		#set axes
		img_extent = x[0]-0.5*steps, x[-1] + 0.5*steps, y[0] - 0.5*steps, y[-1] + 0.5*steps
		title('Image of Intensity vs. Position, Current Map')
		xlabel('X in steps')
		ylabel('Y in steps')
		ylabel('Y in steps') #advance the frame
	else:
		pass
		
	#set the signal handler
	signal.signal(signal.SIGTSTP,xy_interupt)
	print 'CTRL-Z to stop mapping after current row'
	global exit_index
	exit_index = 0 #becomes 1 (true) when signal sent
	
	#nested for loops activate mappingxplt
	#first for loop gives y coordinate
	#second for loop gives x coordinate	
	
	for i in range(len(y)):
		if exit_index == 1:
			intensity = intensity[0:i][:]
			y = y[0:i]
			print 'intensity[][] and y[] have been reduced'
			break
		else:
			pass
		
		common.lum.move_absolute( stage , 3, y[i] )
		for j in range(len(x)):
			#if statement decides which direction to map across x
			if i % 2 == 0:
				common.lum.move_absolute( stage, 2 , x[j] )
				#sleep(seconds)
				#Read intensity into 2-D list 
				intensity[i][j] = get_intensity(dat, ij_range)
			else:
				common.lum.move_absolute( stage, 2 , x[-1 - j] );
				#sleep(seconds)
				#Read intensity into 2-D list 
				intensity[i][-1 - j] = get_intensity(dat, ij_range)
		if disp_img == 1:
			#clf() #clear current window
			#plot image with colors scaled to only that which has been mapped with vmin & vmax
			#plot it twice to advance the frame
			pl.imshow(intensity,origin = 'lower',cmap =cm.hot,vmin = intensity[0:i+1].min(), vmax = intensity[0:i+1].max(), extent = img_extent,interpolation='nearest', aspect='auto')
			ylabel('Y in steps')
		else:
			pass
	
				
	#reset signal to default
	signal.signal(signal.SIGTSTP, signal.SIG_DFL)
			
	#move back to original location	
	common.lum.move_absolute(stage,2,x0)
	common.lum.move_absolute(stage,3,y0)

	#close()
	
	#rcParams['interactive'] = False
		
	return intensity,np.array(x),np.array(y),False



def xy_interupt(signum,frame):
	print 'Interupt signal sent, mapping will end after this row...'
	global exit_index
	exit_index = 1
	#sleep(1)
	return 

def get_max(intensity,x,y):
	
	maximum = 0
	x_max_index = 0
	y_max_index = 0
	
	ny = len(y)
	nx = len(x)
	
	for i in range(ny):
		for j in range(nx):
			if intensity[i][j] > maximum:
				maximum = intensity[i][j]
				x_max_index = j
				y_max_index = i
	
	#move_absolute(stage,2,x[x_max_index])
	#move_absolute(stage,3,y[y_max_index])
	
	#del(dat)
		
	return [x[x_max_index],y[y_max_index],intensity[y_max_index][x_max_index]]
	
def get_intensity_random():
	return randint(0,100)

# get_intensity takes the range of pixels to be sampled and returns the total intensity accross those pixels
def get_intensity(dat, ij_range = [0,256,0,256]):
	irs = IRCamSocket.socketIRCam()
	dat.Exposure()
	#sleep(0.5)
	
	#to get total intensity, sum all pixels (all elements of dat.img)
	
	total_intensity = dat.img[ij_range[0]:ij_range[1], ij_range[2]:ij_range[3]].sum()
## 	for i in range(ij_range[0], ij_range[1]):
## 		for j in range(ij_range[2], ij_range[3]):
## 			total_intensity += dat.img[i][j]
	return total_intensity

def xy_surf(intensity,x,y):
	
	start_x = x[0]
	start_y = y[0]
	
	x_microns = [None] * len(x)
	y_microns = [None] * len(y)
	
	for i in range(len(x)):
		x_microns[i] = (x[i] - start_x) / 250.
	for i in range(len(y)):
		y_microns[i] = (y[i] - start_y) / 250.	
		
	gplt.surf(x_microns,y_microns,intensity)
	gplt.title('Intensity on camera vs. Luminos stage XY position')
	gplt.xtitle('X position in microns')
	gplt.ytitle('Y position in microns')
	gplt.ztitle('Intensity')
	gplt.hidden("remove")
	return


	



