import gtk
import picoLowLevel
import picoData
import numpy as np
import arrows

def scale_set_default_values(scale):
    scale.set_update_policy(gtk.UPDATE_CONTINUOUS)
    scale.set_digits(1)
    scale.set_value_pos(gtk.POS_TOP)
    scale.set_draw_value(True)

def create_arrow_button(arrow_type, shadow_type):
    button = gtk.Button();
    arrow = gtk.Arrow(arrow_type, shadow_type);
    button.add(arrow)
    button.show()
    arrow.show()
    return button

class pico(gtk.Window, picoLowLevel.PicoControl):
    def __init__(self, picoName = 'cyl lens', parent=None):
    	#init pico control
	picoIP = picoData.getFieldValue(picoData.picoId, 'IP', 'picoId', picoName)
        #index, = picoData.picoId.field('picoId').match(picoName)
        index = np.where(picoData.picoId.field('picoId') == picoName)[0]
        nAxis = len(index)
	self.driver = picoData.picoId.field('driver')[index]
	self.motor = picoData.picoId.field('motor')[index]
	self.steps = np.array([500]*nAxis)
	self.Relative = np.zeros(nAxis)
	print "picoName = ", picoName
        print "available axis: ", picoData.picoId.field('axis')[index]
	print "picoIP = ", picoIP

        if picoData.dbg == 0:
            #intialisation of the telnet connection with the picoMotor
            picoLowLevel.PicoControl.__init__(self, picoIP)
        
            print "self.tn.host = ", self.tn.host
            for i in range(nAxis):
                self.SetVelocity(self.driver[i], self.motor[i], 2000)

	# Create the toplevel window
        gtk.Window.__init__(self)
        try:
            self.set_screen(parent.get_screen())
        except AttributeError:
            self.connect('destroy', lambda *w: gtk.main_quit())

	self.set_title(picoName)
# 	self.set_default_size(300, 200) 
        
        table = gtk.Table(7, 4, False)
        self.add(table)
        self.entry = []
        self.entrySteps = []
        for i in range(nAxis):
            self.entry.append(gtk.Entry())
            self.entrySteps.append(gtk.Entry())
            
            frame = gtk.Frame(picoData.picoId.field('axis')[index[i]])
            self.entry[i].set_size_request(60, 20)
            self.entry[i].set_text(str(self.Relative[i]))
            self.entry[i].set_editable(False)
            frame.add(self.entry[i])
            table.attach(frame,
                         # X direction #          # Y direction
                         4, 5,                      i+1, i+2,
                         0,     0,
                         0,                         0);
 
            
            frame = gtk.Frame("STEP SIZE")            
            self.entrySteps[i].set_size_request(60, 20)
            self.entrySteps[i].set_text(str(self.steps[i]))
            self.entrySteps[i].connect("activate", self.enter_callback, i)
            frame.add(self.entrySteps[i])
            table.attach(frame,
                         # X direction #          # Y direction
                         5, 6,                      i+1, i+2,
                         0,     0,
                         0,                         0);
            buttons, x_pos, y_pos = self.create_buttons(picoData.picoId.field('axis')[index[i]])
            if picoData.dbg != 0:
                self.move_cb = self.move_cb_dbg
            buttons[0].connect("clicked", self.move_cb, "left", i)
            table.attach(buttons[0],
                         # X direction #          # Y direction
                         x_pos[0], x_pos[1],        y_pos[0], y_pos[1],
                         gtk.EXPAND | gtk.FILL,     gtk.EXPAND | gtk.FILL,
                         0,                         0);

            buttons[1].connect("clicked", self.move_cb, "right", i)
            table.attach(buttons[1],
                         # X direction #          # Y direction
                         x_pos[2], x_pos[3],        y_pos[2], y_pos[3],
                         gtk.EXPAND | gtk.FILL,     gtk.EXPAND | gtk.FILL,
                         0,                         0);
  
        menubar = gtk.MenuBar()
        menuitem = gtk.MenuItem('ZERO')
        menuitem.set_submenu(self.create_menu(picoName, 1, nAxis))
##             menubar.add(menuitem)

##             button = gtk.Button('ZERO')

##             button.connect("clicked", self.move_cb, "zero", i)
        menubar.add(menuitem)
        table.attach(menubar,
                     # X direction #          # Y direction
                     1, 2,                      2, 3,
                     gtk.EXPAND | gtk.FILL,     gtk.EXPAND | gtk.FILL,
                     0,                         0);

        
        separator = gtk.VSeparator()
        separator.set_size_request(20, 100)
        table.attach(separator,
                     # X direction #          # Y direction
                     3, 4,                      0, 4,
                     gtk.EXPAND | gtk.FILL,     0,
                     0,                         0);
  
        
        separator = gtk.HSeparator()
        separator.set_size_request(200, 20)
        table.attach(separator,
                     # X direction #          # Y direction
                     0, 7,                      4, 5,
                     gtk.EXPAND | gtk.FILL,     0,
                     0,                         0);
  
        
	button = gtk.Button('CLOSE')
        button.connect('clicked', lambda button, w=self: w.destroy())
        table.attach(button,
            # X direction #          # Y direction
            5, 7,                      5, 6,
            gtk.EXPAND | gtk.FILL,     0,
            0,                         0);
        
	button.set_flags(gtk.CAN_DEFAULT)
        button.grab_default()	
	
        self.show_all()

    def create_buttons(self, axis):
        buttons = []
        x_pos = [0,0,0,0]
        y_pos = [0,0,0,0]
        if axis == 'X':
            buttons.append(gtk.Button())
            image = arrows.makeIcon('left')
            buttons[0].add(image)
            buttons.append(gtk.Button())
            image = arrows.makeIcon('right')
            buttons[1].add(image)
            x_pos = [0, 1, 2, 3]
            y_pos = [2, 3, 2, 3]
            
        if axis == 'Y':
            buttons.append(gtk.Button())
            image = arrows.makeIcon('down')
            buttons[0].add(image)
            buttons.append(gtk.Button())
            image = arrows.makeIcon('up')
            buttons[1].add(image)
            x_pos = [1, 2, 1, 2]
            y_pos = [3, 4, 1, 2]
            
        if axis == 'Z':
            buttons.append(gtk.Button())
            image = arrows.makeIcon('downleft')
            buttons[0].add(image)
            buttons.append(gtk.Button())
            image = arrows.makeIcon('upright')
            buttons[1].add(image)
            x_pos = [0, 1, 2, 3]
            y_pos = [3, 4, 1, 2]
            
        if axis == 'TILT':
            buttons.append(gtk.Button())
            image = arrows.makeIcon('downright')
            buttons[0].add(image)
            buttons.append(gtk.Button())
            image = arrows.makeIcon('upleft')
            buttons[1].add(image)
            x_pos = [2, 3, 0, 1]
            y_pos = [3, 4, 1, 2]
            
        return buttons, x_pos, y_pos

    def create_menu(self, picoName, depth, length):
        index, = picoData.picoId.field('picoId').match(picoName)
        if depth < 1:
            return None

        menu = gtk.Menu()
        group= None

        if length > 1:
            menuitem = gtk.RadioMenuItem(group, 'ALL')
            group = menuitem
            menu.add(menuitem)
            menuitem.show()
            if depth > 1:
                submenu = self.create_menu(picoName, depth -1, 1)
                menuitem.set_submenu(submenu)
            menuitem.connect("activate", self.move_cb, "zero",'ALL')
        for i in range(length):
            menuitem = gtk.RadioMenuItem(group, picoData.picoId.field('axis')[index[i]])
            group = menuitem
            menu.add(menuitem)
            menuitem.show()
            if depth > 1:
                submenu = self.create_menu(picoName, depth -1, 1)
                menuitem.set_submenu(submenu)
            menuitem.connect("activate", self.move_cb, "zero", i)
        return menu            
	
    def move_cb_dbg(self, widget, status, ndx):
	if (status == "right"):
            self.Relative[ndx] += self.steps[ndx]
            self.entry[ndx].set_text(str(self.Relative[ndx]))
	if (status == "left"):
            self.Relative[ndx] -= self.steps[ndx]
            self.entry[ndx].set_text(str(self.Relative[ndx]))
	if (status == "zero" and widget.get_active() ):
            if ndx == 'ALL':
                for i in range(len(self.Relative)):
                    self.Relative[i] = 0
                    self.entry[i].set_text(str(self.Relative[i]))
            else:
                self.Relative[ndx] = 0
                self.entry[ndx].set_text(str(self.Relative[ndx]))
                
		
    def move_cb(self, widget, status, ndx):
    	print "status = ", status
	if (status == "right"):
            self.MoveRelative(self.driver[ndx],self.motor[ndx],self.steps[ndx])
            self.Relative[ndx] += self.steps[ndx]
            self.entry[ndx].set_text(str(self.Relative[ndx]))
	if (status == "left"):
            self.MoveRelative(self.driver[ndx],self.motor[ndx],-self.steps[ndx])
            self.Relative[ndx] -= self.steps[ndx]
            self.entry[ndx].set_text(str(self.Relative[ndx]))
	if (status == "zero"):
            for i in range(len(self.Relative)):
		self.MoveRelative(self.driver[i],self.motor[i],-self.Relative[i])
                self.Relative[i] = 0
                self.entry[i].set_text(str(self.Relative[i]))
		
    def enter_callback(self, widget, ndx):
	entry_text = self.entrySteps[ndx].get_text()
	self.steps[ndx] = int(entry_text)
	print "New step size: %d\n" % self.steps[ndx]


    
    
def main(pico):	
    functionToCall = picoData.tokenDict[pico]
    functionToCall()

    gtk.main()
	
if __name__ == '__main__':
    main()
