#include <math.h>
#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "misc.h"

float sinc(float x)
{
	if ( fabs(x) < 1e-8 )
		return 1.;
	else
		return sin(x) / x;
}

float max(float* vector, int n)
{
  register int i;
  float temp = -1e99;
  for(i=0; i< n ; i++)
    {
      if(vector[i]>temp)
	temp = vector[i];
    }
  return temp;
}

// Solve 3 simultaneous equations
//
// y1 = a11 x1 + a12 x2 + a13 x3
// y2 = a21 x1 + a22 x2 + a23 x3
// y3 = a31 x1 + a32 x2 + a33 x3
//
// y1 = a1 x1 + b1 x2 + c1 x3
// y2 = a2 x1 + b2 x2 + c2 x3
// y3 = a3 x1 + b3 x2 + c3 x3
void dewarp(float *bias_matrix, int indx, float *biased_n, float *biased_x, float *biased_y, double *debiased_n, double *debiased_x, double *debiased_y)
{
	double a11, a21, a31, a12, a22, a32, a13, a23, a33, y1, y2, y3; 
	double invdet, det, m11, m12, m13, m21, m22, m23, m31, m32, m33;
	a11 = bias_matrix[9 * indx];
	a12 = bias_matrix[9 * indx + 1];
	a13 = bias_matrix[9 * indx + 2];
	a21 = bias_matrix[9 * indx + 3];
	a22 = bias_matrix[9 * indx + 4];
	a23 = bias_matrix[9 * indx + 5];
	a31 = bias_matrix[9 * indx + 6];
	a32 = bias_matrix[9 * indx + 7];
	a33 = bias_matrix[9 * indx + 8];
	y1 = biased_n[indx];
	y2 = biased_x[indx];
	y3 = biased_y[indx];


	det = a11 * (a33 * a22 - a32 * a23) - a21 * (a33 * a12 - a32 * a13) + a31 * (a23 * a12 - a22 * a13);


	if( (det > 0.)||(det < 0.) )
	  invdet = 1./ det;
	else 
	  invdet = 0.;

	m11 = a33 * a22 - a32 * a23;
	m12 = -(a33 * a12 - a32 * a13);
	m13 = a23 * a12 - a22 * a13;
	m21 = -(a33 * a21 - a31 * a23);
	m22 = a33 * a11 - a31 * a13;
	m23 = -(a23 * a11 - a21 * a13);
	m31 = a32 * a21 - a31 * a22;
	m32 = -(a32 * a11 - a31 * a12);
	m33 = a22 * a11 - a21 * a12;

	*debiased_n = (m11 * y1 + m12 * y2 + m13 * y3) * invdet;
	*debiased_x = (m21 * y1 + m22 * y2 + m23 * y3) * invdet;
	*debiased_y = (m31 * y1 + m32 * y2 + m33 * y3) * invdet;

	//	printf("Inverse\n");
	//	printf("\n %f \t %f \t %f \n", m11, m12, m13);
	//	printf("\n %f \t %f \t %f \n", m21, m22, m23);
	//	printf("\n %f \t %f \t %f \n", m31, m32, m33);
}

// champ-bs4  (bs=bootstrap)
// pass this program the weights and measured OPDS and this program
// will return an optimized set of delay line shifts (always assume delay0=0)
// same as MIRC.
//
// the assumption is that
// opd0 = delay0-delay_(n-1),
// opd1 = delay1-delay0
// opd2 = delay2-delay1
// etc.
// OPDS1 are the self-consistent best fit.
// one will need to write a 3, 5 , 6 telescope version of this for future work...
//
void champ_bs4(double *w, double *OPD0, double *OPD1, double *DL0)
{
	double wf[4];
	double a, b, c, ab, bc, y[4];
	double m[3][3];
	double det3, invdet3;
	double OPD[4];
	int i;

	// OPD = [ OPD30, OPD01, OPD12, OPD23] = [dl0-dl3,dl1-dl0,dl2-dl1,dl3-dl2]
	// DL0 = [dl0,dl1,dl2,dl3]

	// zero out the OPDS for weak fringes.
	// could be done elsewhere..
	for ( i = 0; i < 4 ; i++ )
	{
		if ( w[i] < 1.0 )
			OPD[i] = 0.0;
		else
			OPD[i] = OPD0[i];

		wf[i] = (double) MAX(w[i], .01);
	}

	a = wf[1] + wf[2];
	b = wf[2] + wf[3];
	c = wf[3] + wf[0];

	ab = - wf[2];
	bc = - wf[3];

	y[1] = wf[1] * OPD[1] - wf[2] * OPD[2];
	y[2] = wf[2] * OPD[2] - wf[3] * OPD[3];
	y[3] = wf[3] * OPD[3] - wf[0] * OPD[0];

	det3 = a * (c * b - bc * bc) - ab * (c * ab);

	if(det3 != 0)
	  invdet3 = 1./det3;
	else 
	  {
	    invdet3 = 0.;
	  //	    printf("Warning -- bootstrap singularity \n");
	  }

	m[0][0] = c * b - bc * bc;
	m[0][1] = -(c * ab);
	m[0][2] = bc * ab;
	m[1][0] = -(c * ab);
	m[1][1] = c * a;
	m[1][2] = -(bc * a);
	m[2][0] = bc * ab;
	m[2][1] = -(bc * a);
	m[2][2] = b * a - ab * ab;

	DL0[0] = 0.0; // by definition
	DL0[1] = (m[0][0] * y[1] + m[0][1] * y[2] + m[0][2] * y[3]) * invdet3;
	DL0[2] = (m[1][0] * y[1] + m[1][1] * y[2] + m[1][2] * y[3]) * invdet3;
	DL0[3] = (m[2][0] * y[1] + m[2][1] * y[2] + m[2][2] * y[3]) * invdet3;

	OPD1[0] = DL0[0] - DL0[3];
	OPD1[1] = DL0[1] - DL0[0];
	OPD1[2] = DL0[2] - DL0[1];
	OPD1[3] = DL0[3] - DL0[2];
}


void champ_bs6(double *w, double *OPD0, double *OPD1, double *DL0) //FB: equations derived with maple, see CHAMP:Documents on wiki
{
	double w0, w1, w2, w3, w4, w5;
	double y[6];
	double m[5][5];
	double det, invdet;
	double OPD[6];
	register int i, j;

	for ( i = 0; i < 6 ; i++ )
	{
		if ( w[i] < 1.0 )
			OPD[i] = 0.0;
		else
			OPD[i] = OPD0[i];

	}

	w0 = (double) MAX(w[0], .01);
	w1 = (double) MAX(w[1], .01);
	w2 = (double) MAX(w[2], .01);
        w3 = (double) MAX(w[3], .01);
	w4 = (double) MAX(w[4], .01);
	w5 = (double) MAX(w[5], .01);

	y[0] = 0.;
	y[1] = w1 * OPD[1] - w2 * OPD[2];
	y[2] = w2 * OPD[2] - w3 * OPD[3];
	y[3] = w3 * OPD[3] - w4 * OPD[4];
	y[4] = w4 * OPD[4] - w5 * OPD[5];
	y[5] = w5 * OPD[5] - w0 * OPD[0];

	det =  w2 * w1 * w4 * w3 * w5 + w2 * w3 * w4 * w1 * w0 + w2 * w5 * w4 * w0 * w3 + w2 * w3 * w1 * w5 * w0 + w2 * w1 * w5 * w4 * w0 + w1 * w5 * w4 * w0 * w3;

	if(det != 0.)
          invdet = 1./det;
        else
	  {
	    invdet = 0.;
	    //	    printf("Warning -- bootstrap singularity \n");
	  }

	m[0][0] = w2 * w4 * w3 * w5 + w2 * w3 * w4 * w0 + w2 * w3 * w5 * w0 + w2 * w5 * w4 * w0 + w5 * w4 * w0 * w3;
	m[0][1] = (w4 * w3 * w5 + w3 * w4 * w0 + w3 * w5 * w0 + w5 * w4 * w0) * w2;
	m[0][2] =  w2 * w3* (w4 * w5 + w4 * w0 + w5 * w0);
	m[0][3] =  w3 * w2 * w4* (w5 + w0);
	m[0][4] =  w2 * w4 * w3 * w5;
	m[1][0] = (w4 * w3 * w5 + w3 * w4 * w0 + w3 * w5 * w0 + w5 * w4 * w0) * w2;
	m[1][1] =  w2 * w4 * w3 * w5 + w2 * w3 * w4 * w0 + w2 * w3 * w5 * w0 + w2 * w5 * w4 * w0 + w1 * w4 * w3 * w5 + w3 * w4 * w1 * w0 + w3 * w1 * w5 * w0 + w1 * w5 * w4 * w0;
	m[1][2] =  w3* (w2 * w4 * w5 + w2 * w4 * w0 + w2 * w5 * w0 + w1 * w4 * w5 + w4 * w1 * w0 + w1 * w5 * w0);
	m[1][3] =  w3 * w4* (w2 * w5 + w2 * w0 + w1 * w5 + w1 * w0);
	m[1][4] =  w4 * w3 * w5* (w1 + w2);
	m[2][0] =  w2 * w3* (w4 * w5 + w4 * w0 + w5 * w0);
	m[2][1] =  w3* (w2 * w4 * w5 + w2 * w4 * w0 + w2 * w5 * w0 + w1 * w4 * w5 + w4 * w1 * w0 + w1 * w5 * w0);
	m[2][2] =  w2 * w4 * w3 * w5 + w2 * w3 * w4 * w0 + w2 * w3 * w5 * w0 + w2 * w1 * w4 * w5 + w2 * w4 * w1 * w0 + w2 * w1 * w5 * w0 + w1 * w4 * w3 * w5 + w3 * w4 * w1 * w0 + w3 * w1 * w5 * w0;
	m[2][3] =  w4* (w2 * w3 * w5 + w2 * w3 * w0 + w2 * w1 * w5 + w1 * w2 * w0 + w3 * w1 * w5 + w1 * w3 * w0);
	m[2][4] =  w4 * w5* (w1 * w2 + w1 * w3 + w2 * w3);
	m[3][0] =  w3 * w2 * w4* (w5 + w0);
	m[3][1] =  w3 * w4* (w2 * w5 + w2 * w0 + w1 * w5 + w1 * w0);
	m[3][2] =  w4* (w2 * w3 * w5 + w2 * w3 * w0 + w2 * w1 * w5 + w1 * w2 * w0 + w3 * w1 * w5 + w1 * w3 * w0);
	m[3][3] =  w2 * w4 * w3 * w5 + w2 * w3 * w4 * w0 + w2 * w3 * w1 * w5 + w2 * w1 * w3 * w0 + w2 * w1 * w4 * w5 + w2 * w4 * w1 * w0 + w1 * w4 * w3 * w5 + w3 * w4 * w1 * w0 ;
	m[3][4] =  w5* (w1 * w2 * w3 + w2 * w1 * w4 + w3 * w2 * w4 + w1 * w3 * w4);
	m[4][0] =  w2 * w4 * w3 * w5;
	m[4][1] =  w4 * w3 * w5* (w1 + w2) ;
	m[4][2] =  w4 * w5* (w1 * w2 + w1 * w3 + w2 * w3);
	m[4][3] =  w5* (w1 * w2 * w3 + w2 * w1 * w4 + w3 * w2 * w4 + w1 * w3 * w4);
	m[4][4] =  w2 * w4 * w3 * w5 + w1 * w3 * w2 * w4 + w2 * w3 * w1 * w5 + w2 * w1 * w4 * w5 + w1 * w4 * w3 * w5;


	for(i=0;i<6;i++)
          DL0[i] = 0.;

	for(i=1;i<6;i++)
	  for(j=1;j<6;j++)
	    DL0[i] += m[i-1][j-1] * y[j]; 

	for(i=1;i<6;i++)
	  DL0[i] *= invdet;

	OPD1[0] = DL0[0] - DL0[5];
	OPD1[1] = DL0[1] - DL0[0];
	OPD1[2] = DL0[2] - DL0[1];
	OPD1[3] = DL0[3] - DL0[2];
	OPD1[4] = DL0[4] - DL0[3];
        OPD1[5] = DL0[5] - DL0[4];
}




