/*
 * astropci_usr.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <getopt.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <sys/io.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <math.h>
#include <complex.h>  // no conflict with fftw3.h if placed before
#include <sched.h>
#include <native/task.h>
#include <native/sem.h>
#include <native/pipe.h>
#include <native/timer.h>
#include <nucleus/registry.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "misc.h"
#include "DSPCommand.h"
#include "common.h"
#include "astropci_usr.h"
#include "loadDspFile.h"
#include "chara_messages.h"
//#define SERVER
#include "cliserv.h"
#include "ople_mirc.h"

#include "time.h"
#include <NIDAQmx.h> // I think this needs to be ahead of astropci.usr because of the datatype float64

//#include <rtdk.h> // rt_printf function from Xenomai newer version

#define VERBOSE 0
#define TALK_TO_DELAYLINES TRUE
#define DAQ_WAVEFORM 1//0: sawtooth, 1:triangle

//#define FAKE_FRINGES 1

int pci_fd = 0;
int pipeMinor = 0;
RT_TASK sem_task_ptr;
RT_TASK target_task_ptr;
RT_PIPE pipe_desc;
ssize_t pipe_err = 0;

// CHARA shutters 
//#include "cliserv.h"
//#include "chara_messages.h"
#include "shutter.h"
int shutter_server = -1;
struct sshutter_conf shutter_conf;
int shutter_status[6]; // for spooler comm, tries to follow the shutters (not using shutter_conf)
int initialize_shutters(void);
int setup_messages(void);
int message_shutter_status(int server, struct smessage *mess);
int message_shutter_conf(int server, struct smessage *mess);
int send_shutter_open(unsigned char shutter);
int send_shutter_close(unsigned char shutter);
int send_shutter_status(unsigned char shutter);
int shutter_id(char *name);
int message_noop(int server, struct smessage *mess);
int message_message(int server, struct smessage *mess);
int message_ping(int server, struct smessage *mess);
int message_server_stopping(int server, struct smessage *mess);
int process_message_socket(void);

//NIDAQ global
TaskHandle NIDAQ_mod_taskHandle = 0, NIDAQ_clock_taskHandle = 0, NIDAQ_trigger_taskHandle = 0;

unsigned short *mem_fd, *dataBuffer;
daq_struct daq;
daqOut_struct daqOut;

// All the following needs cleaning -- lots of unused variables

exchange_struct exc = { 0, /* pixelCount */
		0, /* byte_count_short */
		0, /* byte_count_float */
		FALSE, /* daqRunning */
		FALSE, /* endProg */
		FALSE, /* resetBkgnd */
		FALSE, /* takeBkgnd */
		FALSE, /* useBkgnd */
		C_IDLE, /* cmd      */
		35, /* bkgndNoiseCol */
		{ 15, 20, 10, 5, 25, 29 }, /* cpeak */
		{ 14, 19, 9, 5, 24, 28 }, /* cpeak0 */
		{ 16, 21, 11, 6, 26, 30 }, /* cpeak1 */
		{ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /* Weight */
		{ 1., 1., 1., 1., 1., 1. }, /* gdtThreshold */
			{ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /*  gdtOpd */
			{ 0.0, 0.0, 0.0 }, /* gdtOptOpd */
			{ -1, -1, +1, -1, +1, +1 }, /* OPDSigns */ // unused
			{ W1, E1, E2, S1, S2, W2 }, /* BeamOrder */
			NUMTELS, { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /* DLOffset */
			0, /*OffsetsReady */
			0, /*manOffsetsReady */
			W1, /* RefTel */
			2.15,//1.25,	//2.15, /*  lambda */
			0.3, //0.2,	//0.3, /* bandwidth */
			1024, /* framesPerBuffer */
			FALSE, /* abort */
			FALSE, /*  continuousReadout */
			"champ0", /*image_filename */
			"                " /* string for calling spooler */
};

pythonconfig_struct pyconfig = { NUMTELS, //USED in COMBINER.
		{ W1, E1, E2, S1, S2, W2  }, W1, NUMTELS, { 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0 }, //FiberExplorerY
		1.65, 0.3, { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 }, //gdtThreshold
		0.0,//gdtGain
		{ 0, 0, 0, 0, 0, 0 }, { 0, 0, 0, 0, 0, 0 }, { 0, 0, 0, 0, 0, 0 }, 0,
		/* NOTE***** string must be shorter one character than declaration */
		/* because last caracter must be a NULL */
		"File Description             ", // File Description
		"Filter 1                     ", // Filter 1
		"Filter 2                     ", // Filter 2
		"K                            ", // Config Name
		"S1=0,S2=1,E1=2,E2=3,W1=4,W2=5", // TelKey
		"Observer Name                ", // Observer
		"Seeing information here      ", // comment1
		"Acquisition info here        " // comment2
};

gdt_struct gdt = { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /* dlOffsets */
		{ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /* manualOffsets */
		{ FALSE, FALSE, FALSE, FALSE, FALSE, FALSE }, /* activeBaselines */
		{ FALSE, FALSE, FALSE, FALSE, FALSE, FALSE }, /* gdtLock */
		FALSE /* continuousReadout */
};

ft_struct ftdata = { 50, /*dither_freq*/
		     0.7, /*dither_amp FB 2012 */
		     7.3, /*opdamp microns*/
		     0.05, /*dither_smoothfrac*/
		     0.0, /*dither_delay*/
		     { FALSE, FALSE, FALSE, FALSE, FALSE, FALSE }, /*active baseline*/
		     { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /*ft offset*/
		     { 0.1, 0.1, 0.1, 0.1, 0.1, 0.1 }, /*ft gain*/
		     { 0, 0, 0, 0, 0, 0 }, /*search state*/
		     { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /*search step*/
		     { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /*search threshold*/
		     "jdm_opds.txt       " // OPD maps
};

ft_struct ftdata_old = { 50, /*dither_freq*/
			 0.7, /* dither_amp FB 2012 */
			 7.3, /*opdamp microns*/
			 0.05, /*dither_smoothfrac*/
			 0.0, /*dither_delay*/
			 { FALSE, FALSE, FALSE, FALSE, FALSE, FALSE }, /*active baseline*/
			 { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /*ft offset*/
			 { 0.1, 0.1, 0.1, 0.1, 0.1, 0.1 }, /*ft gain*/
			 { 0, 0, 0, 0, 0, 0 }, /*search state*/
			 { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /*search step*/
			 { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }, /*search threshold*/
			 "jdm_opds.txt       "
};

double dloffsets[NUMDLS] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
double last_dloffsets[NUMDLS] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
double dloffsets_test[NUMDLS];
float autoThreshold[6][AUTONUM];
unsigned int autoThresholdCounter = -1 ;
int DL_DEBUG = 0;
int VMEOPLE = 1; // 0 - use ople socket   1= talk to vme directly
int vmecounter = 0;

/* JDM  2009Aug20
 * New Regime of variables for better configuration control
 * Some of these should be grouped into a structure...
 */

float BC_FT_Servo_Gains[6] = { .1, .1, .1, .1, .1, .1 };
float BC_FT_GDT_Gains[6] = { -0.25, -0.25, -0.25, -0.25, -0.25, -0.25 };
float BC_FT_Auto_Threshold_Mean[6] = {0, 0, 0, 0, 0, 0};
float BC_FT_Auto_Threshold_RMS[6] = {10000,10000,10000,10000,10000,10000};
unsigned char BC_Search_Mode[6] = { BC_PADDLE, BC_PADDLE, BC_PADDLE, BC_PADDLE, BC_PADDLE, BC_PADDLE };
float BC_Search_Speed[6] = { 10., -10., 10., -10., 10., -10. }; // mu/sec

double BC_Ref_Offset[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
double BC_FT_Offset[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
double BC_FT_Stage[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }; // staging offsets until end.
double BC_FT_Stage1[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }; // staging offsets until end.
double BC_Spiral_Edge[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }; // Edges for spiral search
double BC_Spiral_Step[6] = { 10.0, 10.0, 10.0, 10.0, 10.0, 10.0 }; // in millimeters
double BC_Spiral_Position[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }; 
double BC_FT_Stage_Weight[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
unsigned char BC_FT_State[6] = { BC_FT_OFF, BC_FT_OFF, BC_FT_OFF, BC_FT_OFF, BC_FT_OFF, BC_FT_OFF };
unsigned char BC_FT_State_Last[6] = { BC_FT_OFF, BC_FT_OFF, BC_FT_OFF, BC_FT_OFF, BC_FT_OFF, BC_FT_OFF };

float BC_Amp_Threshold[6] = { 1., 1., 1., 1., 1., 1. }; // used to be 100
float BC_Amp_Threshold_Mean[6] = { 1., 1., 1., 1., 1., 1. }; 
float BC_Amp_Threshold_RMS[6] = { 1., 1., 1., 1., 1., 1. }; 

float BC_FT_StoSL_SNR_Threshold[6] = { 5., 5., 5., 5., 5., 5. };
float BC_FT_SLtoL_SNR_Threshold[6] = { 10., 10., 10., 10., 10., 10. };
float BC_FT_LtoSL_SNR_Threshold[6] = { 5., 5., 5., 5., 5., 5. };
float BC_FT_SLtoS_SNR_Threshold[6] = { 4., 4., 4., 4., 4., 4. };

unsigned int BC_FT_StoSL_Transition[6] = { 1, 1, 1, 1, 1, 1 };
unsigned int BC_FT_SLtoL_Transition[6] = { 10, 10, 10, 10, 10, 10 };
unsigned int BC_FT_LtoSL_Transition[6] = { 100, 100, 100, 100, 100, 100 };
unsigned int BC_FT_SLtoS_Transition[6] = { 10, 10, 10, 10, 10, 10 };

unsigned int BC_FT_StoSL_Counter[6] = { 0, 0, 0, 0, 0, 0 };
unsigned int BC_FT_SLtoL_Counter[6] = { 0, 0, 0, 0, 0, 0 };
unsigned int BC_FT_LtoSL_Counter[6] = { 0, 0, 0, 0, 0, 0 };
unsigned int BC_FT_SLtoS_Counter[6] = { 0, 0, 0, 0, 0, 0 };

double BC_Sweep_Target[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };  // current sweeping target
double BC_Sweep_High[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }; // Upper edge
double BC_Sweep_Low[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }; // Upper edge
double BC_Sweep_Step[6] = { 100.0, 100.0, 100.0, 100.0, 100.0, 100.0 }; // microns
double BC_Sweep_Direction[6]= {1, 1, 1, 1, 1, 1};
double BC_Sweep_Speed[6] = { 20.0, 20.0, 20.0, 20.0, 20.0, 20.0 };  // microns / sec
double BC_Sweep_Position[6] = { 20.0, 20.0, 20.0, 20.0, 20.0, 20.0 }; // current position
double BC_Sweep_LongTermA[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }; // long term trend linear A + B * ditherCount
double BC_Sweep_LongTermB[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }; // see line above
//double *BC_Sweep_LastKnownPositions; // stores the last known positions 
double BC_Sweep_LastKnownPositions[6] = {0, 0, 0, 0 , 0 , 0}; 
unsigned int BC_Sweep_LastKnownIndex[6] = {0, 0, 0, 0 , 0 , 0}; // the very last position where we detected a fringe
float BC_Sweep_SNR_Threshold[6] = { 5., 5., 5., 5., 5., 5. }; // determine if detected event should be stored as last position
unsigned char BC_Sweep_Unsuccessful[6] = {0, 0, 0, 0 ,0, 0};

int searchFlag = 0;
int vme_active = 0;

#define METHOD_CLASSIC_SCAN 1
#define METHOD_PREDICITVE_SCAN 2

int METHOD = METHOD_CLASSIC_SCAN;

// New scanning mode
// we need: 
// - history of FT_Offsets (as a function of dithers ?)
// - FT_Num weight history and/or BC_FT_State history 
// BC_Predict_Position : predicted position based on last good positions + long term trend
// could be given by linear regression or Kalman filter (requires error estimation)
// simplest form is based on running averages
// BC_Predict_Range : dynamic range/probability range 


unsigned int NACCUMULATIONS[MAXCOMBOS/2] = {2, 2, 2, 2, 2, 2};

long dither_accum0[MAXCOMBOS/2] = {0, 0, 0, 0, 0, 0};

int NACCUMULATIONS_CHANGED[MAXCOMBOS/2] = {FALSE, FALSE, FALSE, FALSE, FALSE, FALSE};

/* Global Paddle parameters */
int BC_PADDLE_State = 0 ;
int BC_PADDLE_Index = 0 ;
double BC_PADDLE_Offset = 0.0;
/* Global Init Reference */
int BC_INIT_Ref_State =0 ;
int BC_INIT_Ref_Index =0 ;


/* JDM 2009Aug20 */

/* VME globals */
char vme_hostname[81] = ("10.0.0.9");
struct sockaddr_in vme_addr;
int vme_sd;
int send_to_vme = FALSE;
int vme_port = 6001;
char vme_message[81];
double vme_beam_offsets[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
double temp_dl_offsets[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
double vme_dl_offsets[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
double vme_dl_offsets_byteswap[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
double socket_double6[6];
int socket_int6[6];
float socket_float6[6];
float socket_float1;
double socket_double1;
int socket_int1;

; // in microns for lab testing
float ft_offsets[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
unsigned int nsaved = 0, dithersleft = 0;
int write_header = TRUE , write_footer = FALSE;
unsigned int nBkgnd = 0, nBkgnd_total = 0; //, nDebias =0, nDebias_total;
int pixelMap[MAXIMAGE];


// GDT global variables for transfer                                                                                                                                                              
float* gdt_amp_transfer = NULL;
float* gdt_opds_transfer = NULL ;
float* gdt_filtered_amp_transfer = NULL;
double* ft_phi_transfer = NULL;
//float parabola_transfer[3 * MAXCOMBOS] ;

float estimated_delay_transfer[MAXCOMBOS];     
float* ft_phi_stddev_transfer = NULL;
float* ft_phi_mean_history = NULL;
float* flux_history = NULL;
float* gdt_delay_history;
float* ft_offsets_history = NULL;
float* ft_num_history = NULL;
double *BG_ft_num = NULL;

unsigned int ditherCount_transfer = 0 ;
unsigned int scanCount_transfer = 0 ;
unsigned int ditherCount_transfer_modAvg = 0 ;

int slice_history_transfer( float* data_to_transfer, message_struct* message, unsigned int current_position, unsigned int wrap, unsigned int history_size  );



//************************************************************************/
/* set_manual_target()							*/
/*									*/
/* Function to set a manual target.					*/
/************************************************************************/

void set_manual_target(int server, int dl, double pos)
{
	struct smessage mess;
	struct s_target target;

	/* Set out the command data */

	target.dl_number = (int) dl;
	target.position = (double) pos; /* meters */
	target.rate = (double) 0.0; // rate; /* meters per second */
	target.time = 0; //(unsigned long) opletime; /* Only important if rate != 0 */

	/* Send messages */

	mess.type = OPLE_MANUAL_TARGET;
	mess.length = sizeof(target);
	mess.data = (unsigned char *) &target;
	//printf("Talking DIRECTLY to OPLE !!\n");
	if ( !send_message(server, &mess) )
	{
		fprintf(stderr, "Failed to set Manual target.");
	}

} /* set_manual_target() */

/************************************************************************/
/* set_manual_target()							*/
/*									*/
/* Function to set a manual target.					*/
/************************************************************************/

void vme_set_manual_target(int ople_socket, int dl_number, double position)
{
	/* this is essentially a direct copy of the routine ople_manual_target
	 which can be found in control/modules/ople/ople_comms.c
	 */

	double rate = 0.0;
	OPLE_TIME time = 0;
	ISIT_CMD command, command_send;
	int iGizmoId = 0;
	int delay_line_gizmo_id[NUM_DELAY_LINES] = { 1, 2, 3, 4, 5, 6 };
	char *c1, *c2; /* For copying things */
	int i; /* Loop variable */

	position = position * LASER_SIGN / LASER_TO_METERS;

	/* Is this a valid delay line number? */

	if ( dl_number < 0 || dl_number >= NUM_DELAY_LINES )
	{
		printf("Invalid delay line number %d.\n", dl_number);
		return;
	}
	/* Rightio, construct the command */

	command.genericCmd.Command = INST_CMD_SET_MANUAL_TARGET;
	command.instsmtCmd.iDlGizmoId = delay_line_gizmo_id[dl_number];
	command.instsmtCmd.Target.dPosition = rint(position);
	command.instsmtCmd.Target.dRate = rint(rate);
	command.instsmtCmd.Target.tTime = time;

	/* next part is copy of send_ople_command() in ople_comms.c */

	/* Fill in some of the bits of Command structure */

	command.genericCmd.iGizmoId = iGizmoId;
	command.genericCmd.tTimeStamp = 0; // can I get away with this? JDM

	/* copy the command structure over */

	for ( c1 = (char *) &command, c2 = (char *) &command_send, i = 0; i < (int) sizeof(command) ; i++, c1++, c2++ )
		*c2 = *c1;

	/* First thing we do is check out what command this is */
	/* hey.. its INST_CMD_SET_MANUAL_TARGET */
	byte_swap(command_send.instsmtCmd.iDlGizmoId);
	byte_swap(command_send.instsmtCmd.Target.dPosition);
	byte_swap(command_send.instsmtCmd.Target.dRate);
	byte_swap(command_send.instsmtCmd.Target.tTime);

	/* byte swap the command itself */
	byte_swap(command_send.genericCmd.tTimeStamp);
	byte_swap(command_send.genericCmd.iGizmoId);
	byte_swap(command_send.genericCmd.iWhichRte);
	byte_swap(command_send.genericCmd.iWhichCpu);
	byte_swap(command_send.genericCmd.Command);

	/* Finally!!! we send it away */
	//printf("Talking DIRECTLY to vme !!\n");
	if ( write(ople_socket, &command_send, sizeof(command_send)) != sizeof(command_send) )
	{
		printf("Failed to send command to ople cage.");
		return;
	}

} /* vme set_manual_target() */

/************************************************************************/
/* set_manual_offsets()							*/
/*									*/
/* Function to set a manual target.					*/
/************************************************************************/

void set_manual_offsets(int server, double *pos)
{
	struct smessage mess;

	/* Send messages */

	mess.type = OPLE_MANUAL_OFFSETS;
	mess.length = 6 * sizeof(double);
	mess.data = (unsigned char *) pos;

	if ( !send_message(server, &mess) )
	{
		fprintf(stderr, "Failed to set Manual offsets.");
	}

} /* set_manual_offsets() */

/***********************************************************************
 *	Calculate frames per buffer
 ***********************************************************************/
int nFramesPerBuffer(unsigned short *mem_fd, int bufferSize, int rows, int cols, int bits_per_pixel)
{
	long x, framesPerBuffer; // JDM int->long
	unsigned short *imageEndAddr;

	x = 1;
	imageEndAddr = mem_fd + (rows * cols * (bits_per_pixel / 8));
	while (((long) (imageEndAddr + x) % 1024) != 0)
	{ //JDM int-> long
		x++;
		//		printf("x: %ld div: %ld\t", x, (long) (imageEndAddr + x) % 1024);
		//	printf("User memory address:  0x%X\t", (int) mem_fd);

	}
	//	printf("\n");
	framesPerBuffer = (int) floor((double) (bufferSize / (imageEndAddr - mem_fd + x)));
	/* The framesPerBuffer must be evenly divisible by 4 for this mode, which requires
	 the buffer be sub-divided in quarters. */
	while (framesPerBuffer % 4 != 0)
	{
		framesPerBuffer--;
		printf("framesPerBuffer %ld\n", framesPerBuffer);
	}

	return framesPerBuffer;
}
/* new version from Scott Streit on 2009 Jul 22 */
int nFramesPerBuffer2(long lBuffSize, long lRows, long lCols)
{
	long lBoundedImageSize = 0;
	long lImageSize = lRows * lCols * sizeof(unsigned short);
	long lFramesPerBuffer = 0;
	if ( (lImageSize & 0x3FF) != 0 )
		lBoundedImageSize = lImageSize - (lImageSize & 0x3FF) + 1024;
	else
		lBoundedImageSize = lImageSize;

	lFramesPerBuffer = (long) floor((float) (lBuffSize / lBoundedImageSize));

	return (int) lFramesPerBuffer;
}

/*******************************************************************************
 *
 *	Function:
 *	---------
 *	int create_memory(int pci_fd, int rows, int cols)
 *
 *	Description:
 *	------------
 *       This function is used to allocate space to an image buffer.
 *	The function returns a pointer to a block of at least 'image_size'
 *	bytes suitably aligned for any use.
 *
 *	Parameters:
 *	-----------
 *	pci_fd  	The reference to the PCI driver returned by open().
 *	rows	  	The number of image rows (pixels).
 *	cols	  	The number of image columns (pixels).
 *	size		The size of the buffer in bytes (no affect on WIN2K).
 *
 *	Returns:
 *	--------
 *	NULL if a not successfull, Pointer to image buffer otherwise.
 *
 *
 *       Version:        1.00
 *       Author:         Scott Streit
 *       Date:           02/05/2001
 *
 *	Revision History:
 *	-----------------
 *	Date            Who   Version    Description
 *	----------------------------------------------------------------------
 *	02/05/2001      sds     1.00    Initial
 *
 *
 *******************************************************************************/
unsigned short *create_memory(int pci_fd, unsigned int bufferSize)
{
	unsigned short *mem_fd;
	//printf("\nCalling mmap ...\n");

	/* Map the kernel buffer */
	mem_fd = (unsigned short *) mmap(0, bufferSize, (PROT_READ | PROT_WRITE), MAP_SHARED, (int) pci_fd, 0);

	//printf("\nmmap fd = 0x%X\n", (int) mem_fd);

	/* Check for an out of physical memory error. */
	if ( mem_fd == NULL && errno == ENOMEM )
	{
		sprintf(error_string, "Error Allocating Image Buffers. The Systems \n Physical Limits Have Been Exceeded. \n");
		return NULL;
	}

	/* Check for an out of memory error. */
	else if ( mem_fd == NULL && errno == EAGAIN )
	{
		sprintf(error_string, "Error Allocating Image Buffers. \n Not Enough Memory CURRENTLY Available. \n");
		return NULL;
	}

	/* Check for any other errors. */
	else if ( mem_fd == NULL )
	{

		sprintf(error_string, "Error Allocating Memory (errno = %d) \n", errno);
		return NULL;
	}

	return mem_fd;

}

/***********************************************************************
 *	DEINTERLACE IMAGE
 ***********************************************************************/
int deinterlace(int cols, int rows, unsigned short *image_fd)
{
	unsigned short *old_iptr;
	unsigned short *new_iptr;
	int i = 0;
	//int j = rows-1;
	int j = 0;
	int counter = 0;
	int end = 0;
	int begin = 0;

	old_iptr = image_fd;

	/* Allocate a new buffer to hold the deinterlaced image. */
	if ( (new_iptr = (unsigned short *) malloc(cols * rows * sizeof(unsigned short))) == NULL )
	{
		sprintf(error_string, "Error in allocating memory for deinterlacing. \nSaving interlaced image to disk.");
		free(new_iptr);
		return _ERROR;
	}

	if ( (float) cols / 2 != (int) cols / 2 || (float) rows / 2 != (int) rows / 2 )
	{
		sprintf(error_string, "Number of cols AND rows must be EVEN for quad readout. \nSkipping deinterlace.");
		free(new_iptr);
		return _ERROR;
	}

	/*--------------- IR SPLIT QUAD READOUT --------------------------------------*/
	while (i < cols * rows)
	{
		if ( counter % (cols / 2) == 0 )
		{
			//end = (j-(rows/2))*cols;
			end = (j + (rows / 2)) * cols;
			begin = j * cols;
			//j--; 			/*number of completed rows*/
			j++; /*row counter*/
			counter = 0; /*reset for next convergece*/
		}

		//*(new_iptr+begin+counter)       = *(old_iptr+i++);		/* front_row--->  */
		//*(new_iptr+begin+(cols/2)+counter) = *(old_iptr+i++);	/* front_row<--   */
		//*(new_iptr+end+(cols/2)+counter)   = *(old_iptr+i++);	/* end_row<----   */
		//*(new_iptr+end+counter)         = *(old_iptr+i++);		/* end_row---->   */

		//DHB reorder quads but still need fix j (flip within quad)
		*(new_iptr + begin + (cols / 2) + counter) = *(old_iptr + i++); /* front_row<--   */
		*(new_iptr + begin + counter) = *(old_iptr + i++); /* front_row--->  */
		*(new_iptr + end + counter) = *(old_iptr + i++); /* end_row---->   */
		*(new_iptr + end + (cols / 2) + counter) = *(old_iptr + i++); /* end_row<----   */
		counter++;
	}

	memcpy(old_iptr, new_iptr, cols * rows * sizeof(unsigned short));
	free(new_iptr);

	return _NO_ERROR;
}

/***********************************************************************
 *	CUMULATORS FOR LINEAR FIT
 *  src_fd  = mem_fd = camera data
 *  dets_fd = pointer containing 12(Sxy - 0.5(N+1)Sy)/(N(N^2-1))
 *  r       = number of rows
 *  c       = number of columns
 *  nr      = number of reads
 *  nsr     = number of skipped reads
 *  nca     = number of coadds
 *  cac     = coaddCount
 *  m       = frame number in mem_fd
 ***********************************************************************/
void linearFit(float *src_fd, float *dest_fd, unsigned short r, unsigned short c, unsigned short nr, unsigned short nsr, unsigned short nca, unsigned int cac)
{
	register float *ptr_src;
	register float *ptr_dest;
	register int i, j;

	  float n;
	unsigned short num_skip_reads;

	if ( nsr >= nr )
	{
		num_skip_reads = nr - 1;
	}
	else
	{
		num_skip_reads = nsr;
	}

	// number of coadds = 1, no fit, add reads
	if ( nca == 1 )
	  {
	    float c1;
	    c1 = 1. / (nr - num_skip_reads);
	    for ( i = 0, ptr_src = src_fd, ptr_dest = dest_fd; i < r * c ; i++ )
	      {
		// sum n read into one pixel
		ptr_src = ptr_src + num_skip_reads;
		for ( j = num_skip_reads; j < nr ; j++ )
		  *(ptr_dest + i) += (float) *(ptr_src++) * c1;
	      }
	}
	// number of coadds > 1, linear fit, add reads
	else
	{
	  float c1, c2;
	  n = (float) nca;
	  c1 = 0.5 * (n + 1);
	  c2 = 12. / (n * (n * n - 1.) * nr);
	  for ( i = 0, ptr_src = src_fd, ptr_dest = dest_fd; i < r * c ; i++ )
	    {
	      // sum n read into one pixel
	      ptr_src = ptr_src + num_skip_reads;
	      for ( j = num_skip_reads; j < nr ; j++ )
		*(ptr_dest + i) += (float) *(ptr_src++) * ((float) cac - c1) * c2;
	    }
	}
}

/*
void fluxQuad(float *src_fd, float flux, unsigned short r, unsigned short c, unsigned short nr, int quad)
{
	// Not sure best way to use register for speed....jdm
	register float *ptr_src;
	register int i, j, k;
	int i0, i1, j0, j1; // for choosing which Quad to add up.
	float c1;

	c1 = 1.0 / ((float) nr * (float) c / 2 * (float) r / 2); //Normalization
	if ( quad == 0 )
	{ // Do bottom-left quad. other quads not defined yet.
		i0 = 0;
		i1 = c / 2;
		j0 = 0;
		j1 = r / 2;
	}
	flux = 0.0;
	for ( j = j0, ptr_src = src_fd; j < j0 ; j++ )
	{
		for ( i = i0; i < i1 ; i++ )
		{
			for ( k = 0; k < nr ; k++ )
			{
				flux += *(ptr_src + j * c * nr + i * nr + k);
			}
		}
	}
	flux *= c1;
}
*/

/*******************************************************************************
 *
 *	Function:
 *	---------
 *	void swapmemory_memory()
 *
 *	Description:
 *	------------
 *       used to byte swap the words within the data Dwords.
 *
 *	Parameters:
 *	-----------
 *	mem_fd	The reference to the image buffer returned by create_memory().
 *	size    The number of bytes to swap.
 *
 *       Returns:
 *	--------
 *	None.
 *
 *       Version:        1.00
 *       Author:         Scott Streit
 *       Date:           08/16/2001
 *******************************************************************************/
void swap_memory(unsigned short *mem_fd, int size)
{
	int x;
	unsigned int temp1;
	unsigned int temp2;
	unsigned int temp3;
	unsigned int temp4;
	unsigned short *fd = mem_fd;

	/* Check for a NULL image buffer. */
	if ( mem_fd == NULL )
	{
		sprintf(error_string, "Error! Attempting To Access A NULL Image Buffer. \n");
		return;
	}

	/* Swap the image data bytes within the words. */
	for ( x = 0; x < (size / 2) ; x++ )
	{
		temp1 = fd[x];
		temp2 = fd[x];

		temp3 = ((temp1 & 0x00ff) << 8);
		temp4 = ((temp2 & 0xff00) >> 8);

		fd[x] = (temp4 | temp3);
	}
}

/***********************************************************************
 *     Client task to send offsets to delay lines
 ***********************************************************************/
void target_task(void *cookie)
{
	char c;
	int charbufi;
	int opleserver;
	int ople_socket = 0; //vme
	unsigned char j;
	struct smessage mess; //JDM
	unsigned short messtype;
	unsigned int messlength;
	//  struct sockaddr_in serv_addr, cli_addr;
	int ople_setup;
	//  RTIME start_clock,now_clock;

	// Init server's data structure
	printf("init jobs\n");
	exc.endTarget = FALSE;
	init_jobs();
	//JDM  Setup OPLE socket as completely separate socket that stays open.
	opleserver = (int) open_server_socket(OPLE, 2200, "OPLE");
	printf("opleserver = %d   ", opleserver);
	if ( VMEOPLE )
	{
		printf("initing vme communiction.\n");
		ople_socket = (int) open_connect_socket(OPLE_MACHINE, OPLE_SOCKET);
		printf("ople_socket (vme) = %d", ople_socket);
	}

	//JDM Get current offsets -- from now on, the server will be only keeper
	//of this information
	mess.type = OPLE_CURRENT_MANUAL_TARGETS;
	mess.length = 0;
	mess.data = (unsigned char *) malloc(4 * sizeof(unsigned char));
	printf("Sending OPLE message number == %d\n", mess.type);
	if ( !send_message(opleserver, &mess) )
	{
		fprintf(stderr, "Failed to send OPLE CURRENT MANUAL TARGETS\n");
	}

	//JDM Wait for answer (do I need to cast the char -> doubles?)
	ople_setup = 0; //kludgy way of dealing with ople server to get offsets initially!
	while (ople_setup == 0)
	{
		while (client_read_socket(servers[opleserver].fd, (char *) &messtype, 2) == 0)
		{
			rt_task_sleep(1000000);
		};
		printf("message type = %d\n", messtype);

		while (client_read_socket(servers[opleserver].fd, (char *) &messlength, 4) == 0)
		{
			rt_task_sleep(1000000);
		};

		// allocate data buffer
		mess.data = (unsigned char *) realloc(mess.data, messlength);

		while (client_read_socket(servers[opleserver].fd, (char *) mess.data, messlength) == 0)
		{
			rt_task_sleep(1000000);
		}
		printf("got the data for %d %d %d \n", messlength, messtype, OPLE_CURRENT_MANUAL_TARGETS);

		if ( messtype == OPLE_CURRENT_MANUAL_TARGETS )
		{
			if ( !VMEOPLE )
			{
				memcpy(dloffsets, mess.data, messlength);
				memcpy(last_dloffsets, mess.data, messlength);
			}
			ople_setup = 1;
		}
	}

	//start_time = (long)   (  (double) (start_clock>>18) /3.8147 ); //convert to ms

	printf("\n dloffset 0= %lf 1= %lf 2= %lf 3= %lf 4= %lf 5= %lf\n", *dloffsets, *(dloffsets + 1), *(dloffsets + 2), *(dloffsets + 3), *(dloffsets + 4), *(dloffsets + 5));

	// start_time=GETCLOCK();

	/* Loop to send offsets to DLs */
	while (!exc.endProg && !exc.endTarget)
	{
		rt_task_sleep(50000000); //erg!


		// *******************************************************************************
		// JDM KEEP AFTER TEST     rt_task_sleep(500000000); // 0.5 second delay (for Now)

		/* Next set of code used for testing. */
		//start_clock=rt_timer_read();
		//rt_task_sleep(500000000); // 0.5 second delay (for now)
		//  rt_task_sleep(1000000000); // 1.0 second delay (for now)

		/*  test_counter++; */
		/*       for (j=0;j<NUMDLS;j++){ */
		/* 	dloffsets[j]=j+ test_counter* 1e-6; */
		/* 	} */

		/* 	//	printf("DL %d :  %lf",j,dloffsets[j]); */
		/* 	exc.DLOffset[j] = 0; */
		/*        gdt.manualOffset[j] = 0; */

		/*       // printf("\n"); */
		/*        //	set_manual_offsets(opleserver, dloffsets); */
		/*        // for (j=0;j<NUMDLS;j++){ */
		/*        for (j=0;j<3;j++){ */
		/* 	 now_time=GETCLOCK(); */
		/* 	 rt_task_sleep(300000000); */
		/*         set_manual_target(opleserver,j,dloffsets[j]); */
		/*         printf("DELAY %i: %d (ms) \n",j,(int) (GETCLOCK()-now_time)); */
		/*        } */
		/* 	    printf("(%d) DL Offsets SENT: %lf %lf %lf %lf %lf %lf \n", */
		/*              (int)(GETCLOCK()-start_time) ,  */
		/* 	     dloffsets[0], dloffsets[1],dloffsets[2],dloffsets[3],dloffsets[4],dloffsets[5]); */
		/* 	    printf("AVERAGE TIME (ms): %f \n",(float)(GETCLOCK()-start_time)/test_counter); */
		/* mess.type=OPLE_CURRENT_MANUAL_TARGETS; */
		/*   mess.length=0; */
		/*   // mess.data=(char *)malloc(4);  //JDM commented out 2006Mar -- not needed, right? */
		/*    //printf("Sending OPLE message number == %d\n",mess.type); */
		/* /\*   if (!send_message(opleserver,&mess)) *\/*/
		/* /\*     { *\/*/
		/* /\*       fprintf(stderr,"Failed to send OPLE CURRENT MANUAL TARGETS\n"); *\/*/
		/* /\*     } *\/*/

		/*   // Now ask for offsets  */
		/*   //rt_task_sleep(100000000); */
		/*   ople_setup=1;  //kludgy way of dealing with ople server to get offsets initially! */
		/*   while(ople_setup == 0){ */
		/*     while (client_read_socket(servers[opleserver].fd,(char *) &messtype,2) == 0){ */
		/*       rt_task_sleep(1000000); */
		/*     }; */
		/*     //  printf("message type = %d\n",messtype ); */

		/*     while (client_read_socket(servers[opleserver].fd,(char *) &messlength,4) == 0){ */
		/*       rt_task_sleep(1000000); */
		/*     }; */
		/*     printf("message length = %d\n", messlength ); */
		/*     // allocate data buffer */
		/*     mess.data = (unsigned char *) realloc(mess.data, messlength); */
		/*     while (client_read_socket(servers[opleserver].fd,(char *) mess.data, messlength) == 0){ */
		/*       rt_task_sleep(1000000); */
		/*     } */
		/*     if (messtype == OPLE_CURRENT_MANUAL_TARGETS) { */
		/*       memcpy(dloffsets_test,mess.data,messlength); */
		/*       ople_setup = 1; */
		/*              printf("(%ld) DL Offsets RCVD: %lf %lf %lf %lf %lf %lf \n",(long)(GETCLOCK()-start_time),  */
		/* 	     dloffsets_test[0], dloffsets_test[1],dloffsets_test[2],dloffsets_test[3], */
		/*          dloffsets_test[4],dloffsets_test[5]); */
		/*     }} */

		// ********************** END OF TEST *****************************************


		// if vme communcation, then check to see if there is any data that needs
		// read out -- if so, then read and dump.

		if ( VMEOPLE )
		{
			charbufi = 0;
			while (char_waiting(ople_socket))
			{
				read(ople_socket, &c, 1); //empty buffer
				charbufi++;
			}
			//if (charbufi > 0)
			//     printf("Emptied VME Socket Buffer: %i \n",charbufi);// TEST
		}

		if ( exc.OffsetsReady || exc.manOffsetsReady )
		{

			for ( j = 0; j < NUMDLS ; j++ )
			{

				if ( (exc.DLOffset[j] == exc.DLOffset[j]) && (exc.OffsetsReady) )
				{
					dloffsets[j] = dloffsets[j] + exc.DLOffset[j]; //increment dl
					exc.DLOffset[j] = 0;
				}
				if ( (gdt.manualOffset[j] == gdt.manualOffset[j]) && (exc.manOffsetsReady) )
				{
					dloffsets[j] = dloffsets[j] + gdt.manualOffset[j];
					gdt.manualOffset[j] = 0;
				}

				//	printf("DL %d :  %lf",j,dloffsets[j]);
				//exc.DLOffset[j] = 0;
				//gdt.manualOffset[j] = 0;

			}
			// printf("\n");
			//JDM I need to modify this to only update DLs that change since there is
			// overhead. 300ms per channel
			for ( j = 0; j < NUMDLS ; j++ )
			{
				if ( dloffsets[j] != last_dloffsets[j] )
				{ // if changed, then update
					printf("Sending %lf to DL %d\n", dloffsets[j], j);

					if ( !VMEOPLE )
					{
						set_manual_target(opleserver, j, dloffsets[j]);
					}
					if ( VMEOPLE )
					{
						vme_set_manual_target(ople_socket, j, dloffsets[j]);
					}

					last_dloffsets[j] = dloffsets[j];
					//rt_task_sleep(330000000);  // this might be too fast when
					// observing with CLASSIC TOO... slowwing down
					// slightly...
					rt_task_sleep(400000000);

					// rt_task_sleep(100000000); // can we speed up? anwer: no!
					//rt_task_sleep(750000000); // should be 1/3 second -- what changed?
				}
			}

			//printf("Offsets sent\n");
			exc.OffsetsReady = FALSE;
			exc.manOffsetsReady = FALSE;
		}
	}
	//JDM close out the opleserver socket which had remained open.
	printf("Shutting down the delay line communication task.\n");
	close_server_socket(opleserver);
	if ( VMEOPLE )
		close(ople_socket);
	fflush(NULL);
}
/* end of target_task */


void* CHAMPrealloc(void* ptr, size_t size) {
  ptr = realloc(ptr, size);
  if ( !ptr ) {
    printf("WARNING -- REALLOC FAILURE \n");
    exit(1);
  }
  return ptr;
}

void* CHAMPalloc(void* ptr, size_t size) 
{
  if( !ptr )
    {
      CHAMPrealloc(ptr, size) ;
    }
  else 
    {
      ptr = malloc(size);
      if ( !ptr ) 
	{
	  printf("WARNING -- MALLOC FAILURE\n");
	  exit(1);
	}
    }
  return ptr;
}





/***********************************************************************
 *     Task to transfer data from astropci on interrupt
 ***********************************************************************/
void sem_task(void *cookie)
{
	RT_SEM switch_sem;
	//  RT_SEM dataReady;
	KEY_STATE ks = S_IDLE;
	KEY_STATE ks_next = S_IDLE;

	//  int vme_flag=0;
	int astropci_reply = 0;
	int astropci_reply_prev = 0;
	int calc_offset = 0;
	unsigned short memrange;
	unsigned short ditherend, scanend;
	float num_boundaries;
	int num_boundaries_int;
	int roundup;
	long frameSizePixels = 0;
	long frameSizeShort = 0;
	int n = 0, b;
	float *channelFlux, *channelLastFlux ; 
	int normalflux;
	//  float params[5];
	long f, i, j, k =0 , kprime;
	//	int hh;
			double sss, ttt;
	int frame_offset;
	//	double delta_wavenumber;

	//  RTIME start_frame_clock=0;
	RTIME start_calc_clock = 0;
	//	RTIME timer1 = 0, timer2 = 0;
	RTIME *frametimes;
	//  int firstframe=0;
	double frametime = 0;
	//  double frametime0=0.0;
	long nframetimes = 0;
	double meanframetime = 0.0;
	float ft_phi_mean[MAXCOMBOS/2], ft_phi_stddev[MAXCOMBOS/2];
	//  double ft_a, ft_b, ft_c, ft_d;
	double *ft_x, *ft_y, *ft_n, *ft_num, *ft_snr, *ft_pow, *ft_phi;
	double ft_n1 , ft_x1, ft_y1, ft_n2, ft_x2, ft_y2;
	double *fts_x, *fts_y, *fts_n, *fts_num, *fts_phi, *fts_snr, *fts_pow;
	double *temp_offsets;

	int indx1 = 0, indx2 =0 , indx3 =0;
	int mem_fd_offset;
	int startBkgnd = FALSE;

	FILE *fp_temp;
	char outputfile[81];


	int writenumber = 42 * MAXCOMBOS / 2; 
	int save_step = 0;
	saveddata_struct saveddata;

	struct smessage message;
	int pixshift = 0;
	int sem_count = 0, t_i = 0, t_j = 0;
	int err = 0;
	RTIME now = 0, previous = 0;
	RTIME sem_preexp;
	//	RTIME gdt_time1, gdt_time2;
	unsigned int m = 0;
	unsigned int frameCount = 1,  timing_loop = 0, ntloops = 30;
	unsigned int ditherCount = 0, scanCount = 0;

	unsigned int totalFrameCount = 0;
	unsigned short *tmpBuffer;
	float *tmpBuffer1;
	float *coaddBuffer;
	unsigned int coaddCount = 1; // the number of completed coadds so far.
	unsigned int coherentCount = 1;
	unsigned int coaddFrame = 1;  //nth frame in a coadd sequence
	float *totalBuffer, *bkgndBuffer = NULL;
	double *timing_values;

        double round_delay = 0 ;
	float mean_opds = 0;
	float *channel_opds;
	float *channel_cospart;
	float *channel_sinpart;
	float *channel_x0, *channel_y0, *channel_n0; // uncorrect ones.

	float *abcd_cospart;
	float *abcd_sinpart;
	float *abcd_trigon;
	//        float *abcd_bwspart;
	int frame0, frame1, nframes;
	int frame_valid = 0;
	float nwave0;
	RTIME *timing_values_all;
	double reset0_to_reset1 = 0.0, dither_time = 0.0, dtime = 0.0;
	unsigned int loop_counter = 0, loop = 0;
	float medianFilter[BASELINES][NUMMEDIAN];
	char pipeName[7];
	int loopMsg = FALSE;
	float t_int = 0; // average frame time in ms

	int npix_channel[MAXCOMBOS];
	//	int npix_fringe[MAXCOMBOS/2] = { 4, 4, 4, 4, 4, 4};
	int npix_fringe[MAXCOMBOS/2] = { 5, 5, 5, 5, 5, 5 };
	float invn;
	int beams2combo[MAXCOMBOS] = { 1, -1, 1, -1, 1, -1, 1, -1, 1, -1, 1, -1 };
	float channel_delay[MAXCOMBOS];
	int *channel_pix[MAXCOMBOS];
	
	unsigned short frameskip = 0 ; // 0: all ok, 1: we've just skipped a frame 



	// Fringe amplitude statistics
	/*
	float fringe_amp_mean[MAXCOMBOS];
	float fringe_amp_mean_old[MAXCOMBOS];
	float fringe_amp_var[MAXCOMBOS];	
	float fringe_amp_var_old[MAXCOMBOS];
	float fringe_amp_stddev[MAXCOMBOS];
	unsigned int fringe_mean_counter = 0 ;  
	memset(fringe_amp_mean, 0, MAXCOMBOS * sizeof(float));
	memset(fringe_amp_mean_old, 0, MAXCOMBOS * sizeof(float));
	memset(fringe_amp_var, 0, MAXCOMBOS * sizeof(float));
	memset(fringe_amp_var_old, 0, MAXCOMBOS * sizeof(float));
	memset(fringe_amp_stddev, 0, MAXCOMBOS * sizeof(float));
	*/
	//	struct timespec tp;

	//	BC_Sweep_LastKnownPositions = malloc(MAXCOMBOS / 2 * FTMEMORY * sizeof(double)); // sweep memory
	//	memset(BC_Sweep_LastKnownPositions, 0 , MAXCOMBOS / 2 * FTMEMORY * sizeof(double));

	header_struct header 
	  = { 1000, MAXCOMBOS, 16, 160, 5, 38, 8, 10, 1000, 100, //scans per file
	      1, 4.0, 0.6, 5.0, 3.3, 0.1, 0, "/data/",
	      { NUMTELS, //USED in COMBINER.
		{ 0, 0, 0, 0, 0, 0 }, W1, NUMTELS, { 0, 0, 0, 0, 0, 0 }, //FiberExplorerX
		{ 0, 0, 0, 0, 0, 0 }, //FiberExplorerY
		1.65, 0.3, { 1, 1, 1, 1, 1, 1 }, //gdtThreshold
		0.6,//gdtGain
		{ 0, 0, 0, 0, 0, 0 }, { 0, 0, 0, 0, 0, 0 }, { 0, 0, 0, 0, 0, 0 }, 0,
		/* NOTE***** string must be shorter one character than declaration */
		/* because last caracter must be a NULL */
		"DATA                         ", // File Description
		"6. H Band                    ", // MIRC Filter 1
		"1. OPEN                      ", // MIRC Filter 2
		"H_PRISM                      ", // Config Name
		"S1=0,S2=1,E1=2,E2=3,W1=4,W2=5", // TelKey
		"Observer                     ", // Observer
		"Insert your comment here    ", // comment1
		"Insert your comment here     ", // comment2
	      } 
	  };
	
	// note that this header structure is not filled in for the last two
	// elements -- gdt and dloffsets.

	bzero(medianFilter, BASELINES * NUMMEDIAN * sizeof(float));
	daqOut.coaddCount = 0;


	// Creating pipe for spooler
	
	int pipe_creation = 1;
	
	while ( (pipe_creation !=0) && (pipeMinor < 32) )
	  {
	    sprintf(pipeName, "pipe%d", pipeMinor);
	    pipe_creation = rt_pipe_create(&pipe_desc, pipeName, pipeMinor, 0);
	    printf("Trying to open %s returned %d\n", pipeName, pipe_creation);
	    pipeMinor++;
	  }

	if ( pipeMinor < 32 )
	  { 
	    printf("Created rt_pipe: %s\n", pipeName);
	  }
	else
	  {
	    printf("Unable to create rt_pipe after %d attempts\n", pipeMinor);
	    exit(1);
	  }
	

	/* Create the accumulator image buffer */
	totalBuffer = (float *) malloc(MAXIMAGE * sizeof(float));
	if ( totalBuffer == NULL )
	{
		printf("Cannot allocate totalBuffer \n");
		exit(1);
	}
	coaddBuffer = (float *) malloc(MAXIMAGE * sizeof(float));
	/* Create the background frame buffer */
	bkgndBuffer = CHAMPalloc(bkgndBuffer, MAXIMAGE * sizeof(float));

	flux_history = malloc( MAXCOMBOS * FTMEMORY * sizeof(float));
	if ( flux_history == NULL )
	  {
	    printf("Cannot allocate flux history\n");
	    exit(1);
	  }

	channel_opds = malloc(MAXCHANDATA * sizeof(float));
	channel_cospart = malloc(MAXCHANDATA * sizeof(float));
	channel_sinpart = malloc(MAXCHANDATA * sizeof(float));
	channel_x0 = malloc(MAXCHANDATA * sizeof(float));
	channel_y0 = malloc(MAXCHANDATA * sizeof(float));
	channel_n0 = malloc(MAXCHANDATA * sizeof(float));
	abcd_cospart = malloc(MAXCHANDATA * sizeof(float));
	abcd_sinpart = malloc(MAXCHANDATA * sizeof(float));
	//	abcd_bwspart = (float *) malloc(MAXCHANDATA * sizeof(float));
	abcd_trigon = malloc(9 * MAXCHANDATA * sizeof(float)); // 3x3 matrix
	timing_values = malloc(sizeof(double) * 5000); // will be reallocated later

	if ( timing_values == NULL )
	{
		printf("Cannot allocate timing_values\n");
		exit(1);
	}

	timing_values_all = (RTIME *) malloc(sizeof(RTIME) * 50000); // will be reallocated later
	if ( timing_values_all == NULL )
	{
		printf("Cannot allocate timing_values_all\n");
		exit(1);
	}

	/* Create a temporary buffer */
	tmpBuffer = (unsigned short *) malloc(MAXIMAGE * sizeof(unsigned short));
	if ( tmpBuffer == NULL )
	{
		printf("Cannot allocate temporary buffer \n");
		exit(1);
	}

	/* Create a temporary buffer */
	tmpBuffer1 = (float *) malloc(MAXIMAGE * sizeof(float));
	if ( tmpBuffer1 == NULL )
	{
		printf("Cannot allocate temporary1 buffer \n");
		exit(1);
	}

	/* Create space for beam fluxes */
	channelFlux = (float *) malloc(MAXCHANDATA * sizeof(float));
	if ( channelFlux == NULL )
	{
		printf("Cannot allocate channel flux buffer \n");
		exit(1);
	}


	channelLastFlux = (float *) malloc(MAXCHANDATA * sizeof(float));
        if ( channelLastFlux == NULL )
	  {
	    printf("Cannot allocate channel flux buffer \n");
	    exit(1);
	  }



	/* create space for index array for flux channels */
	for ( i = 0; i < MAXCOMBOS ; i++ )
	{
		channel_pix[i] = (int *) malloc(sizeof(int) * MAXIMAGE); // expensive to reserve full array for each... but..
	}


	for ( n = 0; n < MAXCOMBOS / 2 ; n++ )
	{
	  ft_phi_mean[n] = 0;
	  ft_phi_stddev[n] = 0;
	}

	saveddata.ditherCount =  malloc(sizeof(unsigned int) * writenumber);
	saveddata.frame1 = malloc(sizeof(unsigned char) * writenumber);	
	saveddata.state = malloc(sizeof(unsigned char) * writenumber);
	saveddata.time = malloc(sizeof(double) * writenumber);
	saveddata.ftoffsets = malloc(sizeof(double) * writenumber);
	saveddata.refoffsets = malloc(sizeof(double) * writenumber);

	memset(  saveddata.frame1, 0, sizeof(unsigned char) * writenumber);
	memset(  saveddata.state, 0, sizeof(unsigned char) * writenumber);
	memset(  saveddata.ditherCount, 0, sizeof(unsigned int) * writenumber);
	memset( saveddata.ftoffsets, 0 , sizeof(double) * writenumber);
	memset( saveddata.refoffsets, 0 , sizeof(double) * writenumber);


	/* Setup up pointer for timing information */
	frametimes = (RTIME *) malloc(sizeof(RTIME) * 2000);
	BG_ft_num = malloc(sizeof(double) * MAXCOMBOS / 2);
	ft_x = malloc(sizeof(double) * MAXCHANDATA);
	ft_y = malloc(sizeof(double) * MAXCHANDATA);
	ft_n = malloc(sizeof(double) * MAXCHANDATA);
	ft_num = malloc(sizeof(double) * MAXCHANDATA);
	ft_phi = malloc(sizeof(double) * MAXCHANDATA);
	ft_snr = malloc(sizeof(double) * MAXCHANDATA);
        ft_pow = malloc(sizeof(double) * MAXCHANDATA);
	fts_x = malloc(sizeof(double) * FTMEMORY * MAXCHANDATA);
	fts_y = malloc(sizeof(double) * FTMEMORY * MAXCHANDATA);
	fts_n = malloc(sizeof(double) * FTMEMORY * MAXCHANDATA);
	fts_num = malloc(sizeof(double) * FTMEMORY * MAXCHANDATA);
	fts_phi = malloc(sizeof(double) * FTMEMORY * MAXCHANDATA);
	fts_snr = malloc(sizeof(double) * FTMEMORY * MAXCHANDATA);
	fts_pow = malloc(sizeof(double) * FTMEMORY * MAXCHANDATA);
	//	fts_gdt = malloc(sizeof(double) * 2 * FTMEMORY * MAXCHANDATA);
	temp_offsets = malloc(sizeof(double) * FTMEMORY); // only for lab setup

	if(VERBOSE == 1)
	  printf("Memory allocation done");
	// GDT variables
	int gdt_fftsamples = MAXCHANDATA;
	unsigned short gdt_update_dl = FALSE; 
	unsigned short gdt_above_threshold[6] = {FALSE,FALSE, FALSE,FALSE,FALSE,FALSE};

	// GDT variables dewarping
	
	float inv_nsamples;
	unsigned int naccumulated[MAXCOMBOS/2];
	int gdt_jump_accumulator[MAXCOMBOS/2];

	for(i=0; i<MAXCOMBOS/2;i++)
	  {
	    naccumulated[i]=0;
	    gdt_jump_accumulator[i] = 0;
	  }
	
	// Parabola estimation
	//	float tx, ty, txx, txy, txxx,

	// GDT variables
	int NINTERLEAVED = 1;
	//	int COPHASING = FALSE;
	float* gdt_X_accumulator = NULL; // malloc(sizeof(float) * FTMEMORY * MAXCHANDATA);
	float* gdt_Y_accumulator = NULL; // malloc(sizeof(float) * FTMEMORY * MAXCHANDATA) ;
	float* gdt_N_accumulator = NULL; // malloc(sizeof(float) * FTMEMORY * MAXCHANDATA);
	float* gdt_amp_accumulator = NULL; // malloc(sizeof(float) * FTMEMORY * MAXCHANDATA) ;
	float* filtered_amp = NULL;
	float* gdt_opds = NULL ; // malloc(sizeof(float) * FTMEMORY * MAXCHANDATA) ;
	//	float parabola_level[MAXCOMBOS/2], parabola_concavity[MAXCOMBOS/2], parabola_delay[MAXCOMBOS/2];
	float estimated_delay[MAXCOMBOS/2];

	float maxamp[MAXCOMBOS/2] ;

	float filter_halfsize = 10; //dummy value to prevent compiler warning
	float filter_exp;
	float* filter_gauss = NULL;

	filtered_amp =  CHAMPrealloc(filtered_amp, MAXCHANDATA * sizeof(float));
	//	int npix_filter;

	err = rt_sem_bind(&switch_sem, "dispsem", TM_INFINITE);
	if ( err )
	  {
	    fprintf(stderr, "semaphore_task: cannot bind semaphore: %s\n", strerror(-err));
	    
	    // free everything
	    free(channelFlux);
	    free(channelLastFlux);
	    free(frametimes);
	    free(ft_x);
	    free(ft_y);
	    free(ft_n);
	    free(ft_num);
	    free(ft_snr);
	    free(ft_pow);
	    free(ft_phi);
	    free(fts_x);
	    free(fts_y);
	    free(fts_n);
	    free(fts_num);
	    free(fts_phi);
	    free(fts_snr);
	    free(fts_pow);
	    free(temp_offsets);
	    free(tmpBuffer);
	    free(tmpBuffer1);
	    free(coaddBuffer);
	    free(totalBuffer);
	    free(bkgndBuffer);
	    free(timing_values);
	    free(channel_opds);
	    free(channel_cospart);
	    free(channel_sinpart);
	    free(channel_x0);
	    free(channel_y0);
	    free(channel_n0);
	    free(abcd_cospart);
	    free(abcd_sinpart);
	    free(abcd_trigon);
	    free(timing_values_all);
	    return;
	  }
	else
	  fprintf(stdout, "semaphore_task: semaphore bound to kernel task\n");

	//config pyconfig with defaults.  will be updated via python in future.
	for ( j = 0; j < NUMTELS ; j++ )
	  pyconfig.BeamOrder[j] = exc.BeamOrder[j];
	
	pyconfig.RefTel = exc.RefTel;
	pyconfig.lambda = exc.lambda;
	pyconfig.bandwidth = exc.bandwidth;
	
	for ( j = 0; j < BASELINES ; j++ )
	  {
	    pyconfig.cpeak[j] = exc.cpeak[j];
	    pyconfig.cpeak0[j] = exc.cpeak0[j];
	    pyconfig.cpeak1[j] = exc.cpeak1[j];
	    pyconfig.gdtThreshold[j] = exc.gdtThreshold[j];
	  }
	pyconfig.bkgndNoiseCol = exc.bkgndNoiseCol;
	
	// Init RT printf 
	//	rt_print_auto_init(1);

	// Main loop
	while (!exc.endProg)
	  {

	    //data acquisition loop until collected framesPerReset
	    
	    // this will measure the execution time of the RT loop
	    previous = now;
	    now = rt_timer_read();
	    /* DHB changes the 1ms sleep to 10 microsec */
	    rt_task_sleep(10000); /* Sleep for 10 microsec */
	    
	    loop_counter++;
	    if ( (loop_counter % 100000) == 0 )
	      {
		loopMsg = TRUE;
		loop_counter = 0; /* restart loop_counter */
	      }
	    else
	      loopMsg = FALSE;
	    
	    /*    // blocking semaphore from interrupt handler */
	    /*     err = rt_sem_p(&switch_sem, timeout); */
	    /*     if (err && (ks == C_RUN)) { */
	    /*       printf("Timeout on data interrupt!!!\n"); */
	    /*       exc.cmd = C_IDLE; */
	    /*     } */
	    
	    /* Get new state from Finite State Machine lookup table */
	    ks_next = FSM[ks][exc.cmd];
	    //if (loopMsg)
	    //printf("Getting new state from FSM: %i, %i, %i", ks_next,ks,exc.cmd);
	    
	    
	  /*-----------------------------------------------------------*/
	  /* State 1: IDLE STATE                                       */
	  /* "Do nothing"                                              */
	  /*-----------------------------------------------------------*/
	  
	  if ( (S_IDLE == ks_next) && (S_IDLE == ks) )
	    {
	      if ( loopMsg )
		printf("IDLE state. Waiting command...\n");
	      sem_count = 0;
	    }
	  
	  if ( (S_IDLE == ks_next) && !(S_IDLE == ks) )
	    {
	      printf("Transition to S_IDLE\n");
	      exc.cmd = C_IDLE;
	    }
	  
	  /*-----------------------------------------------------------*/
	  /* Transition from S_IDLE to S_INIT state                    */
	  /* "Synchronise local vars with sh_mem when idle"            */
	  /* Does major memory reallocation, etc. SLOW                 */
	  /*-----------------------------------------------------------*/
	  
	  if ( (S_INIT == ks_next) && (S_IDLE == ks) )
	    {
	      printf("S_IDLE -> S_INIT state\n");
	      
	      m = 0;
	      frameCount = 1;
	      totalFrameCount = 0;
	      coaddFrame = 0;
	      coaddCount = 0;
	      coherentCount = 1;
	      scanCount = 0;
	      ditherCount = 0;
	      memset(coaddBuffer, 0, daq.rows * daq.cols * sizeof(float));

	      /*
	       * calc how big (in pixel space) each frame is
	       * and round up to next 512 boundary
	       * (1024 in "short" memory space)
	       */
	      num_boundaries = (float) exc.pixelCount / 512;
	      if ( num_boundaries == (num_boundaries_int = (int) num_boundaries) )
		{
		  roundup = 0;
		}
	      else
		{
		  roundup = 1;
		}
	      frameSizePixels = (num_boundaries_int + roundup) * 512;
	      frameSizeShort = frameSizePixels * sizeof(unsigned short);
	      
	      if(VERBOSE == 2)	
		printf("Debug, daq.numberOfFrames %ld \n",  daq.numberOfFrames );
	      
	      /* Will zero out background whenever INIT happens (sensible) */
	      bkgndBuffer = CHAMPrealloc(bkgndBuffer, frameSizePixels * daq.numberOfFrames * sizeof(float));
	      bzero(bkgndBuffer, frameSizePixels * daq.numberOfFrames * sizeof(float));

	      if(BG_ft_num != NULL)
		memset(BG_ft_num, 0, sizeof(double) * MAXCOMBOS / 2);
	      
	      nBkgnd = 0;
	      nBkgnd_total = 0;
	      
	      //	      nDebias = 0;
	      //	      nDebias_total = 0;
	      
	      printf("Reset Background at main INIT\n");
	      
	      printf("exc.pixelCount = %d\t frameSizePixels = %ld\n", exc.pixelCount, frameSizePixels);
	      
	      tmpBuffer = (unsigned short *) realloc(tmpBuffer, frameSizeShort * daq.numberOfFrames);
	      if ( tmpBuffer == NULL )
		{
		  printf("Cannot allocate tmpBuffer \n");
		  exit(1);
		}
	      memset(tmpBuffer, 0, frameSizeShort * daq.numberOfFrames);
	      
	      tmpBuffer1 =  realloc(tmpBuffer1, exc.byte_count_float * daq.numberOfFrames);
	      if ( tmpBuffer1 == NULL )
		{
		  printf("Cannot allocate tmpBuffer1 \n");
		  exit(1);
		}
	      memset(tmpBuffer1, 0, exc.byte_count_float * daq.numberOfFrames);
	      
	      totalBuffer =  realloc(totalBuffer, exc.byte_count_float / daq.nreads * daq.numberOfFrames);
	      if ( totalBuffer == NULL )
		{
		  printf("Cannot allocate totalBuffer \n");
		  exit(1);
		}
	      memset(totalBuffer, 0, exc.byte_count_float / daq.nreads * daq.numberOfFrames);
	      
	      /* Memory allocation/reallocation */
	      channelFlux =  CHAMPrealloc(channelFlux, MAXCOMBOS * daq.numberOfFrames * sizeof(float));	      
	      channelLastFlux =  CHAMPrealloc(channelLastFlux, MAXCOMBOS * daq.numberOfFrames * sizeof(float));
	      flux_history =  CHAMPrealloc(flux_history, MAXCOMBOS * FTMEMORY * sizeof(float));
	      memset(flux_history, 0, MAXCOMBOS * FTMEMORY * sizeof(float));
	      gdt_delay_history = CHAMPrealloc(gdt_delay_history , MAXCOMBOS * FTMEMORY  *sizeof(float) ); // note: MAXCOMBOS/2 * (FTMEMORY * 2), one gdt per scan
	      memset(gdt_delay_history, 0, MAXCOMBOS * FTMEMORY * sizeof(float));
	      ft_num_history  = CHAMPrealloc(ft_num_history,  MAXCOMBOS * FTMEMORY  *sizeof(float) );
	      memset(ft_num_history, 0,  MAXCOMBOS * FTMEMORY  * sizeof(float) );
	      ft_phi_mean_history = CHAMPrealloc(ft_phi_mean_history , MAXCOMBOS / 2 * FTMEMORY  *sizeof(float) );
              memset( ft_phi_mean_history, 0, MAXCOMBOS /2 * FTMEMORY * sizeof(float));
	      ft_phi_stddev_transfer = CHAMPrealloc(ft_phi_stddev_transfer , MAXCOMBOS / 2 * FTMEMORY  *sizeof(float) );
              memset( ft_phi_stddev_transfer, 0, MAXCOMBOS /2 * FTMEMORY * sizeof(float));

	      channel_opds =  CHAMPrealloc(channel_opds, daq.numberOfFrames * MAXCOMBOS * sizeof(float));
	      channel_cospart =  CHAMPrealloc(channel_cospart, daq.numberOfFrames * MAXCOMBOS * sizeof(float));
	      channel_sinpart =  CHAMPrealloc(channel_sinpart, daq.numberOfFrames * MAXCOMBOS * sizeof(float));
	      channel_x0 =  CHAMPrealloc(channel_x0, daq.numberOfFrames * MAXCOMBOS * sizeof(float));
	      channel_y0 =  CHAMPrealloc(channel_y0, daq.numberOfFrames * MAXCOMBOS * sizeof(float));

	   

	      ft_num =  CHAMPrealloc(ft_num, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2);
	      ft_snr =  CHAMPrealloc(ft_snr, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2);
	      ft_pow =  CHAMPrealloc(ft_pow, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2);
	      ft_n =  CHAMPrealloc(ft_n, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2);
	      ft_x =  CHAMPrealloc(ft_x, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2);
	      ft_y =  CHAMPrealloc(ft_y, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2);
	      ft_phi =  CHAMPrealloc(ft_phi, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2);
	      
	      fts_n =  CHAMPrealloc(fts_n, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2 * FTMEMORY);
	      fts_x =  CHAMPrealloc(fts_x, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2 * FTMEMORY);
	      fts_y =  CHAMPrealloc(fts_y, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2 * FTMEMORY);
	      fts_num =  CHAMPrealloc(fts_num, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2 * FTMEMORY);
	      fts_phi =  CHAMPrealloc(fts_phi, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2 * FTMEMORY);
	      fts_snr =  CHAMPrealloc(fts_snr, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2 * FTMEMORY);
	      fts_pow =  CHAMPrealloc(fts_pow, daq.numberOfFrames * MAXCOMBOS * sizeof(double) / 2 * FTMEMORY);
	      
	      channel_n0 =  CHAMPrealloc(channel_n0, daq.numberOfFrames * MAXCOMBOS * sizeof(float));
	      abcd_sinpart =  CHAMPrealloc(abcd_sinpart, daq.numberOfFrames * MAXCOMBOS * sizeof(float));
	      abcd_cospart =  CHAMPrealloc(abcd_cospart, daq.numberOfFrames * MAXCOMBOS * sizeof(float));
	      //abcd_bwspart =  CHAMPrealloc(abcd_bwspart, daq.numberOfFrames * MAXCOMBOS * sizeof(float));
	      abcd_trigon =  CHAMPrealloc(abcd_trigon, 9 * daq.numberOfFrames * MAXCOMBOS * sizeof(float));

	      // GDT
	      gdt_X_accumulator = CHAMPrealloc(gdt_X_accumulator, NINTERLEAVED  * MAXCOMBOS * daq.framesPerReset * sizeof(float) );
	      gdt_Y_accumulator = CHAMPrealloc(gdt_Y_accumulator, NINTERLEAVED  * MAXCOMBOS * daq.framesPerReset * sizeof(float) );
	      gdt_N_accumulator = CHAMPrealloc(gdt_N_accumulator, NINTERLEAVED  * MAXCOMBOS * daq.framesPerReset * sizeof(float) );
	      gdt_amp_accumulator = CHAMPrealloc(gdt_amp_accumulator, NINTERLEAVED  * MAXCOMBOS * daq.framesPerReset * sizeof(float) );
	      gdt_opds = CHAMPrealloc(gdt_opds , MAXCOMBOS / 2 * daq.numberOfFrames  * sizeof(float) );	      
	      gdt_amp_transfer = CHAMPrealloc(gdt_amp_transfer,  MAXCOMBOS / 2 * daq.numberOfFrames  *sizeof(float) );
	      gdt_opds_transfer = CHAMPrealloc(gdt_opds_transfer, MAXCOMBOS / 2 * daq.numberOfFrames  *sizeof(float) );
	      ft_phi_transfer = CHAMPrealloc(ft_phi_transfer, MAXCOMBOS / 2 * daq.numberOfFrames  * sizeof(double) );
	      filtered_amp = CHAMPrealloc(filtered_amp, MAXCOMBOS / 2 * daq.numberOfFrames * sizeof(float) );
	      memset(gdt_amp_transfer , 0, MAXCOMBOS / 2 * daq.numberOfFrames  *sizeof(float) );
	      gdt_filtered_amp_transfer = CHAMPrealloc(gdt_filtered_amp_transfer,  MAXCOMBOS / 2 * daq.numberOfFrames  *sizeof(float) );
	      memset(gdt_filtered_amp_transfer , 0, MAXCOMBOS / 2 * daq.numberOfFrames  *sizeof(float) );
	      ft_offsets_history = CHAMPrealloc( ft_offsets_history, MAXCOMBOS  * FTMEMORY * sizeof(float));
	      memset(ft_offsets_history, 0, MAXCOMBOS * FTMEMORY * sizeof(float));

		// init gdt arrays
 	      memset(gdt_X_accumulator, 0, NINTERLEAVED * MAXCOMBOS * daq.framesPerReset * sizeof(float));
	      memset(gdt_Y_accumulator, 0, NINTERLEAVED * MAXCOMBOS * daq.framesPerReset * sizeof(float));
	      memset(gdt_N_accumulator, 0, NINTERLEAVED * MAXCOMBOS * daq.framesPerReset * sizeof(float));
	      memset(gdt_amp_accumulator, 0, NINTERLEAVED * MAXCOMBOS * daq.framesPerReset * sizeof(float));
	      memset(gdt_opds , 0, MAXCOMBOS / 2 * daq.numberOfFrames  *sizeof(float) );
              memset(gdt_opds_transfer , 0, MAXCOMBOS / 2 * daq.numberOfFrames  *sizeof(float) );
              memset(ft_phi_transfer , 0, MAXCOMBOS / 2 * daq.numberOfFrames  *sizeof(double) );
	      memset(filtered_amp, 0, MAXCOMBOS / 2 * daq.numberOfFrames * sizeof(float));

	      if(VERBOSE == 1)
		printf("Memory reallocation done\n");

	      /* allocate the right amount of memory for pixel indexmap. */
	      memset(npix_channel, 0, sizeof(int) * MAXCOMBOS);
	      memset(channel_delay, 0, sizeof(float) * MAXCOMBOS);
	      
	      gdt_fftsamples = daq.numberOfFrames / 2;

	      // first count  -- for now will keep my original big malloc since memory is cheap!!
	      for ( j = 0; j < daq.rows * daq.cols ; j++ )
		{
		  n = pixelMap[j];
		  if ( n > 0 )
		    {
		      *(channel_pix[n - 1] + npix_channel[n - 1]) = j;
		      channel_delay[n - 1] += j;
		      npix_channel[n - 1]++;
		    }
		}

	      for ( i = 0; i < MAXCOMBOS ; i++ )
		{
		  n = npix_channel[i];
		  if ( n > 0 )
		    {
		      channel_delay[i] = channel_delay[i] / ((float) n) / ((float) (daq.rows * daq.cols));
		    }
		}

	      /* setup array for timing information */
	      timing_values =  CHAMPrealloc(timing_values, daq.numberOfFrames * sizeof(double));
	      timing_values_all = CHAMPrealloc(timing_values_all, ntloops * daq.numberOfFrames * sizeof(RTIME));	   
	      frametimes = CHAMPrealloc(frametimes, daq.numberOfFrames * sizeof(RTIME));
	      frametime = 0;
	      nframetimes = 0;
	      meanframetime = 0;
	      
	    } //  if ((S_INIT == ks_next) && (S_IDLE == ks))
	  
	  
	  /*-----------------------------------------------------------*/
	  /* Transition to RUN STATE                                   */
	  /* "Acquire frames and run GDT"                              */
	  /*-----------------------------------------------------------*/
	  
	  if ( (S_RUN == ks_next) && (S_RUN != ks) )
	    {
	      printf("Transition to S_RUN State\n");
	    }
	  
	  if ( S_RUN == ks )
	    {
	      exc.daqRunning = TRUE;
	    }
	  else
	    {
	      exc.daqRunning = FALSE;
	    }
	  
	  /*-----------------------------------------------------------*/
	  /* State 3: RUN STATE                                        */
	  /* "Acquire frames and run GDT"                              */
	  /* Will stay in this STATE until timeout or done             */
	  /*-----------------------------------------------------------*/
	  
	  // Note -- the user is expected to have done an C_INIT at least once!
	  if ( (S_RUN == ks_next) && (S_RUN == ks) )
	    {
			sem_preexp = rt_timer_read();
			//Baby Init
			sem_count = 0;
			m = 0;
			frameCount = 1;
			totalFrameCount = 0;
			coaddFrame = 0;
			coaddCount = 1;
			coherentCount = 1;
			bzero(totalBuffer, exc.byte_count_float / daq.nreads * daq.numberOfFrames);
			bzero(timing_values, sizeof(double) * daq.numberOfFrames);
			timing_loop = 0;
			memset(totalBuffer, 0, daq.cols * daq.rows * sizeof(float) * daq.numberOfFrames);
			memset(channelFlux, 0, MAXCOMBOS * daq.numberOfFrames * sizeof(float));
			memset(channelLastFlux, 0, MAXCOMBOS * daq.numberOfFrames * sizeof(float));
			memset(channel_x0, 0, MAXCOMBOS * sizeof(float) * daq.numberOfFrames);
			memset(channel_y0, 0, MAXCOMBOS * sizeof(float) * daq.numberOfFrames);
			memset(channel_n0, 0, MAXCOMBOS * sizeof(float) * daq.numberOfFrames);
			memset(abcd_trigon, 0,  9 *  MAXCOMBOS * sizeof(float) * daq.numberOfFrames);

			/* FRINGE TRACKER DEVELOPMENT */

			while (exc.continuousReadout)
			{
				/* First thing must setup timing loops to
				 * 1. get accurate timing information about the readout pattern.
				 * 2. Also, will check against the trigger to make sure there is enough time.
				 * 3. set up the opds and realpart/imagingary part looks up tables
				 * */

				if ( timing_loop == 0 )
				{ /* do a bunch of timing and array setups the first time!! */
				  // OR could have this recalculate if there is a flag flipped by gui
				  printf("Starting timing loop\n"); 
				  t_i = 0;
				  
				  while (t_i < ntloops)
				    {
				      // poll astropci (some latency here.. will need to average
				      astropci_reply_prev = astropci_reply;
				      while (astropci_reply == astropci_reply_prev)
					{
					  rt_task_sleep(1000); // 1 musec
					  ioctl(pci_fd, ASTROPCI_GET_FRAMES_READ, &astropci_reply);
					}
				      timing_values_all[astropci_reply - 1 + t_i * daq.numberOfFrames] = rt_timer_read();
				      if ( astropci_reply == daq.numberOfFrames )
					{
					  t_i++;
					  //	   printf("working on timing loop: %i\n",t_i);
					}
				    }
				  
				  printf("End of timing loop\n"); 
				  
				  // now analyze the values
				  reset0_to_reset1 = 0.0; //ms
				  dither_time = 0.0; //ms
				  dtime = 0.0; //ms
				  for ( t_i = 2; t_i < (ntloops - 1) ; t_i++ )
				    {
						for ( t_j = 0; t_j < daq.numberOfFrames ; t_j++ )
						{
							timing_values[t_j] += (double) (timing_values_all[t_j + t_i * daq.numberOfFrames] 
											- timing_values_all[0 + t_i * daq.numberOfFrames])
							/ ((double) 1000000) / (double) (ntloops - 3);
					
						}
						reset0_to_reset1 += (double) (timing_values_all[daq.framesPerReset + t_i * daq.numberOfFrames]
									      - timing_values_all[0 + t_i * daq.numberOfFrames])
						/ ((double) 1000000) / (double) (ntloops - 3);
						dither_time += (double) (timing_values_all[(t_i + 1) * daq.numberOfFrames] 
				      			 - timing_values_all[0 + t_i * daq.numberOfFrames]) / ((double) 1000000)
						/ (double) (ntloops - 3);

					}

					for ( t_j = 2; t_j < daq.framesPerReset - 1 ; t_j++ )
					{
						dtime += ((timing_values[t_j] - timing_values[t_j - 1]) 
							  + (timing_values[t_j + daq.framesPerReset] - timing_values[t_j - 1 + daq.framesPerReset]))
						          / (2.0 * (daq.framesPerReset - 3.0));
					}

					//	for (t_j=0;t_j<daq.numberOfFrames;t_j++){
					//	printf("frame, time (ms): %i %f \n",t_j,timing_values[t_j]);
					//	}

					printf("reset0 to reset1 (ms): %f \n", reset0_to_reset1);
					printf("dither_time (ms): %f \n", dither_time);
					printf("frametime dtime(ms): %f \n", dtime);
					printf("Number of reset frames: %f \n", (reset0_to_reset1 - daq.framesPerReset * dtime) / dtime);


					timing_loop = 1;

					// OPD MAPS based on linear model, using the ftdata.opdamp and dither_time. 
					// Note: channel_opds are in wavelenght units (i.e. in this code, microns)
					for ( i = 0; i < MAXCOMBOS ; i++ )
					  {

					    for ( j = 0; j < daq.framesPerReset ; j++ )
					      {
						
						channel_opds[j + i * daq.numberOfFrames] =
						  ftdata.opdamp / dither_time * 2. * dtime * ((float) j + channel_delay[i]);
						
						//Alternate opds ramp signs
						if( (MAXCOMBOS / 2) % 2 == 0)
						  channel_opds[j + i * daq.numberOfFrames] *= -1.;
						
						if(DAQ_WAVEFORM == 1)
						  {
						    // OPD map for second ramp of triangle - FB June 2012						    
						    channel_opds[ daq.numberOfFrames -1 - j + i * daq.numberOfFrames] = channel_opds[j + i * daq.numberOfFrames] ;						
						  }
						else
						  {
						// OPD map for second ramp of sawtooth 
						    channel_opds[j + daq.framesPerReset + i * daq.numberOfFrames] =  channel_opds[j + i * daq.numberOfFrames];
						  }
					      }

					    //					    for ( j = 0; j < daq.framesPerReset ; j++ )
					    //					      printf("%ld %ld %f %f\n ", i, j, channel_opds[j + i * daq.numberOfFrames],
					    //						     channel_opds[daq.framesPerReset + j + i * daq.numberOfFrames]); 


					  }
				
					// Substract mean from opd maps                                                                                       
                                        for(n = 0 ; n < 2 * exc.WrapBeam ; n+=2) // 0, 2, 4, 6 for 4 beams                                                   
					  {
					    mean_opds = 0;
					    for ( j = 0; j < daq.numberOfFrames ; j++ )                                                                      
					      mean_opds += channel_opds[j + n * daq.numberOfFrames] + channel_opds[j + (n+1) * daq.numberOfFrames] ; 
					    
					    mean_opds /= (2. * (float)daq.numberOfFrames );
					    printf("Mean opds from opd map, for B%d B%d : %f\n",n,n+1, mean_opds);                                          
					    for ( j = 0; j < daq.numberOfFrames ; j++ )
					      {
						channel_opds[j + n * daq.numberOfFrames] -= mean_opds;
						channel_opds[j + (n+1) * daq.numberOfFrames] -= mean_opds;
					      }
					  }
  
									       
					// Gaussian filter used to determine GDT
					filter_halfsize = daq.framesPerReset / 2 - 1;
					filter_exp = 2. * exc.bandwidth / (exc.lambda * exc.lambda) * ftdata.opdamp / daq.framesPerReset;
					filter_gauss = malloc((2*filter_halfsize+1)*sizeof(float)) ;
					for(i=-filter_halfsize;i<filter_halfsize+1;i++)
					  filter_gauss[i] = exp(-(float)(i*i)*filter_exp);

					for ( i = 0; i < MAXCOMBOS ; i++ )
					  {
					    for ( j = 0; j < daq.numberOfFrames ; j++ )
					      {
						indx1 = j + i * daq.numberOfFrames;
						nwave0 = channel_opds[indx1] / exc.lambda ;
						abcd_cospart[indx1] = cos(2. * PI * nwave0);
						abcd_sinpart[indx1] = sin(2. * PI * nwave0);
						// abcd_bwspart[indx1] = sinc(PI * exc.bandwidth / exc.lambda * nwave0);
						/*
						  if ( ( i == 6 || i == 7 ))
						  printf("combo %ld frame %ld opd %f cospart %f sinpart %f nwave %f \n",
						  i + 1, j, channel_opds[indx1], abcd_cospart[indx1],
						  abcd_sinpart[indx1], nwave0);
						  printf("combo %ld frame %ld opd %f cospart %f sinpart %f bandsmearing %f nwave %f \n", 
						  i + 1, j, channel_opds[indx1], abcd_cospart[indx1],
						  abcd_sinpart[indx1], abcd_bwspart[indx1], nwave0);  */					      
					      }
					  }
				}

				
				/* get the number of frames read from the ASTROPCI card */
				astropci_reply_prev = astropci_reply;
				ioctl(pci_fd, ASTROPCI_GET_FRAMES_READ, &astropci_reply);

				/* which range of memory are we in? */
				memrange = (int) astropci_reply / daq.framesPerReset;
				//		printf("Astropci_reply %d Astropci_reply_prev %d daq.numberOfFrames %d daq.framesPerReset %d\n", 
				//      astropci_reply,  astropci_reply_prev, daq.numberOfFrames , daq.framesPerReset );


				/* report which frame number we're on */
				if ( astropci_reply_prev == astropci_reply )
				  {
				    rt_task_sleep(10000); //10 microseconds
				  }

				// Check to see if the show is over:
				if ( astropci_reply >= (daq.numberOfFrames - daq.framesPerReset + 1) && daq.continuousReadout == FALSE )
				{
					exc.continuousReadout = FALSE;
					//printf("i am here at frame %i\n", astropci_reply);
					printf("Stopping Exposure after next dither...\n");
				}

				/* set the flag to calc offset when done taking frames */
				if ( astropci_reply > 0 )
				{
					if ( (astropci_reply_prev != astropci_reply) )
					{
						calc_offset = 1;
					}
				}
				else
				{
					calc_offset = 0;
				}

				/*begin offset calc if we're ready*/
				if ( calc_offset )
				{
					if ( astropci_reply == daq.numberOfFrames )
					{
						ditherend = 1;
					}
					else
					{
						ditherend = 0;
					}
					if ( (astropci_reply == daq.framesPerReset) || (astropci_reply == daq.numberOfFrames) )
					{
						scanend = 1;
					}
					else
					{
						scanend = 0;
					}

					start_calc_clock = rt_timer_read();

					/* clear the flag to not check again this trigger cycle */
					calc_offset = 0;

					/* copy the data from memory to a user buffer */
					//if (scanCount % scanRepeat)
					// 	printf("copying memory\n");
					// cp since the astropci_reply_prev
					frame0 = astropci_reply_prev;
					frame1 = astropci_reply;
					//					printf("Frame 0: %d Frame 1: %d \n", frame0, frame1);
					if ( frame0 > frame1 )
					{
						if ( frame0 != daq.numberOfFrames )
						{
							printf("Missed %ld frames at end of dither! Frame 0: %d Frame 1: %d daq.numberOfFrames %ld\n",
							       daq.numberOfFrames - frame0, frame0, frame1 ,  daq.numberOfFrames );
						}
						frame0 = 0;
					}

					nframes = frame1 - frame0;
					if ( nframes > 1 )
					  {
					    printf("Skipped a frame! frame0: %i  frame1: %i\n", frame0, frame1);
					    // June 2012 
					    frame0 = frame1-1;
					    nframes = 1;
					    frameskip = 1; // this is only reset at the end of the dither
					  }
					
					mem_fd_offset = pixshift + frameSizePixels * frame0;
					//printf("mem_fd_offset: %i\n", mem_fd_offset);

					memcpy((short *) tmpBuffer, mem_fd + mem_fd_offset, frameSizeShort * nframes);

					/* deinterlace each frame */
					//printf("deinterlacing\n");
					for ( f = 0; f < nframes ; f++ )
					{
					  /* deinterlace */ 
					  if ( deinterlace(daq.cols * daq.nreads, daq.rows, tmpBuffer + f * frameSizePixels) != _NO_ERROR ) 
					    {
					      fprintf(stderr, "ERROR:  deinterlace error\n");
					    }
					}

					/* byte swap the whole buffer*/
					swap_memory(tmpBuffer, frameSizeShort * nframes);
					/*
					 * BACKGROUND
					 */

					/* if the bg counter runs down, set the bg flags to false */
					if ( nBkgnd == 0 )
					{
						exc.takeBkgnd = FALSE;
						startBkgnd = FALSE;
					}

					/* if we're taking bgs and at the start of the memory range */
					if ( exc.takeBkgnd && frame0 == 0 )
					{
						startBkgnd = TRUE;
					}

					/* take a background */
					if ( startBkgnd && (nBkgnd > 0) )
					{
						/* accumulate a background into the bg buffer (floats) */
						for ( f = 0; f < (frameSizePixels * nframes) ; f++ )
						{
							*(bkgndBuffer + f + (frame0) * frameSizePixels) += (float) *(tmpBuffer + f);
						}
						/* decrement the "background" counter */
						if ( frame1 == daq.numberOfFrames )
						{
							nBkgnd_total++;
							printf("Background Counter: %i    Total Number of Background: %i   \n", nBkgnd, nBkgnd_total);

							nBkgnd--;
						}
					}

					/* use or don't use the background */
					/* either way, compactify the buffer and recast to float */
					if ( (exc.useBkgnd == TRUE) && (nBkgnd == 0) &&(nBkgnd_total>0) )
					{
						//	         if ((scanCount % scanRepeat)==0) printf("using background \n");
						/* use the mean background */
						for ( f = 0; f < nframes ; f++ )
						{
							for ( j = 0; j < exc.pixelCount ; j++ )
							{
								*(tmpBuffer1 + exc.pixelCount * f + j) 
								  = (float) *(tmpBuffer + frameSizePixels * f + j) 
								  - ((float) *(bkgndBuffer + frameSizePixels * (frame0 + f)
										+ j) / nBkgnd_total);
							}
						}
					}
					else
					{
						/* DON'T use the backgrounds so just
						 copy the byte buffer into the float buffer */
						//		 if ((scanCount % scanRepeat) ==0) printf("NOT using the background\n");
						for ( f = 0; f < nframes ; f++ )
						{
							for ( j = 0; j < exc.pixelCount ; j++ )
							{
								*(tmpBuffer1 + exc.pixelCount * f + j) = (float) *(tmpBuffer + frameSizePixels * f + j);
							}
						}
					}

					/* combine reads */
					for ( f = 0; f < nframes ; f++ )
					{
						linearFit(tmpBuffer1 + exc.pixelCount * f, totalBuffer + daq.rows * daq.cols 
							  * (frame0 + f), daq.rows, daq.cols, daq.nreads, NUM_SKIP_READS,
								daq.numberCoherent, coherentCount);
					}

					/* chop into beam combos and bin together */
					for ( f = frame0; f < frame1 ; f++ )
					{
					  if ( ( f != 0 ) && (f != daq.framesPerReset) ) // these frames correspond to transients so we skip them
					    {
					      for ( n = 0; n < MAXCOMBOS ; n++ )
						{
						  indx1 = f - 1 + n * daq.numberOfFrames;
						  
						  for ( j = 0; j < npix_channel[n] ; j++ )
						    {
						      b = *(channel_pix[n] + j);
						      channelFlux[indx1] += totalBuffer[b + daq.rows * daq.cols * f] - totalBuffer[b + daq.rows * daq.cols * (f - 1)];
						      
						      if ( (f == (daq.framesPerReset - 1)) || (f == (daq.numberOfFrames - 1) ) ) // avoid display transient by copying value
							{
							  channelFlux[ indx1 + 1 ] +=  totalBuffer[b + daq.rows * daq.cols * (f)]  - totalBuffer[b + daq.rows * daq.cols * (f - 1)];
							}
						      
						    }
						  
						  ttt = 0; sss = 0; // avoids compiler complains
#ifdef FAKE_FRINGES								


						  // Fake fringes
						  //						  if((ditherCount % 300) < 150 )						    
						  ttt = ( channel_opds[indx1] + 1.  - 0.4*n ) / exc.lambda * 2. * PI ; //+  sin( (ditherCount % 50)/50. ) ;
						  //						  else
						  //						    ttt = ( channel_opds[indx1] + 1.3 ) / exc.lambda * 2. * PI  ;


						  sss = ttt ;
						  if( (n % 2) == 1)
						    {
						      sss = PI + sss ;
						    }
						  
						  channelFlux[indx1] = 50. * ( 1.0 + 0.5 * (0.0 + 1.0 * sinc( 0.5 * exc.bandwidth / exc.lambda * ttt )) * cos(sss) ) ;// +  rand() % 200 ;
						  if( f == (daq.framesPerReset-1) )
						     channelFlux[indx1+1] = channelFlux[indx1]; 
#endif

						  invn = 1./(float)npix_fringe[n/2];
						  // accumulate the cospart and sinpart in the correct x,y accumulators.
						  for ( j = 0; j < npix_fringe[n/2] ; j++ )
						    {
						      i = f - 1 + n * daq.numberOfFrames;
						      kprime = f - 1 - j;
						      normalflux = 1;
						      
						      if(kprime < 0 ) // use stored flux from last dither
							{
							  k = kprime + daq.numberOfFrames - 1 + n * daq.numberOfFrames ; 							  
							  channel_x0[i] += abcd_cospart[k] * channelLastFlux[k];
							  channel_y0[i] += abcd_sinpart[k] * channelLastFlux[k];
							  channel_n0[i] += channelLastFlux[k];
							  normalflux = 0;
							}  
						      
						      if( ( (f-1) >=  daq.framesPerReset ) && (kprime < daq.framesPerReset )  )
							{   
							  k = kprime - 1 + n * daq.numberOfFrames ;							  
							  channel_x0[i] += abcd_cospart[k] * channelFlux[k];
							  channel_y0[i] += abcd_sinpart[k] * channelFlux[k];
							  channel_n0[i] += channelFlux[k];
							  normalflux = 0;
							}
						      
						      if(normalflux == 1)
							{
							  k = kprime + n * daq.numberOfFrames ;
							  channel_x0[i] += abcd_cospart[k] * channelFlux[k];
							  channel_y0[i] += abcd_sinpart[k] * channelFlux[k];
							  channel_n0[i] += channelFlux[k];
							}
						      
						      // NEW DEBIAS
						      // the following bias calcs could be done once the npix_fringe matrix is set.
						      //  ( Nm )   (        )  ( N0 )
						      //  ( Xm ) = ( TRIGON )  ( X0 )
						      //  ( Ym )   (        )  ( Y0 )
						      // Nm = dewarped quantities, N0 = warped quantities
						      abcd_trigon[9 * i] = 1.0;
						      abcd_trigon[9 * i + 1] += 2. * invn * abcd_cospart[k];
						      abcd_trigon[9 * i + 2] += 2. * invn * abcd_sinpart[k];
						      abcd_trigon[9 * i + 3] += invn * abcd_cospart[k];
						      abcd_trigon[9 * i + 4] += 2. * invn * abcd_cospart[k] * abcd_cospart[k];
						      abcd_trigon[9 * i + 5] += 2. * invn * abcd_sinpart[k] * abcd_cospart[k];
						      abcd_trigon[9 * i + 6] += invn * abcd_sinpart[k];
						      abcd_trigon[9 * i + 7] += 2. * invn * abcd_sinpart[k] * abcd_cospart[k];
						      abcd_trigon[9 * i + 8] += 2. * invn * abcd_sinpart[k] * abcd_sinpart[k];
						    }
						  
						}
					    }
					}
					
					if(VERBOSE == 2)  printf("Done precalc\n");
					/*
					 * CALCULATE OFFSET
					 */
					
					if ( frame1 > 1 )
					{
					  for ( n = 0; n < MAXCOMBOS / 2 ; n++ )
					    { // For all 6 combiner outputs
					      
					      // Dewarp the fluxes for each combiner output
					      indx1 = frame1 - 2 + 2 * n * daq.numberOfFrames;
					      indx2 = frame1 - 2 + (2 * n + 1) * daq.numberOfFrames;
					      indx3 = frame1 - 2 + n * daq.numberOfFrames;
					      
					      // New dewarp code - FB
					      dewarp(abcd_trigon, indx1, channel_n0, channel_x0, channel_y0, &ft_n1, &ft_x1, &ft_y1);
					      dewarp(abcd_trigon, indx2, channel_n0, channel_x0, channel_y0, &ft_n2, &ft_x2, &ft_y2);
					      
					      // Combine the dewarped values obtained for both outputs of each combiner
					      ft_n[indx3] = beams2combo[2 * n] * ft_n1 - beams2combo[2 * n + 1] * ft_n2;
					      ft_x[indx3] = beams2combo[2 * n] * ft_x1 + beams2combo[2 * n + 1] * ft_x2;
					      ft_y[indx3] = beams2combo[2 * n] * ft_y1 + beams2combo[2 * n + 1] * ft_y2;
					      
					      if ( frame1 == (daq .framesPerReset + 1)  ) // TBD: improve this
						{
						  ft_n[indx3] = ft_n[indx3 - 1];
						  ft_x[indx3] = ft_x[indx3 - 1];
						  ft_y[indx3] = ft_y[indx3 - 1];
						}

					      // Now compute the derived quantities: squared amplitude, powerspectrum, snr and phase
					      
					      //  SQUARED AMPLITUDE
					      ft_num[indx3] = ft_x[indx3] * ft_x[indx3] + ft_y[indx3] * ft_y[indx3];
					      

					      // Very simple bias correction for ft_num based
					      // Does this require the gain and read noise ?							
					      // FB: use the same number of debias samples as the backgrounds
					      
					      // Correct for background num
					      //							if ( use_Debias == TRUE )
					      //							  ft_num[indx3] -= BG_ft_num[n]  ; // note: at this point num can be negative
					      

					      // SNR
					      if ( ( ft_n[indx3] > 0. ) && ( ft_num[indx3] > 0. ) )
						ft_snr[indx3] = 2. * sqrt( ft_num[indx3] / ( ft_n[indx3] * ft_n[indx3] ) );
					      else
						ft_snr[indx3] = 0.; //if num or snr are < 0, do not trust
					      
					      // POWERSPECTRUM estimate (warning, may be < 0)
					      if ( fabs(ft_n[indx3]) > 0. )
						ft_pow[indx3] = 4. * ft_num[indx3] / ( ft_n[indx3] * ft_n[indx3] );
					      else
						ft_pow[indx3] = 0.; // pow estimator still works at low snr except if div by 0
					      

					      //
					      // Running mean and average of the amplitude 
					      //
					      
					      // reset fringe amp for amp average over scan
					      
					      // if ( ( frame1 == 2 ) || ( frame1 == ( daq.framesPerReset + 1) ) ) // reset 
						//	 {
						   // fringe_amp_mean[ n ] = 0.;
						   // fringe_amp_mean_old[n] = 0;
						   //fringe_amp_var_old[ n ] = 0. ;
						   //  fringe_mean_counter = 0;
						   //	 }
					      /// else 
						//	{
						  // average fringe amplitude over daq.frameperreset or daq.frameperreset -1 
						  // depending on the scan
						  
						  //if(frame1 >= (daq.framesPerReset + 1))
						  //  fringe_amp_mean[ n ] +=  ft_num[ indx3 ] / (float)( daq.framesPerReset );
						  // else
						  //  fringe_amp_mean[ n ] +=  ft_num[ indx3 ] / (float)( daq.framesPerReset - 1 );
						  
						  // Now doing cleaner running average & variance
						  
						  
						  /*
						  fringe_mean_counter ++;
						  fringe_amp_mean[n] = fringe_amp_mean_old[n] + (ft_num[indx3] - fringe_amp_mean_old[n]) / (float)fringe_mean_counter;
						  fringe_amp_var[n]  = fringe_amp_var_old[n]  + (ft_num[indx3] - fringe_amp_mean_old[n])*(ft_num[indx3] - fringe_amp_mean[n]);						   
						  fringe_amp_mean_old[n] = fringe_amp_mean[n]; 
						  fringe_amp_var_old[n] = fringe_amp_var[n];
						  if(fringe_mean_counter > 1)
						    fringe_amp_stddev[n] = sqrt(fringe_amp_var[n]/((float)fringe_mean_counter-1.0));
						  else
						    fringe_amp_stddev[n] = 0; 
						  */

						  //	}

					      //
					      // PHASE computation
					      //
					      
					      ft_phi[indx3] = atan2(ft_y[indx3], ft_x[indx3]);

					      // update all histories
					      fts_x[indx3 + ( ditherCount % FTMEMORY) * daq.numberOfFrames * MAXCOMBOS / 2] = ft_x[indx3];
					      fts_y[indx3 + ( ditherCount % FTMEMORY) * daq.numberOfFrames * MAXCOMBOS / 2] = ft_y[indx3];
					      fts_n[indx3 + ( ditherCount % FTMEMORY) * daq.numberOfFrames * MAXCOMBOS / 2] = ft_n[indx3];
					      
					      fts_phi[indx3 + ( ditherCount % FTMEMORY) * daq.numberOfFrames * MAXCOMBOS / 2] = ft_phi[indx3];
					      fts_num[indx3 + ( ditherCount % FTMEMORY) * daq.numberOfFrames * MAXCOMBOS / 2] = ft_num[indx3];
					      fts_pow[indx3 + ( ditherCount % FTMEMORY) * daq.numberOfFrames * MAXCOMBOS / 2] = ft_pow[indx3];
					      fts_snr[indx3 + ( ditherCount % FTMEMORY) * daq.numberOfFrames * MAXCOMBOS / 2] = ft_snr[indx3];


					      /*						       
					      if ( ( (scanCount % 1000) == 0 ) || ( (scanCount % 1000) == 1 ) )
					      {
						if ( n == 0 )
						  {
						    printf("scanCount %i combo %i indx3 %i phase %5.3f num %5.3f amp_mean %5.3f snr %5.3f visamp %5.3f N %5.3f X %5.3f Y %5.3f F0 %d F1 %d\n",
								     scanCount , n, indx3, ft_phi[indx3], ft_num[indx3], fringe_amp_mean[n], ft_snr[indx3],
							   sqrt(ft_pow[indx3]), ft_n[indx3], ft_x[indx3], ft_y[indx3], frame0, frame1);
						  }
					      }
					      */
					      
					    } //combos
					  
					} // frame1 > 0 // FB: remove this if working

					// printf("scanCount %i ", scanCount);
					if(VERBOSE == 2) printf("Done phase estimation\n");


					for ( n = 0; n < MAXCOMBOS/2 ; n++ )
					  { // combos
					    
					    
					    //					    if ( (frame1 > npix_fringe[n] && frame1 <= daq.framesPerReset) 
					    //						 || (frame1 > (npix_fringe[n] + daq.framesPerReset) && frame1 <= daq.numberOfFrames) )
					    if( ( frame1 != (daq .framesPerReset+1) ) && ( frame1 != (daq .numberOfFrames+1) ) )
					    {
						frame_valid = 1; // second half of a scan
					      }
					    else
					      frame_valid = 0;
					    
					    // note: frame_valid can be used to slow down the number of offsets actually used, rather than
					    // sending vme a command every time or only fringe tracking on CERTAIN frames... based on ft_num, etc.

					    if ( frame_valid == 1  )
					      {
						// Add different cases here..
						// for the moment SEMI-LOCK does not send cophasing offsets
						if (BC_FT_State[n] == BC_FT_LOCK) 
						  {
						    // Kalman Filter should go here.. based on records.
						    BC_FT_Stage[n] = BC_FT_Servo_Gains[n] * ft_phi[frame1 - 2 + n * daq.numberOfFrames]  / (2. * PI) * exc.lambda;

						    BC_FT_Stage_Weight[n] =  ft_num[frame1 - 2 + n * daq.numberOfFrames] / BC_Amp_Threshold[n]; // NEEDS TO BE FIXED, NOT USING BC_Amp_Threshold[n] any more
						    //  if( (ditherCount % 30) == 1)  printf("STATE LOCK: scanCount %i n %i BC_FT_Servo_Gain %f ft_phi %f BC_FT_Stage_Weight %f ft_num %f\n", scanCount, n, 
						    //  BC_FT_Servo_Gains[n], ft_phi[frame1 - 2 + n * daq.numberOfFrames], BC_FT_Stage_Weight[n], ft_num[frame1 - 2 + n * daq.numberOfFrames]); 
						    vme_active=1;
						  } 
						// after this combo loop, we must run champ_bs and move results to BC_FT_Offset
						
						if ( (frame1 == (int) (npix_fringe[n] + daq.framesPerReset / 2) ) && ((ditherCount % 30)==1) ) 
						  {
						    //calc stats
						    ft_phi_stddev[n] = 0;
						    ft_phi_mean[n] = 0;
						    for(i=0; i < 5; i++)
						      {
							ft_phi_mean[n] += 
							  fts_phi[frame1 - 2 + n * daq.numberOfFrames + ( (ditherCount - i) % FTMEMORY) * daq.numberOfFrames * MAXCOMBOS / 2]; 
						      }
						    
						    ft_phi_mean[n] /= 5;

						    for(i=0; i < 5; i++)
                                                      {
							ft_phi_stddev[n] += (fts_phi[frame1 - 2 + n * daq.numberOfFrames
									 + ( (ditherCount - i) % FTMEMORY) * daq.numberOfFrames * MAXCOMBOS / 2] - ft_phi_mean[n])
							  * (fts_phi[frame1 - 2 + n * daq.numberOfFrames
								     + ( (ditherCount - i) % FTMEMORY) * daq.numberOfFrames * MAXCOMBOS / 2] - ft_phi_mean[n]);
						      }

						    ft_phi_stddev[n] = sqrt(ft_phi_stddev[n] / 4.) ;
						    						    
						    // Store values for the GUI
						    ft_phi_stddev_transfer [ ( (ditherCount / 5) % FTMEMORY ) * MAXCOMBOS / 2 + n ] = ft_phi_stddev[ n ];
						    ft_phi_mean_history[ ( (ditherCount / 5) % FTMEMORY ) * MAXCOMBOS / 2 + n ] =  ft_phi_mean[n];

						  } // calc stat condition
						
					      }//valid frame
					  } // combos
					

					if(VERBOSE == 2) printf("Computing staging for FT\n");
					
					if(exc.WrapBeam == 4)
					  champ_bs4(BC_FT_Stage_Weight, BC_FT_Stage, BC_FT_Stage1, temp_dl_offsets);
					if(exc.WrapBeam == 6)
					  champ_bs6(BC_FT_Stage_Weight, BC_FT_Stage, BC_FT_Stage1, temp_dl_offsets);

					// Store/Reset Stage offsets

					for ( n = 0; n < exc.WrapBeam ; n++ )
					  {
					    BC_FT_Offset[n] = BC_FT_Offset[n] + BC_FT_Stage1[n];
					    BC_FT_Stage[n] = 0.0; 
					    BC_FT_Stage_Weight[n] = 0.0;
					  }
					
				
					////////////////////////////////////////////////
					//
					//               FILE SAVING / SPOOLER
					//
					////////////////////////////////////////////////

					//  BUFFERING

					if( (nsaved == 10000) && (ditherend == 1 ) )
					  {
					    sprintf(outputfile, "ftdata_%06i.dat", scanCount);
					    printf("outputfile: %s\n", outputfile);
					    fp_temp=fopen(outputfile,"w");
					    
					    for( f = 0 ;  f< daq.numberOfFrames; f++)
					      {
						for ( i = 0 ; i < daq.rows * daq.cols ; i++)
						{
						  fprintf(fp_temp,"%lf ", totalBuffer[f * daq.rows * daq.cols + i]);
						}
						fprintf(fp_temp,"\n");
					      }
					    fclose(fp_temp);
					    nsaved = 0;
					  }

					if( (nsaved > 0) && (save_step == 1) && (frame1 == 2)) // force start at the beginning of a scan
					  {
					    save_step = 2;
					  }

					// TBD: loop from frame0 to frame 1, would require computation for frame0 above
					if ( (nsaved > 0) && (dithersleft > 0 ) && (frame1 > 1) && (save_step == 2) )
					  {
					    for ( n = 0; n < MAXCOMBOS / 2 ; n++ )
					      {
						j = n * daq.numberOfFrames + frame1 - 2;						
						saveddata.ditherCount[j] = ditherCount;
						saveddata.frame1[j] = frame1 - 2;
						saveddata.time[j] = (double) ((long double)rt_timer_ticks2ns(rt_timer_read()) - (long double)(1.306E18L)); 
						//						//						gettimeofday( &tp, NULL);
						//						clock_gettime(CLOCK_REALTIME, &tp);
						//						saveddata.time[j] = (double)(tp.tv_sec + tp.tv_nsec/1000000);
						//  saveddata.combo[j] = n;
						saveddata.state[j] = BC_FT_State[n];
						saveddata.ftoffsets[j] = BC_FT_Offset[n];
						saveddata.refoffsets[j] = BC_FT_Offset[n];
					      }
					  }
					
					// HEADER (could be put at the end of the dither)
					if ( (nsaved > 0) && write_header && (memrange == 1) ) 
					{

						/* Send header info to the spooler process */
						message.type = HEADER;
						message.length = sizeof(header);

						/* assemble structures into the header struct */
						header.savesPerFile = daq.savesPerFile;
						header.combos = MAXCOMBOS;
						header.rows = daq.rows;
						header.cols = daq.cols;
						header.rowOffset = daq.rowOffset;
						header.colOffset = daq.colOffset;
						header.nreads = daq.nreads;
						header.numberOfCoadds = daq.numberOfCoadds;
						header.framesPerReset = daq.framesPerReset;
						header.exptime = daq.exptime;
						header.numberOfFrames = daq.numberOfFrames;
						header.do_image_save = daq.do_image_save;
						header.voffset = daq.voffset;
						header.vreset = daq.voffset;
						header.vd = daq.vd;
						header.ictl = daq.ictl;
						header.tint = t_int;
						for ( loop = 0; loop < NUMDLS ; loop++ )
							header.dloffsets[loop] = dloffsets[loop];

						memcpy((pythonconfig_struct *) &header.pyconfig, &pyconfig, sizeof(pyconfig));
						memcpy((gdt_struct *) &header.gdt, &gdt, sizeof(gdt));

						pipe_err = 0; 
						pipe_err += rt_pipe_write(&pipe_desc, &message.type, sizeof(message.type), P_NORMAL);
						pipe_err += rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
						pipe_err += rt_pipe_write(&pipe_desc, (unsigned char *) &header, message.length, P_NORMAL);

						if ( pipe_err != (sizeof(message.type) + sizeof(message.length) + (ssize_t) message.length) )
						  fprintf(stderr, "Error writing header for %d\n", nsaved);

						/* flip the header flag */
						write_header = FALSE;
						write_footer = FALSE;
						printf("Generated header for pipe, nsaved = %d\n", nsaved);
						dithersleft = daq.savesPerFile; // number of dithers to add to file
						save_step = 1; 
					}


					// DATA -- FB: all the following code could really make use of a function
					if ( (nsaved > 0) && !write_header && (ditherend == 1 ) && (save_step == 2) ) // use the scan end to send data to pipe
					{
					  // printf("Saving data at dithercount %d\n", ditherCount );
					  message.type = DATA;
					  pipe_err = rt_pipe_write(&pipe_desc, &message.type, sizeof(message.type), P_NORMAL);
					  if ( pipe_err != sizeof(message.type) )
                                              fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA type\n");

					  // Frame number
					  message.length = MAXCOMBOS / 2 * daq.numberOfFrames * ( sizeof(unsigned char) );
					  pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
					  if ( pipe_err != sizeof(message.length) )
                                              fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length Frame\n");
					  pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (saveddata.frame1), message.length , P_NORMAL);
					  if ( pipe_err < 0 )
					    {
					      fprintf(stderr, "ERROR: RT_PIPE_ERR when sending Frame\n");
					      printf("%zi\n", pipe_err);
					    }					  

					 			  
					  // Time
					  message.length = MAXCOMBOS / 2 * daq.numberOfFrames * ( sizeof(double) );
					  pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
					    fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length  Time\n");
					  pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (saveddata.time),  message.length , P_NORMAL);
					  if ( pipe_err < 0 )
					    {
					      fprintf(stderr, "ERROR: RT_PIPE_ERR Time\n");
					      printf("%zi\n", pipe_err);
					    }

					  
					  // Channel OPD
					  message.length = MAXCOMBOS * daq.numberOfFrames * ( sizeof(float) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length Channel OPD\n");
					  pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (channel_opds), message.length, P_NORMAL);
					  if ( pipe_err < 0 )
					    {
					      fprintf(stderr, "ERROR: RT_PIPE_ERR Channel opd\n");
					      printf("%zi\n", pipe_err);
					    }


					  // CHANNEL FLUX
					  message.length = MAXCOMBOS * daq.numberOfFrames * ( sizeof(float) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length CHANNEL FLUX\n");
					  pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (channelFlux), message.length, P_NORMAL);
					  if ( pipe_err < 0 )
					    {
					      fprintf(stderr, "ERROR: RT_PIPE_ERR Channel flux \n");
					      printf("%zi\n", pipe_err);
					    }

					  
					  // STATE
					  message.length = MAXCOMBOS / 2 * daq.numberOfFrames * ( sizeof(unsigned char) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
					  if ( pipe_err != sizeof(message.length) )
					    fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length STATE\n");
					  pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (saveddata.state), message.length, P_NORMAL);
                                          if ( pipe_err < 0 )
					    {
					      fprintf(stderr, "ERROR: RT_PIPE_ERR BC State \n");
					      printf("%zi\n", pipe_err);
					    }

					  // FTOFFSET
					  message.length = MAXCOMBOS / 2 * daq.numberOfFrames * ( sizeof(double) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
					    fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length FTOFFSET\n");
                                          pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (saveddata.ftoffsets), message.length , P_NORMAL);
                                          if ( pipe_err < 0 )                      
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR BC FT Offset\n");

					  // REFOFFSETS
					  message.length = MAXCOMBOS / 2 * daq.numberOfFrames * ( sizeof(double) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
					    fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length\n");
                                          pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (saveddata.refoffsets), message.length , P_NORMAL);
                                          if ( pipe_err < 0 )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR BC Ref Offset \n");
					  
					  // FT X
					  message.length = MAXCOMBOS / 2 * daq.numberOfFrames * ( sizeof(double) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length \n");
                                          pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (ft_x),  message.length , P_NORMAL);
                                          if ( pipe_err < 0 )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR FT Phi \n");

					  // FT Y
					  message.length = MAXCOMBOS / 2 * daq.numberOfFrames * ( sizeof(double) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length \n");
                                          pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (ft_y),  message.length , P_NORMAL);
                                          if ( pipe_err < 0 )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR FT Phi \n");

					  // FT N
					  message.length = MAXCOMBOS / 2 * daq.numberOfFrames * ( sizeof(double) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length \n");
                                          pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (ft_n),  message.length , P_NORMAL);
                                          if ( pipe_err < 0 )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR FT Phi \n");
					  
					  
					  // FT PHI
					  message.length = MAXCOMBOS / 2 * daq.numberOfFrames * ( sizeof(double) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length \n");
                                          pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (ft_phi),  message.length , P_NORMAL);
                                          if ( pipe_err < 0 )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR FT Phi \n");

					  // FT NUM
					  message.length = MAXCOMBOS / 2 * daq.numberOfFrames * ( sizeof(double) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length \n");
                                          pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (ft_num),  message.length , P_NORMAL);
                                          if ( pipe_err < 0 )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR FT Num \n");

					  // GDT DELAY
					  message.length = MAXCOMBOS / 2 * ( sizeof(float) );
                                          pipe_err = rt_pipe_write(&pipe_desc, &message.length, sizeof(message.length), P_NORMAL);
                                          if ( pipe_err != sizeof(message.length) )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR when sending DATA length GDT DELAY \n");
                                          pipe_err = rt_pipe_write(&pipe_desc, (unsigned char *) (&estimated_delay[0]),  message.length , P_NORMAL);
                                          if ( pipe_err < 0 )
                                            fprintf(stderr, "ERROR: RT_PIPE_ERR GDT DELAY\n");
					  
					  
					    dithersleft--;

					  if(dithersleft == 0)
					    write_footer = TRUE;
					  else 
					    write_footer = FALSE;

					} // end if (writing data to the pipe)
					
					
					// FOOTER
				
					if ( write_footer & (nsaved > 0) )
					  {
					    // Send END command to the spooler process 
					    
					    message.type = END;
					    pipe_err = rt_pipe_write(&pipe_desc, &message.type, sizeof(message.type), P_NORMAL);
					    
					    if ( pipe_err != sizeof(message.type) )
					      fprintf(stderr, "ERROR: RT_PIPE_ERR FT Num\n");

					    // decrement save counter 
					    nsaved--;
				    
					    // flip the header/footer flags 
					    write_header = TRUE;
					    write_footer = FALSE;
					    printf("Wrote the footer for nsaved = %d\n", nsaved + 1);

					    save_step = 1 ;
					  }
				
				
					////////////////////////////////////
					//
					//           END OF SCAN
					//
					////////////////////////////////////

					if ( scanend == 1 )
					  {

					    if(VERBOSE == 2) printf("End of scan\n");
					    // test of scanning

					    scanCount++;
					    autoThresholdCounter++;

					    
					    // Start of classic scanning


					    searchFlag = 0;
					    for ( n = 0; n < exc.WrapBeam ; n++ )
					      {
						// Collect fringe information for GUI "calc threshold"						
						// 3 sigma detection ?
						autoThreshold[n][autoThresholdCounter % AUTONUM ] = maxamp[n] ;
						//used to be fringe_amp_mean[n];
						//						if(n==0)// && ((scanend %10)==0))
						//						  printf("M:%f S:%f Max:%f\n",  fringe_amp_mean[n], fringe_amp_stddev[n], maxamp[n]);

						// This threshold can be used by the user as BC_Amp_Threshold
						if ( maxamp[n] > (BC_Amp_Threshold_Mean[n] + BC_Amp_Threshold_RMS[n] * BC_Sweep_SNR_Threshold[n]) ) // fringe detected
						  {
						    BC_Sweep_LastKnownPositions[n] = BC_FT_Offset[n];
						  }

						if  (BC_FT_State[n] != BC_FT_SWEEP)
						  {

						    /////////////////////////////////////////////////////////
						    //
						    // First, if FT is searching
						    //
						    /////////////////////////////////////////////////////////
						    
						    if ( BC_FT_State[n] == BC_FT_SEARCH )  // fringe found, going from search to semilock
						      {
							if ( maxamp[n] > (BC_Amp_Threshold_Mean[n] + BC_Amp_Threshold_RMS[n] * BC_FT_StoSL_SNR_Threshold[n] ) )
							  {
							    if( BC_FT_StoSL_Counter[n] >= BC_FT_StoSL_Transition[n])  
							      { 
								BC_FT_State_Last[n]= BC_FT_State[n];
								BC_FT_State[n] = BC_FT_SEMILOCK;
								
								printf("**************************************************\n");
								printf("FRINGE FOUND in COMBO %i scan %i AMP %f\n", n, scanCount, maxamp[n]);
								printf("vme FT Offset (mu): %f \n", BC_Ref_Offset[n] + BC_FT_Offset[n]);
								printf("**************************************************\n");
								printf("State = %d StoSL %d SLtoL %d LtoSL %d SLtoS %d \n",
								       BC_FT_State[n],  BC_FT_StoSL_Counter[n] , 
								       BC_FT_SLtoL_Counter[n], BC_FT_LtoSL_Counter[n], BC_FT_SLtoL_Counter[n] );
								// BC_FT_StoSL_Counter[n] = 0; // FB: new
							      }
							    else
							      {
								BC_FT_StoSL_Counter[n]++;
							      }
							  }
							else
							  {
							    BC_FT_StoSL_Counter[n] = 0;
							  }
							
						      }
						    
						    // if searching, then increment.  only linear search now.
						    if ( BC_FT_State[n] == BC_FT_SEARCH )
						    {
						      searchFlag = 1; // set if at least one BC is searching (so as to send another vme command)
						      // Linear search
						      if ( BC_Search_Mode[n] == BC_LINEAR )
							{
							  BC_FT_Stage[n] = BC_Search_Speed[n] * (float)daq.framesPerReset * dtime / 1000.;
							  BC_FT_Stage_Weight[n] = 10.;
							  vme_active=1;
							  if ( ( (ditherCount % 30) == 0 ) && (ditherend ==1))
							  {
							    printf("Current Searching combo %i stepsize %f %f %f %f \n", 
								   n , BC_FT_Stage[n] , BC_Search_Speed[n] , dtime , (float) daq.framesPerReset );
							  }
							}
						      
						      // Spiral search 
						      if ( BC_Search_Mode[n] == BC_SPIRAL )
							{
							  if (BC_FT_State_Last[n] == BC_FT_OFF)
							    {
							      BC_FT_State_Last[n] = BC_FT_SEARCH;
							      BC_Spiral_Edge[n] = BC_Spiral_Step[n];
							      BC_Spiral_Position[n] = 0 ;							      
							      printf("Starting Spiral search Step %f Speed %f \n", BC_Spiral_Step[n],  BC_Search_Speed[n]);
							    }
							  
							  BC_FT_Stage[n] = ( BC_Spiral_Edge[n] > 0. ? 1.: -1. ) 
							    * BC_Search_Speed[n] * (float) daq.framesPerReset * dtime / 1000. ;
							  BC_Spiral_Position[n] += BC_FT_Stage[n] ;
							  BC_FT_Stage_Weight[n] = 10. ;
							  
							  if( fabs( BC_Spiral_Position[n] ) > fabs( BC_Spiral_Edge[n] ) )
							    {
							      BC_Spiral_Edge[n] *= - 2.0 ;
							      printf("Spiral search Position %f Edge %f Step %f Speed %f \n",
								     BC_Spiral_Position[n],  BC_Spiral_Edge[n], BC_Spiral_Step[n],  BC_Search_Speed[n]);
							    }
							}
						    }
					      
						///////////////////////////////////////////////////////////
						//
						// FT in semi-lock or lock
						//
						///////////////////////////////////////////////////////////
						
						
						    if ( (BC_FT_State[n] == BC_FT_LOCK) || (BC_FT_State[n] == BC_FT_SEMILOCK)  || (BC_FT_State[n] == BC_FT_FREEZE))
						      {
							
							BC_FT_Stage[n] = 0.0; //Stay in current position --  do not get messed up by searching                                                                                       
							BC_FT_Stage_Weight[n] = 1000.;
						      }
						    
						    if (BC_FT_State[n] == BC_FT_OFF)
						      {
							BC_FT_Stage[n] = 0.0; 
							BC_FT_Stage_Weight[n] = 0.;
						      }
						    
						    if ( BC_PADDLE_State == 1) 
						      {
							BC_FT_Stage[BC_PADDLE_Index] = BC_PADDLE_Offset;
							BC_FT_Stage_Weight[BC_PADDLE_Index] = 1000.;
							BC_PADDLE_State = 0;
							vme_active=1;
						      }



						if ( BC_FT_State[n] == BC_FT_SEMILOCK ) // going from semi-lock to search                                                   
                                                  {
                                                    if ( maxamp[n] < (BC_Amp_Threshold_Mean[n] + BC_Amp_Threshold_RMS[n] * BC_FT_SLtoS_SNR_Threshold[n]) )
                                                      {
							gdt_above_threshold[n] = FALSE;
							if( BC_FT_SLtoS_Counter[n] > BC_FT_SLtoS_Transition[n])
                                                          {
							    BC_FT_State_Last[n]= BC_FT_State[n];
							    BC_FT_State[n] = BC_FT_OFF; // Temporary for finding fringes initially and testing algorithms                       
							    
							    printf("**************************************************\n");
							    printf("LOST FRINGE in COMBO %i scan %i AMP %f\n", n, scanCount, maxamp[n]);
							    printf("GOING TO OFF \n");
							    printf("vme FT Offset (mu): %f \n", BC_Ref_Offset[n] + BC_FT_Offset[n]);
							    printf("**************************************************\n");
							    printf("State = %d StoSL %d SLtoL %d LtoSL %d SLtoS %d \n",
								   BC_FT_State[n],  BC_FT_StoSL_Counter[n] , 
								   BC_FT_SLtoL_Counter[n], BC_FT_LtoSL_Counter[n], BC_FT_SLtoS_Counter[n] );

							    BC_FT_SLtoS_Counter[n] = 0;
							    gdt_update_dl = FALSE; // stop acting on DL with GDT
							  }
							else
							  {
							    BC_FT_SLtoS_Counter[n] ++;
							    
							  }
						      }
						    else
						      {
							gdt_above_threshold[n] = TRUE;
							BC_FT_SLtoS_Counter[n] = 0; 
						      }
						    
                                                  }

						
						if ( BC_FT_State[n] == BC_FT_LOCK ) 
						  {
						   
						    if ( maxamp[n] < (BC_Amp_Threshold_Mean[n] + BC_Amp_Threshold_RMS[n] * BC_FT_LtoSL_SNR_Threshold[n]) )
						      {
							
							//phasetracking_above_threshold = FALSE;
							if( BC_FT_LtoSL_Counter[n] > BC_FT_LtoSL_Transition[n])
                                                          {
							    BC_FT_State_Last[n]= BC_FT_State[n];
							    BC_FT_State[n] = BC_FT_SEMILOCK;
							    
							    printf("**************************************************\n");
							    printf("UNLOCKING FRINGE in COMBO %i scan %i AMP %f\n", n, scanCount, maxamp[n]);
							    printf("vme FT Offset (mu): %f \n", BC_Ref_Offset[n] + BC_FT_Offset[n]);
							    printf("**************************************************\n");
							    printf("State = %d StoSL %d SLtoL %d LtoSL %d SLtoS %d \n",
								   BC_FT_State[n],  BC_FT_StoSL_Counter[n] ,
								   BC_FT_SLtoL_Counter[n], BC_FT_LtoSL_Counter[n], BC_FT_SLtoS_Counter[n] );
							    BC_FT_LtoSL_Counter[n] = 0;
							    // reset the jump accumulator
							    gdt_jump_accumulator[n] = 0;
							    // deactivate COPHASING
							    //							    COPHASING = FALSE;

							  }
							else
							  {
							    BC_FT_LtoSL_Counter[n]++;
							  }
						      }
						    else
						      {
							//phasetracking_above_threshold = TRUE;
							BC_FT_LtoSL_Counter[n] = 0;
						      }

						  }
						
						if ( BC_FT_State[n] == BC_FT_SEMILOCK ) // going from semi-lock to lock
						  {
						    if ( maxamp[n] > (BC_Amp_Threshold_Mean[n] + BC_Amp_Threshold_RMS[n] * BC_FT_SLtoL_SNR_Threshold[n]) )
						      {
							if( BC_FT_SLtoL_Counter[n] > BC_FT_SLtoL_Transition[n])
                                                          {
							    BC_FT_State_Last[n]= BC_FT_State[n];
							    BC_FT_State[n] = BC_FT_LOCK;
							    
							    printf("**************************************************\n");
							    printf("LOCKING FRINGE in COMBO %i scan %i AMP %f \n", n, scanCount, maxamp[n]);
							    printf("vme FT Offset (mu): %f \n", BC_Ref_Offset[n] + BC_FT_Offset[n]);
							    printf("**************************************************\n");
							    printf("State = %d StoSL %d SLtoL %d LtoSL %d SLtoS %d \n",
								   BC_FT_State[n],  BC_FT_StoSL_Counter[n] ,
								   BC_FT_SLtoL_Counter[n], BC_FT_LtoSL_Counter[n], BC_FT_SLtoS_Counter[n] );
							    BC_FT_SLtoL_Counter[n] = 0;
							    // Activate COPHASING ? -- TBD, check how this affects accumulators
							    //							    COPHASING = TRUE;
							  }
							else
							  {
							    BC_FT_SLtoL_Counter[n]++;
							  }
						      }
						    else
						      {
							BC_FT_SLtoL_Counter[n] = 0;
						      }

						  }

					      } // END OF NON-SWEEPING MODES
					    else // BC_FT_State[n] == BC_FT_SWEEP
					      {
						// SWEEP MODE
						searchFlag = 1;
						// Search part
						if (BC_FT_State_Last[n] != BC_FT_SWEEP) // if starting from scratch
						  {	
						    BC_FT_State_Last[n] = BC_FT_SWEEP;
						    BC_Sweep_Direction[n]=1;
						  }
						      //BC_Sweep_LastKnownPositions[n * FTMEMORY + BC_Sweep_LastKnownIndex[n] ];  // last position
						      //+ BC_Sweep_LongTermA[n] + BC_Sweep_LongTermB[n] * (double) scanCount ; // plus long term trend
						    //   printf("Starting sweep Step %f Speed %f around position %f \n", BC_Sweep_Step[n], BC_Sweep_Speed[n], BC_Sweep_Target[n]);
							   //						   BC_Sweep_LastKnownIndex[n], BC_Sweep_LastKnownPositions[n * FTMEMORY + BC_Sweep_LastKnownIndex[n] ]);
						

						// Fringe detection part
						
						// update array of last knwown positions
						    BC_Sweep_Target[n] = BC_Sweep_LastKnownPositions[n];
						    BC_Sweep_High[n] = BC_Sweep_Target[n] + BC_Sweep_Step[n];
						    BC_Sweep_Low[n] = BC_Sweep_Target[n] - BC_Sweep_Step[n];
						   

						    // update long term trend here...
						    // BC_Sweep_LongTermA[n] = 
						    // BC_Sweep_LongTermB[n] =
						  
						
						  if( BC_Sweep_Direction[n] < 0)
						    {
						      if( BC_FT_Offset[n] < BC_Sweep_Low[n] )
							{
							  BC_Sweep_Direction[n] = 1;
							  printf("Turn + Sweep Target %f Current Position %f Last Known Position %f High %f Low %f Step %f Speed %f \n",
								 BC_Sweep_Target[n], BC_FT_Offset[n],  BC_Sweep_LastKnownPositions[n],
								 BC_Sweep_High[n], BC_Sweep_Low[n], BC_Sweep_Step[n],  BC_Sweep_Speed[n]);
							}

						    }

						if( BC_Sweep_Direction[n] > 0)						  
						  {
						    if( BC_FT_Offset[n] > BC_Sweep_High[n] )
						      {
							BC_Sweep_Direction[n] = -1;
							printf("Turn - Sweep Target %f Current Position %f Last Known Position %f High %f Low %f Step %f Speed %f \n",
							       BC_Sweep_Target[n], BC_FT_Offset[n],  BC_Sweep_LastKnownPositions[n],
							       BC_Sweep_High[n], BC_Sweep_Low[n], BC_Sweep_Step[n],  BC_Sweep_Speed[n]);
							
						      }
						  }

					       					     
						/*
						if(BC_Sweep_Unsuccessful[n] > 10) // we went back and forth and found nothing 10 times in a row
						  {						    
						    printf("Hit 10 edges without fringes --- jumping to last known position\n");
						    // Jump to most probable position based on long term trend
						    BC_Sweep_Target[n] = BC_Sweep_LasKnownPositions[n];
						    //	       BC_Sweep_Target[n] = BC_Sweep_LastKnownPositions[n * FTMEMORY + BC_Sweep_LastKnownIndex[n] ] ; // last position
						      //                    + BC_Sweep_LongTermA[n] + BC_Sweep_LongTermB[n] * (float) scanCount ; // plus long term trend   						    
						    BC_Sweep_Edge[n] = BC_Sweep_Step[n]; // reset the range size
						    BC_FT_Stage[n] = BC_FT_Offset[n] - BC_Sweep_Target[n] ; // this will make us jump there
						    BC_FT_Offset[n] = BC_Sweep_Target[n] ; // new target
						    BC_FT_Stage_Weight[n] = 10. ;						   
						    BC_Sweep_Unsuccessful[n] = 0 ; // new start
						  }
						else // there is still hope, we saw a fringe lately...
						*/
						// {
						    // continue sweeping normally
						    
						    BC_FT_Stage[n] = BC_Sweep_Direction[n] * BC_Sweep_Speed[n] * (float) daq.framesPerReset * dtime / 1000. ;
						    BC_FT_Stage_Weight[n] = 10. ;
						    if(scanCount %100 == 1)
						      printf("Sweep Target %f Current Position %f Last Known Position %f High %f Low %f Step %f Speed %f \n",
							     BC_Sweep_Target[n], BC_FT_Offset[n],  BC_Sweep_LastKnownPositions[n],
							     BC_Sweep_High[n], BC_Sweep_Low[n], BC_Sweep_Step[n],  BC_Sweep_Speed[n]);
						    //  }

					      }


					  }// wrapbeams
					    
					  
					    

					if ( searchFlag == 1 )
					  { // then add onto the FT_Offsets
					    
					    if(exc.WrapBeam == 4)
					      champ_bs4(BC_FT_Stage_Weight, BC_FT_Stage, BC_FT_Stage1, temp_dl_offsets);
					    if(exc.WrapBeam == 6)
					      champ_bs6(BC_FT_Stage_Weight, BC_FT_Stage, BC_FT_Stage1, temp_dl_offsets);
					    
					    for ( n = 0; n < exc.WrapBeam ; n++ )
					      {
						BC_FT_Offset[n] = BC_FT_Offset[n] + BC_FT_Stage1[n];
						BC_FT_Stage[n] = 0.0; // reset to zero.
						BC_FT_Stage_Weight[n] = 0.0;
					      }
					  }

					    // end classic scanning

					  
					    //////////////////////////////////////////////////////////////
					    //
					    // Starting GDT
					    // This method uses the last dewarped values stored in history						
					    //
					    ///////////////////////////////////////////////////////////////


					    gdt_update_dl = FALSE; // will be set to 1 if GDT updates anything                                                                                                        

                                            if(VERBOSE == 1)
                                              printf("Beginning GDT by pow amp method\n");

					    for ( n = 0; n < exc.WrapBeam ; n++ )
					      { 

						inv_nsamples = 1./ ((float) (daq.framesPerReset - npix_fringe[n] )) ; 
						
						frame_offset = ( naccumulated[n] % 2 ) * daq.framesPerReset;
						
						// modified 12 June 2012 
						//frame_offset = ( scanCount % 2 ) * daq.framesPerReset;

						// Accumulate the results of this scan
						// if we are currently cophasing then it will add the amplitudes coherently                                                                   
						// if we are only trying to do coherencing, then amplitudes are added incoherently          
						// Accumulation is interleaved

						for(i = npix_fringe[n] - 1 ; i < daq.framesPerReset - 1 ; i++)
						  {
						    indx1 = i  + n * daq.numberOfFrames + frame_offset;

						    indx2 = ( naccumulated[n] % (2 * NINTERLEAVED) ) * MAXCOMBOS / 2 * daq.framesPerReset 
						    						          + i  + n * daq.framesPerReset ; 

						    // modified 12 June 2012
						    // indx2 = ( scanCount % (2 * NINTERLEAVED) ) * MAXCOMBOS / 2 * daq.framesPerReset 
						    //  + i  + n * daq.framesPerReset ; 

						    indx3 = ( ( ( 
                                                                   ( naccumulated[n] - 2 * NINTERLEAVED * NACCUMULATIONS[n] ) / 2) 
								+ dither_accum0[n]) % FTMEMORY  ) 
						      * MAXCOMBOS / 2 * daq.numberOfFrames + indx1 ; 

						    // if( COPHASING == TRUE ) //cophasing -- add coherently
						    //						      {
							if(  naccumulated[n] < (2 * NINTERLEAVED * NACCUMULATIONS[n]) ) // we have just started accumulating
							  {
							    gdt_X_accumulator[ indx2 ] += ft_x[ indx1 ] / (float) NACCUMULATIONS[n]; 
							    gdt_Y_accumulator[ indx2 ] += ft_y[ indx1 ] / (float) NACCUMULATIONS[n];
							    gdt_N_accumulator[ indx2 ] += ft_n[ indx1 ] / (float) NACCUMULATIONS[n];
							    
							  }
							else // accumulation ongoing, we have to remove values from history 
							  {

							    gdt_X_accumulator[ indx2 ] += (ft_x[ indx1 ] - fts_x[ indx3 ]) / (float) NACCUMULATIONS[n];
							    gdt_Y_accumulator[ indx2 ] += (ft_y[ indx1 ] - fts_y[ indx3 ]) / (float) NACCUMULATIONS[n];
							    gdt_N_accumulator[ indx2 ] += (ft_n[ indx1 ] - fts_n[ indx3 ]) / (float) NACCUMULATIONS[n];
							  }

						    
							//						      }
							//						    else // coherencing -- add incoherently
							//						      {
							if( naccumulated[n] <  (2 * NINTERLEAVED * NACCUMULATIONS[n])  )
							  {
							    gdt_amp_accumulator[ indx2 ] += ft_num[ indx1 ] / (float) NACCUMULATIONS[n];
							  }
							else
							  {
							    // temp_var = gdt_amp_accumulator[ indx2 ];
							    gdt_amp_accumulator[ indx2 ] += (ft_num[ indx1 ] - fts_num[ indx3 ]) / (float) NACCUMULATIONS[n];
							  }



							gdt_amp_accumulator[indx2] = ft_num[indx1];  // DESPERATION KLUDGE by JDM
							  


							//						      }

						    // GDT opd = average of opd now + opd from npix_fringe[n] frames before
						    gdt_opds[ indx2 ] =  0.25 * (
						            channel_opds[ i +  2 * n  * daq.numberOfFrames  + frame_offset ] 
							 +  channel_opds[ i + (2 * n + 1) * daq.numberOfFrames + frame_offset ]
							 +  channel_opds[ i - npix_fringe[n] + 1 + 2 * n * daq.numberOfFrames + frame_offset ]
						         +  channel_opds[ i - npix_fringe[n] + 1 + (2 * n + 1) * daq.numberOfFrames + frame_offset] );
						   
						  }


						if( naccumulated[n] >=  ( 2 * NINTERLEAVED * NACCUMULATIONS[n] )  )
						  {
						    //						    if( COPHASING == TRUE )

						    /* // Commented out the coherent GDT estimator 
						       // Performance seems no better than the incoherent GDT estimator
						    if(BC_FT_State[n] == BC_FT_LOCK)
						      {
							for(i = npix_fringe[n] - 1; i < daq.framesPerReset - 1  ; i++)
							  {
							indx2 = (naccumulated[n] % ( 2  * NINTERLEAVED ) ) * MAXCOMBOS / 2 * daq.framesPerReset 
							  + n * daq.framesPerReset 
							  + i  ;
							
							if((i==npix_fringe[n])&&((scanCount % 100) == 1))
							    printf("Using coherent GDT on %d\n",n);
							// Compute vis for cophasing mode
							
							gdt_amp_accumulator[indx2 ] =  (gdt_X_accumulator[ indx2 ] * gdt_X_accumulator[ indx2 ] 
       							   +  gdt_Y_accumulator[ indx2 ]  * gdt_Y_accumulator[ indx2 ]);
							
							  }
						      }
						    */

						    //////////////////////////
						    //
						    // MATCH FILTER
						    //
						    //////////////////////////
						
						    maxamp[n] = 0;
						    for(i = 0 ; i < daq.framesPerReset ; i++) 
						      {
							indx1 = (scanCount %  2 ) * MAXCOMBOS / 2 * daq.framesPerReset  + n * daq.framesPerReset + i; 
							indx2 = (scanCount % ( 2 * NINTERLEAVED) ) * MAXCOMBOS /2 * daq.framesPerReset + n * daq.framesPerReset + i ; 

							// indx1 = (naccumulated[n] %  2 ) * MAXCOMBOS / 2 * daq.framesPerReset 
							//  + n * daq.framesPerReset + i;
							// indx2 = (naccumulated[n] % ( 2 * NINTERLEAVED) ) * MAXCOMBOS /2 * daq.framesPerReset 
							//  + n * daq.framesPerReset + i ;

							filtered_amp[indx1] =  0;
							//    npix_filter = 0; // mainly for filter normalization at the edge							
							for(j=-filter_halfsize; j<filter_halfsize+1;j++)
							  {
							    if ( ( (i+j) > 0 ) && ( (i+j) < daq.framesPerReset ))
							      {
								filtered_amp[indx1] += gdt_amp_accumulator[indx2+j] * filter_gauss[j]; 
								//								npix_filter ++;
							      }							    
							  }
							//							filtered_amp[indx1] /= (float)npix_filter;
							// Estimated delay = max  
							if(filtered_amp[indx1] > maxamp[n]) // we found the max
							  {
							    maxamp[n] = filtered_amp[indx1];
							    estimated_delay[n] = gdt_opds[indx2];							      							      
							  }
							
						      }

						    if (frameskip == 1)
						      { 
							printf("Zeroing the maxmap \n");
							maxamp[n] = 0;
							estimated_delay[n] = 0;
						      }
						      
						    								// if we skipped a frame, maxamp will be 0 and estimated_delay stay the same as before
						    
						    //						    if(n == 1) printf("n %i Estimated Delay: %f Ramp: %d\n", n, estimated_delay[n], scanCount %  2 );
							  
						    /* Parabola fit

						    //Compute the sums necessary for the parabola fit 
						    tx = 0; ty = 0; txx=0; txy =0; txxx = 0;
						    maxamp[n] = 0.;

						    for(i = npix_fringe[n] - 1 ; i < daq.framesPerReset - 1 ; i++)
						      {
							indx2 = (naccumulated[n] % ( 2 * NINTERLEAVED) ) * MAXCOMBOS /2 * daq.framesPerReset 
							  + n * daq.framesPerReset + i;

							if(gdt_amp_accumulator[indx2] > maxamp[n])
							    maxamp[n] = gdt_amp_accumulator[indx2];
														
							tx   += gdt_opds[indx2];
							ty   += gdt_amp_accumulator[indx2 ];
							txx  += gdt_opds[indx2] * gdt_opds[indx2];
							txy  += gdt_opds[indx2] * gdt_amp_accumulator[indx2 ];
							txxx +=  gdt_opds[indx2] * gdt_opds[indx2] * gdt_opds[indx2];
							
							//							if (((ditherCount % 100) == 0)&&(n==0))
							// printf("n: %d i: %ld amp %f opd %f \n", n, i, gdt_amp_accumulator[indx2 ], gdt_opds[indx2]);
						      }
						    

						    // We try try to fit the amplitude with a parabola
						    // We want to minimize sum_i (y_i - (a + b ( x_i - c ) ^ 2 ) )^2
						    // where y_i = amp value, x_i = opd values
						    // a = parabola_level, b = parabola_concavity, c = estimated group delay = parabola_delay
						    
						    //init value for the parabola width based on the maximum visibility 
						    if(maxamp[n] > 1e-8)
						      parabola_concavity[n] = - maxamp[n] * 0.01;
						    else
						      parabola_concavity[n] = 1e-8; 

						    //Recursive estimation of the parabola coefficients
						    for(k=0; k < 20; k++)
						      {
							parabola_delay[n] = (tx * ty * inv_nsamples - txy + parabola_concavity[n] *
									     ( txxx - txx * tx * inv_nsamples) ) 
							  / (2.* parabola_concavity[n] * ( txx - tx *tx * inv_nsamples ) );
							parabola_level[n] = ( ty - parabola_concavity[n] * txx 
									      + 2 * parabola_concavity[n] * parabola_delay[n] * tx 
									   - parabola_concavity[n] * parabola_delay[n] * parabola_delay[n] / inv_nsamples)
							                    * inv_nsamples;
							parabola_concavity[n] = - parabola_level[n] * 0.01;
						       }
						    
						    
						    */

						    if( BC_FT_State[n] == BC_FT_LOCK )
						      {
							// SNR check
							//							round_delay = round(parabola_delay[n] / exc.lambda);
							round_delay = round(estimated_delay[n] / exc.lambda);
							if( fabs(round_delay) > 0.)
							  {
							    if( round_delay > 1.)
							      round_delay = 1.;
							    
							    if( round_delay < -1.)
							      round_delay = -1.;
							    gdt_jump_accumulator[n] += round_delay ;
							  }
							
							if ( ( fabs( gdt_jump_accumulator[n] ) > NACCUMULATIONS[n] * 5 ) )
							  {
							    printf("GDT - COMBINER %d - trying to jump %d fringes gdt_acc %d NACC %d \n",
								   n, (int)round_delay,  gdt_jump_accumulator[n],  NACCUMULATIONS[n] );
							    
							    BC_FT_Stage[n] =  0.5 * exc.lambda * round_delay; 
							    // note: round_delay same sign as direction we want to go
							    BC_FT_Stage_Weight[n] = 10000.;
							    gdt_update_dl = TRUE;
							    gdt_jump_accumulator[n] = 0;
							  }
							else
							  {
							    gdt_update_dl = FALSE;
							  }
						      }

						    if( BC_FT_State[n] == BC_FT_SEMILOCK )
                                                      {
							// Update the delay lines with these values (even if below threshold)                                                                                                                
							gdt_update_dl = TRUE;
                                                     
							if( gdt_above_threshold[n] == TRUE )							 
							  BC_FT_Stage[n] = BC_FT_GDT_Gains[n] * estimated_delay[n]; 
							else
							  BC_FT_Stage[n] = 0; // do not move if below threshold 
							
							BC_FT_Stage_Weight[n] = (maxamp[n] - BC_Amp_Threshold_Mean[n]) / BC_Amp_Threshold_RMS[n];
						
						      }

						    /*
						    if(  ( (  (scanCount % 100) == 0 ) || ( (scanCount % 100 ) == 101 )))
						     printf("Channel: %d Nacc: %d Ncycles: %d \t Parabola Level: %f Concavity:%f Delay: %f MaxAmp: %f Num %f Phi_mid %f Phi_hist_avg %f Phi_hist_dev %f\n",
						    	     n, naccumulated[n], naccumulated[n] / (2 * NACCUMULATIONS[n] * NINTERLEAVED) , parabola_level[n],
						    	     parabola_concavity[n], parabola_delay[n], maxamp[n], 
						            fringe_amp_mean[n], ft_phi[npix_fringe[n] +daq.framesPerReset / 2  + n * daq.numberOfFrames], ft_phi_mean[n], ft_phi_stddev[n]); 
						    */
						  } // end of if( naccumulated[n] == NACCUMULATIONS)
					       
						// Store GDT quantities to transfer to the GUIs
						
						//                                                parabola_transfer[     ( naccumulated[n] % 2 ) * 3 + 6 * n] = 0; // parabola_level[n];
						//                                                parabola_transfer[ 1 + ( naccumulated[n] % 2 ) * 3 + 6 * n] = 0; // parabola_concavity[n];
						//                                                parabola_transfer[ 2 + ( naccumulated[n] % 2 ) * 3 + 6 * n] = 0; // parabola_delay[n];

						estimated_delay_transfer[( scanCount % 2 ) + 2 * n] = estimated_delay[n];
						//						if(frameskip != 1) 
						naccumulated[n]++;

						// Store the delay actually used in history                                             
						gdt_delay_history[ (scanCount % (FTMEMORY * 2) ) * MAXCOMBOS / 2 + n ] = estimated_delay[n];
					      } // end of for loop on combos/2

					    if(VERBOSE) printf("End GDT\n");
					    memcpy( gdt_opds_transfer, gdt_opds,  MAXCOMBOS / 2 *  daq.numberOfFrames * sizeof(float));
					    memcpy( gdt_amp_transfer, gdt_amp_accumulator,  MAXCOMBOS / 2 *  daq.numberOfFrames * sizeof(float));
					    memcpy( gdt_filtered_amp_transfer, filtered_amp,  MAXCOMBOS / 2 *  daq.numberOfFrames * sizeof(float));
					    
					    if( gdt_update_dl == TRUE) // update the delay offsets
					      {				
						if(exc.WrapBeam == 4)
						  champ_bs4(BC_FT_Stage_Weight, BC_FT_Stage, BC_FT_Stage1, temp_dl_offsets);
						if(exc.WrapBeam == 6)
						  champ_bs6(BC_FT_Stage_Weight, BC_FT_Stage, BC_FT_Stage1, temp_dl_offsets);

					        for ( n = 0; n < exc.WrapBeam ; n++ )
						{
						  BC_FT_Offset[n] += BC_FT_Stage1[n];
						  BC_FT_Stage[n] = 0.0; // reset to zero.
						  BC_FT_Stage_Weight[n] = 0.0;
						}
						
					      }	
					    
					    // Transfer histories for FT GUI
					    for(n=0; n < MAXCOMBOS/2; n++)
                                              {
						ft_offsets_history[(scanCount % (FTMEMORY*2)) * MAXCOMBOS / 2 + n] = (float) BC_FT_Offset[n];
						ft_num_history[ ( scanCount % (FTMEMORY*2) ) * MAXCOMBOS / 2 + n] = maxamp[n];
						if (frameskip == 1) printf("maxamp %i %f \n",n,maxamp[n]);
					      }

					    scanCount_transfer = scanCount; 

					    if(frameskip == 1) // we skipped a frame during this scan
					      {
						frameskip = 0; // we only reset now
						printf("frameskip has been reset\n");
					      }

					  } // end of scanend




					/////////////////////////////////////////////////
					//
					//  TALK TO DELAY LINES 
					//
					////////////////////////////////////////////////


					if ( BC_INIT_Ref_State == 1)
					  {
					    /* MOVE FT Offsets into the BC Ref Offsets -- */
					    for (n=0;n<exc.WrapBeam;n++)// First fill staging area with 0.0 weighted by SNR
					      {
						BC_FT_Stage[n]=0.0;
						BC_FT_Stage_Weight[n] = (maxamp[n] - BC_Amp_Threshold_Mean[n]) / BC_Amp_Threshold_RMS[n];
						if((BC_FT_State[n] == BC_FT_SEARCH) || (BC_FT_State[n] == BC_FT_SWEEP) || (BC_FT_State[n] == BC_FT_FREEZE))
						  BC_FT_Stage_Weight[n] = 1000;
						if(BC_FT_State[n] == BC_FT_OFF)
						  BC_FT_Stage_Weight[n] = 0;
					      }

					    BC_FT_Stage[BC_INIT_Ref_Index] = BC_FT_Offset[BC_INIT_Ref_Index];//use current FT Offset
					    BC_FT_Stage_Weight[BC_INIT_Ref_Index] = 10000.;

					    if (exc.WrapBeam == 4) 
					      champ_bs4(BC_FT_Stage_Weight, BC_FT_Stage, BC_FT_Stage1, temp_dl_offsets);
					    if (exc.WrapBeam == 6)
					      champ_bs6(BC_FT_Stage_Weight, BC_FT_Stage, BC_FT_Stage1, temp_dl_offsets);

					    for ( n = 0; n < exc.WrapBeam ; n++ )
					      {
						BC_FT_Offset[n] = BC_FT_Offset[n] - BC_FT_Stage1[n];
						BC_Ref_Offset[n] = BC_Ref_Offset[n] + BC_FT_Stage1[n];
						BC_FT_Stage[n] = 0.0; // reset to zero.
						BC_FT_Stage_Weight[n] = 0.0;
					      }
					    BC_INIT_Ref_State = 0;
					    vme_active = 1;
					  }
					else if( BC_INIT_Ref_State == 2)
					  {
					    /* Reset the BC Ref Offsets -- */
					    /* Do this just before talking to VME -- will try to keep other fringes */
					    for (n=0;n<exc.WrapBeam;n++)// First fill staging area with 0.0 weighted by SNR
					      {
						BC_FT_Stage[n]=0.0;
						BC_FT_Stage_Weight[n] = (maxamp[n] - BC_Amp_Threshold_Mean[n]) / BC_Amp_Threshold_RMS[n];

						if((BC_FT_State[n] == BC_FT_SEARCH) || (BC_FT_State[n] == BC_FT_SWEEP) || (BC_FT_State[n] == BC_FT_FREEZE))
						  BC_FT_Stage_Weight[n] = 1000;
						if(BC_FT_State[n] == BC_FT_OFF)
						  BC_FT_Stage_Weight[n] = 0;
					      }

					    BC_FT_Stage[BC_INIT_Ref_Index] = -BC_FT_Offset[BC_INIT_Ref_Index];
					    BC_FT_Stage_Weight[BC_INIT_Ref_Index] = 10000.;
					   
					    if (exc.WrapBeam == 4) 
					      champ_bs4(BC_FT_Stage_Weight, BC_FT_Stage, BC_FT_Stage1, temp_dl_offsets);
					    if (exc.WrapBeam == 6)
					      champ_bs6(BC_FT_Stage_Weight, BC_FT_Stage, BC_FT_Stage1, temp_dl_offsets);

					    for ( n = 0; n < exc.WrapBeam ; n++ )
					      {
						BC_FT_Offset[n] = BC_FT_Offset[n] + BC_FT_Stage1[n];
						//BC_Ref_Offset[n] = BC_Ref_Offset[n] + BC_FT_Stage1[n];
						BC_FT_Stage[n] = 0.0; // reset to zero.
						BC_FT_Stage_Weight[n] = 0.0;
					      }
					    BC_INIT_Ref_State = 0;
					    vme_active = 1;
					  }

					
					if ( send_to_vme == TRUE  )
					  { // over-arching send-to-vme.. if vme is not on, then this will not actually send anything
					    // should we do a check and only communicate with vme if there is a change in delay lines?
					    if(VERBOSE) printf("Sending commands to vme\n");
					    vmecounter++;
					    send_champ2vme(exc.BeamOrder, exc.RefTel, exc.WrapBeam, BC_Ref_Offset, BC_FT_Offset);
					    vme_active=0;
					    //}

					  } // send to vme
					

					////////////////////////////////////////////////////
					//
					//         END OF DITHER
					//
					///////////////////////////////////////////////////

					if ( ditherend == 1 )
					  {					    

					    for( n = 0; n < exc.WrapBeam ; n++ )
					      {
						// Check accumulation status
						if(NACCUMULATIONS_CHANGED[n] == TRUE) // reset accumulations if number of accumulations parameter changed
						  {
						    NACCUMULATIONS_CHANGED[n] = FALSE;
						    naccumulated[n] = 0;
						    dither_accum0[n] = ditherCount + 1;
						    
						    memset(&gdt_amp_accumulator[n * daq.framesPerReset],
                                                           0, daq.framesPerReset * sizeof(float));

						    memset(&gdt_X_accumulator[n * daq.framesPerReset],
                                                           0, daq.framesPerReset * sizeof(float));

						    memset(&gdt_Y_accumulator[n * daq.framesPerReset],
                                                           0, daq.framesPerReset * sizeof(float));

						    memset(&gdt_N_accumulator[n * daq.framesPerReset],
                                                           0, daq.framesPerReset * sizeof(float));

						    memset(&gdt_amp_accumulator[MAXCOMBOS / 2 * daq.framesPerReset + n * daq.framesPerReset], 
							   0, daq.framesPerReset * sizeof(float));
						  
						    memset(&gdt_X_accumulator[MAXCOMBOS / 2 * daq.framesPerReset + n * daq.framesPerReset],
                                                           0, daq.framesPerReset * sizeof(float));

                                                    memset(&gdt_Y_accumulator[MAXCOMBOS / 2 * daq.framesPerReset + n * daq.framesPerReset],
                                                           0, daq.framesPerReset * sizeof(float));

                                                    memset(&gdt_N_accumulator[MAXCOMBOS / 2 * daq.framesPerReset + n * daq.framesPerReset],
                                                           0, daq.framesPerReset * sizeof(float));
						    printf("NACCUMULATIONS changes for beam %d -- Resetting all arrays \n", n);
						  }						
					      }



					    //JDMTEST
					    if ( 1 )
					      {
						if(VERBOSE) printf("End of dither\n");
						
						if ( (ditherCount % 80) == 0 )
						  {
						    for (i=0;i<exc.WrapBeam;i++)
						      {
							//							if(i==1)
							  printf("State(n= %ld ): %d  Ditherend vmecounts %i scanCount %i  BC_FT_Offsets: %f %f %f %f %f %f \n",
								 i, BC_FT_State[i], vmecounter, scanCount, BC_FT_Offset[0], BC_FT_Offset[1], BC_FT_Offset[2], BC_FT_Offset[3],
								 BC_FT_Offset[4], BC_FT_Offset[5]);
							
						      }
						  }
					      }
					    					    
					    //
					    // Transfer values to the GUIs
					    //

					    // Prepare flux vs OPD for IRCCam.py
					    memcpy(daqOut.fluxSeries, channelFlux, daq.numberOfFrames * MAXCOMBOS * sizeof(float));
					    memcpy(ft_phi_transfer, ft_phi, daq.numberOfFrames * MAXCOMBOS / 2 * sizeof(double));

					    // Prepare phase vs OPD
					    // TBD
					    
					    // Prepare flux vs time history for CHAMPmain.py
					    for(n=0; n < MAXCOMBOS; n++)
					      {
						indx1 = (ditherCount % FTMEMORY) * MAXCOMBOS + n  ;
						flux_history[ indx1 ] = 0 ;
						for(i = 0 ; i < daq.numberOfFrames; i++)
						  {
						    flux_history[ indx1 ] += channelFlux[i + n * daq.numberOfFrames];
						  }
						flux_history[ indx1 ] /= (float) daq.numberOfFrames ;
					      }



					    //	printf("Dithercount %d %d %f\n", ditherCount, ditherCount % FTMEMORY, flux_history[ (ditherCount % FTMEMORY) * MAXCOMBOS + 2 ]);
					    //	if( flux_history[ (ditherCount % FTMEMORY) * MAXCOMBOS + 2 ] < 60)
					    //  printf("****************************************************************************************************************************\n");
					    ditherCount_transfer = ditherCount;

					    /* difference the first frame from the last and
					       copy it to the GUI output structure */
					    
                                              
					      for ( j = 0; j < (daq.rows * daq.cols) ; j++ )
						{
						  *(coaddBuffer+j) += *(totalBuffer + daq.rows * daq.cols * (daq.framesPerReset - 1) + j)- *(totalBuffer+j) +
						                      *(totalBuffer + daq.rows * daq.cols * (daq.numberOfFrames -1 ) + j)- 
						    *(totalBuffer + daq.rows * daq.cols * (daq.framesPerReset) + j);
						}
					      
					      coaddFrame++;
					      
					      if ( (coaddFrame >= daq.numberOfCoadds) || (daq.numberOfCoadds == 0))
						{
						  for ( j = 0; j < (daq.rows * daq.cols) ; j++ )
						    *(daqOut.coadd + j) = *(coaddBuffer + j) / (  (float)coaddFrame*((float)daq.numberOfFrames-2));
						  coaddFrame=0;
						  coaddCount++;
						  daqOut.coaddCount = coaddCount;
						  memset(coaddBuffer, 0, daq.rows * daq.cols * sizeof(float));
						}				      


    					   
					    // reset frame accumulators
					    memset(totalBuffer, 0, daq.cols * daq.rows * sizeof(float) * daq.numberOfFrames);
					    memcpy(channelLastFlux, channelFlux, MAXCOMBOS * daq.numberOfFrames * sizeof(float));					    
					    memset(channelFlux, 0, MAXCOMBOS * daq.numberOfFrames * sizeof(float));
					    memset(channel_x0, 0, MAXCOMBOS * sizeof(float) * daq.numberOfFrames);
					    memset(channel_y0, 0, MAXCOMBOS * sizeof(float) * daq.numberOfFrames);
					    memset(channel_n0, 0, MAXCOMBOS * sizeof(float) * daq.numberOfFrames);
					    memset(abcd_trigon, 0, 9 * MAXCOMBOS * sizeof(float) * daq.numberOfFrames);
					    ditherCount++;
					  }
					
				} /*end offset calc if*/
			
				
				if ( exc.continuousReadout )
				  exc.cmd = C_RUN;
				else
				  exc.cmd = C_IDLE;
				
				// e.g., reset background
				if ( exc.resetBkgnd == TRUE )
				  {
				    nBkgnd = 0;
				    nBkgnd_total = 0;
				    if(bkgndBuffer)
				     {
				       bkgndBuffer = CHAMPrealloc(bkgndBuffer, frameSizePixels * daq.numberOfFrames * sizeof(float));
				       memset(bkgndBuffer, 0, frameSizePixels * daq.numberOfFrames * sizeof(float));
				     }
				    else
				      {
					bkgndBuffer = CHAMPalloc(bkgndBuffer, frameSizePixels * daq.numberOfFrames * sizeof(float));
				      }
				    
				    exc.resetBkgnd = FALSE;
				    exc.useBkgnd = FALSE; 
				    //				    use_Debias = FALSE;
				    //				    memset(BG_ft_num, 0, sizeof(double) * MAXCOMBOS / 2);
				  }

			} //end BIG while
			

			if(VERBOSE) printf("while loop finished\n");

		} // Ending the S_RUN State..
		
		
		/************************************************/
		/*  Do misc commands                            */
		/*    only executed outside of single exposure  */
		/************************************************/
		if ( exc.endProg )
		  break;
		
		/* enter the next state: */
		ks = ks_next;
		
	} // loop timer
	



	//
	// kill pipe first!
	//
	
	printf("Trying to kill pipe first\n");
	
	// Send KILL command to the spooler process
	message.type = KILL;
	pipe_err = rt_pipe_write(&pipe_desc, &message.type, sizeof(message.type), P_NORMAL);
	
	/*	
	err = rt_pipe_flush(&pipe_desc, 0);
	err = rt_pipe_flush(&pipe_desc, 1); 
	err = rt_pipe_delete(&pipe_desc);
	if ( err )
	  {
	    fprintf(stderr, "Failed to delete pipe, code %d\n", err);
	  }
	else
	  fprintf(stderr, "Pipe deleted\n");
	fflush(NULL);
	*/	
	
	err = rt_sem_unbind(&switch_sem);
	if ( err )
	{
	  fprintf(stderr, "cannot unbind semaphore: %s\n", strerror(-err));
	}
	else
	  fprintf(stdout, "semaphore unbound: \n");
		
	
	free(saveddata.ditherCount);
        free(saveddata.frame1);
        free(saveddata.state);
	//        free(saveddata.combo);
        free(saveddata.time);
	free(saveddata.ftoffsets);
        free(saveddata.refoffsets);
	/*
        free(saveddata.phi);
        free(saveddata.amp);
	free(saveddata.channel_opds);
	free(saveddata.channelFlux);
	free(saveddata.gdt_opds);
	free(saveddata.gdt_amp_accumulator);
	*/
        free(frametimes);
	
	
        free(ft_x);
        free(ft_y);
        free(ft_n);
        free(ft_num);
        free(ft_phi);
        free(ft_snr);
        free(ft_pow);
	free(fts_x);
        free(fts_y);
        free(fts_n);
        free(fts_num);
        free(fts_phi);
        free(fts_snr);
        free(fts_pow);
        free(temp_offsets);
	
	free(gdt_X_accumulator);
        free(gdt_Y_accumulator);
        free(gdt_N_accumulator);
        free(gdt_amp_accumulator);
	free(gdt_opds) ;
 
	free(bkgndBuffer);
	free(BG_ft_num);

	free(totalBuffer);
	free(tmpBuffer);
	free(tmpBuffer1);
	free(channelFlux);
	free(channel_opds);
	free(channel_cospart);
	free(channel_sinpart);
	free(channel_x0);
	free(channel_y0);
	free(channel_n0);
	free(abcd_sinpart);
	free(abcd_cospart);
	free(abcd_trigon);
	

	for ( i = 0; i < MAXCOMBOS ; i++ )
	{
		free(*(channel_pix + i));
	}

	free(timing_values);
	free(timing_values_all);

	fflush(NULL);
	clean_exit(0); //	exit(0);
}

// signal-handler, to ensure clean exit on Ctrl-C
void clean_exit(int dummy)
{
	int err = 0;
	exc.endProg = TRUE;
	fprintf(stdout, "cleanup\n");
	usleep(2000000);
	if ( exc.daqRunning )
		err = rt_task_delete(&sem_task_ptr);
	if ( err )
	{
		fprintf(stderr, "astropci: failed to delete sem_task, code %d\n", err);
	}
	else
		fprintf(stdout, "Delete sem_task\n");
	stop_chara_server();
	exit(0);
}

//void error(char *msg)
//{
//	perror(msg);
//	exit(1);
//}

int main(int argc, char *argv[])
{
	struct smessage mess; //JDM
	message_struct message;
	unsigned int configWord;
	int sockfd, newsockfd, portno, clilen, validate = 0, trigger = 1, cds = 0;
	char pci_file[40] = "/usr/local/etc/pciInt.lod", tim_file[40] = "/usr/local/etc/tim.lod"; // NORMAL
	unsigned char * temp_ptr;
	int temp;
	struct sockaddr_in serv_addr, cli_addr;
	int n, j, i;
	int nagle = 1;
	int err = 0;
	int status;
	int b,paddle_i;
	double testoffsets[6] = { 1.000000, 2.000000, 3.000000, 4.000000, 5.000000, 6.000000 };
	double testoffsets_bs[6],paddle_offset;
	int vme_timetest = 0; // must turn 'on' acknowledgments in champGizmo on vme
	char c;
	int charbufi;
	int vme_i;
	int current_shutter = 0;
	RTIME vmestarts[1000], vmestops[1000];


	/* Initialize the CHAMP CHARA server */

	printf("Initializing CHAMP/CHARA server\n");
	if(start_chara_server() != 0)
	  printf("ERROR: initialization of CHAMP/CHARA server failed\n");

	/*
	mess.type = SHUTTER_CONF;
	mess.length = 0;
	mess.data = NULL;

	if(!message_astromod_init(mess))
	  printf("Error during init of astromod\n");

	*/


	// Check for a port number
	 
	if ( argc < 2 )
	{
		printf("ERROR: no port provided\n");
		exit(1);
	}
	else
	{
		portno = atoi(argv[1]);
		printf("Port number: %i\n", portno);
	}

	/* reset the modulators and trigger */
	NIDAQ_modtrig(MODTRIG_RESET, 1, 1, 1, 1);
	NIDAQ_modtrig(MODTRIG_START, 1, 1, 1, 1);
	NIDAQ_modtrig(MODTRIG_STOP, 1, 1, 1, 1);

	/*
	 * Open the astropci device
	 */
	printf("Opening device /dev/astropci0... ");
	pci_fd = open("/dev/astropci0", O_RDWR);

	/* Check for Reply or Image Buffer allocation failure. */
	if ( pci_fd == -1 && errno == ENOMEM )
	{
		printf("\nERROR:  Required Buffers (Reply or Image) \n Could Not Be Allocated (errno = %d) \n%s", errno, strerror(errno));
		exit(EXIT_FAILURE);
	}

	/* Check that the device can be opened. */
	else if ( pci_fd == -1 )
	{
		printf("\n ERROR:  Device Could Not Be Opened (errno = %d, %s)\n", errno, strerror(errno));
		exit(EXIT_FAILURE);
	}
	printf("DONE\n");

	/*
	 * Bind user memory to kernel buffer
	 */
	sleep(1);
	printf("Creating mem_fd with size %i (%i MB)... ", BUFFER_SIZE, BUFFER_SIZE / 1024 / 1024);
	if ( (mem_fd = create_memory(pci_fd, BUFFER_SIZE)) == NULL )
	{
		printf("\nERROR:  create_memory failed\n");
		exit(EXIT_FAILURE);
	}
	printf("DONE\n");
	printf("\n************************************\n");
	printf("User memory address:  0x%X\n", (int) mem_fd);
	if ( (int) mem_fd == 1 )
	  {
		printf("Bad memory address... exiting \n");
		exit(1);
	  }
	printf("************************************\n\n");

	/*
	 * Disable paging for this program's memory
	 */
	mlockall(MCL_CURRENT | MCL_FUTURE);
	fprintf(stdout, "start\n");

	/*
	 * signal handlers
	 */
	signal(SIGTERM, clean_exit);
	signal(SIGINT, clean_exit);

	/*
	 * Create the accumulator image buffer
	 */

	message.data = (unsigned char *) malloc(256 * sizeof(unsigned char));
	if ( message.data == NULL )
	{
		printf("Cannot allocate message.data \n");
		exit(1);
	}
	mess.data = (unsigned char *) malloc(sizeof(unsigned char));


	err = rt_task_spawn(&sem_task_ptr, "Sem_task", STACK_SIZE, HIGH_PRIORITY, TASK_MODE_1, &sem_task, NULL);
	if ( err )
	{
		fprintf(stderr, "astropci: failed to create sem_task, code %d\n", err);
		//exit (1);
	}
	else
		fprintf(stdout, "sem_task created\n");




	/* Initilialize CHARA shutters */
	printf("Initializing CHARA shutter configuration\n");
       	status = initialize_shutters();
	if(status == 0)
	  {
	    printf("Shutter configuration ok...\n");
	  }
	else
	  {
	    printf("Could not communicate with shutters\n");
	  }
       

	/*
	 * Open the socket to VME
	 * This is one-way (here to there) TCP/IP transmission
	 */
	printf("Trying to open vme socket\n");
	vme_addr.sin_family = AF_INET;
	vme_addr.sin_port = htons(vme_port);
	vme_addr.sin_addr.s_addr = inet_addr(vme_hostname);
	vme_sd = socket(AF_INET, SOCK_STREAM, 0);
	setsockopt(vme_sd, IPPROTO_TCP, TCP_NODELAY, (char *) &nagle, sizeof(nagle));

	printf("opening socket (vme) = %d \n", vme_sd);
	if ( connect(vme_sd, (struct sockaddr *) &vme_addr, sizeof(vme_addr)) == -1 )
	{
		fprintf(stderr, "astropci: failed to connect to VME socket\n");
		send_to_vme = FALSE;
	}
	else
	{
		fprintf(stdout, "Connected to VME socket \n");
		send_to_vme = TALK_TO_DELAYLINES; 
	}

	if ( vme_timetest )
	{

		for ( vme_i = 0; vme_i < 6 ; vme_i++ )
		{
			printf("%i %e\n ", vme_i, testoffsets[vme_i]);
			printf("%i %f\n ", vme_i, testoffsets[vme_i]);
			testoffsets_bs[vme_i] = testoffsets[vme_i];
			byte_swap(testoffsets_bs[vme_i]);
			printf("%i %f\n ", vme_i, testoffsets[vme_i]);
		}

		for ( vme_i = 0; vme_i < 999 ; vme_i++ )
		{
			//printf("loop %i\n ",vme_i);
			vmestarts[vme_i] = rt_timer_read();
			for ( i = 0; i < 6 ; i++ )
			{
				testoffsets[i] = testoffsets[i] + 1e-3;
				testoffsets_bs[i] = testoffsets[i];
				byte_swap(testoffsets_bs[i]);
			}
			//   vmestarts[vme_i]=rt_timer_read();
			if ( write(vme_sd, &testoffsets_bs, sizeof(testoffsets_bs)) != sizeof(testoffsets_bs) )
			{
				printf("Failed to send command to vme champ port.");
			}
			//printf("socket test completed.\n");


			charbufi = 0;
			while (charbufi < 2)
			{
				while (char_waiting(vme_sd))
				{
					read(vme_sd, &c, 1);
					charbufi++;
				}
			};

			//if (charbufi > 0)
			// printf("Emptied VME Socket Buffer: %i \n",charbufi);// TEST
			vmestops[vme_i] = rt_timer_read();
			vmestarts[vme_i + 1] = rt_timer_read();

		}
		for ( vme_i = 0; vme_i < 1000 ; vme_i++ )
		{
			printf("%i %f ms \n", vme_i, (float) (vmestops[vme_i] - vmestarts[vme_i]) / ((double) 1000000));
		}

	}// vme_timetest


	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if ( sockfd < 0 )
	{
		//error(ERROR,"ERROR opening socket");
		exc.endProg = TRUE;
		usleep(2000000);
	}
	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = INADDR_ANY;
	serv_addr.sin_port = htons(portno);
	if ( bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0 )
	{
		//error(ERROR,"ERROR on binding");
		exc.endProg = TRUE;
		usleep(2000000);
	}
	printf("Listening on port %i\n", portno);
	listen(sockfd, 5);
	clilen = sizeof(cli_addr);

	//system(exc.rtpDev);

	while (!exc.endProg)	
	{
	  while(!char_waiting(sockfd))
		 process_message_socket();
	  
	  newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, (unsigned int *) &clilen);
		if ( newsockfd < 0 )
			error(ERROR,"ERROR on accept");

		n = read(newsockfd, (unsigned short *) &message.type, 2);
		if ( n < 0 )
		{
			error(ERROR,"ERROR reading socket type");
			err = 1;
		}
		else
		{

			switch (message.type)
			{

				case EN:
					//Length of nessage = 4 chars (int). this could differ in other architecture
					n = read(newsockfd, (unsigned int *) &message.length, 4);
					if ( n < 0 )
					{
						error(ERROR,"ERROR reading message length");
						err = 1;
					}
					else
					{
						n = read(newsockfd, (unsigned int *) message.data, message.length);
						if ( n < 0 )
							error(ERROR,"ERROR reading from socket");
						// As experiment, close the socket first!!
						close(newsockfd);

						err = raise(SIGINT);
						if ( err )
						{
							fprintf(stderr, "Cannot raise SIGINT: %s\n", strerror(-err));
						}
					}
					break;

				case AB:
					/* Do not execute if rt task NOT running */
					if ( !exc.daqRunning )
						break;
					printf("Abort Exposure...");

					status = (getHstr(pci_fd) & HTF_BITS) >> 3;
					// Send abort exposure/readout according to the HSTR contents.
					if ( status == READOUT )
					{
						err = hcvr(pci_fd, ABORT_READOUT, UNDEFINED, DON);
					}
					else
					{
						err = doCommand(pci_fd, TIM_ID, AEX, DON);
					}
					if ( err == _ERROR )
					{
						printf("failed. -> Could not abort.\n");
					}
					else
						//exc.cmd = C_IDLE;
						printf("done.\n");
					break;

				case IO:
					/* Do not execute if rt task running */
					if ( exc.daqRunning )
						break;

					/* -----------------------------------
					 LOAD PCI FILE
					 ----------------------------------- */
					printf("Loading PCI file %s...", pci_file);
					if ( loadPciFile(pci_fd, pci_file) == _ERROR )
					{
						printf("failed. -> Could not load pci file.\n");
						//return _ERROR;
					}
					else
					{
						printf("done.\n");
					}

					/* -----------------------------------
					 SET (HARDWARE) BYTE SWAPPING
					 ----------------------------------- */
					daq.byteSwapping = setHardwareByteSwapping(pci_fd);

					/* -----------------------------------
					 RESET CONTROLLER
					 ----------------------------------- */
					printf("Resetting controller...");
					if ( hcvr(pci_fd, RESET_CONTROLLER, UNDEFINED, SYR) == _ERROR )
					{
						printf("failed. -> Could not reset controller.\n");
						//return _ERROR;
					}
					else
						printf("done.\n");

					/* -----------------------------------
					 LOAD TIMING FILE/APPLICATION
					 ----------------------------------- */

					printf("Loading timing file %s...", tim_file);
					loadFile(pci_fd, tim_file, validate);
					printf("done.\n");

					break;

				case CC: /* Send Coadd Counter so external program knows if frame is new*/

					// allocate data buffer
					message.length = sizeof(int);
					message.data = (unsigned char *) realloc(message.data, sizeof(int));
					memcpy((unsigned char *) message.data, &daqOut.coaddCount, sizeof(int));
					// send data through socket

					n = write(newsockfd, (unsigned char *) message.data, message.length);
					if ( n < 0 )
						error(ERROR,"ERROR writing to socket");

					break;

			case RD:
			  
			  //Length of nessage = 4 chars (int). this could differ in other architecture
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      
			      // allocate data buffer
			      message.data = (unsigned char *) realloc(message.data, message.length);
			      memcpy((unsigned char *) message.data, daqOut.coadd, message.length);
			      // send data through socket
			      
			      n = write(newsockfd, (unsigned char *) message.data, message.length);
			      
			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");
			    }
			  // close(fd);
			  break;

			case UO: // open shutter -- currently unused
			  
			  //Length of nessage = 4 chars (int). this could differ in other architecture
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"UO : ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      // Open shutter
			      printf("UO: opening IRB %d\n", message.length);
			      if((message.length > 0)&&(message.length < 7))
				{
				  switch(message.length)
				    {
				    case 1:
				      current_shutter = shutter_id("IRB1");
				      break;
				      
				    case 2:
				      current_shutter = shutter_id("IRB2");
				      break;
				      
				    case 3:
				      current_shutter = shutter_id("IRB3");
				      break;
				      
				    case 4:
				      current_shutter = shutter_id("IRB4");
				      break;

				    case 5:
				      current_shutter = shutter_id("IRB5");
				      break;

				    case 6:
				      current_shutter = shutter_id("IRB6");
				      break;
				    }
			    
				  send_shutter_open(current_shutter);
				}
			      else
				{
				  printf("Bad shutter number\n");
				}
			    }
			  // close(fd);
			  break;

			case UC: // close shutter -- currently unused
			  
			  //Length of nessage = 4 chars (int). this could differ in other architecture
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"UC : ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      // Close shutter
			      printf("UC: closing IRB %d\n", message.length);
			      if((message.length > 0)&&(message.length < 7))
				{
				  switch(message.length)
				    {
				    case 1:
				      current_shutter = shutter_id("IRB1");
				      break;
				      
				    case 2:
				      current_shutter = shutter_id("IRB2");
				      break;
				      
				    case 3:
				      current_shutter = shutter_id("IRB3");
				      break;
				      
				    case 4:
				      current_shutter = shutter_id("IRB4");
				      break;

				    case 5:
				      current_shutter = shutter_id("IRB5");
				      break;

				    case 6:
				      current_shutter = shutter_id("IRB6");
				      break;
				    }
			    
				  send_shutter_close(current_shutter);
				}
			      else
				{
				  printf("Bad shutter number\n");
				}
			    }
			  // close(fd);
			  break;

			case UX:
                          //Length of nessage = 4 chars (int). this could differ in other architecture                                                                                                                                      
                          n = read(newsockfd, (unsigned int *) &message.length, 4);
                          if ( n < 0 )
                            {
                              error(ERROR,"UX : ERROR reading message length");
                              err = 1;
                            }
			  else
                            {
                              // Close all shutters except one -- or open all shutters 
                              if((message.length < 7))
                                {
				  switch(message.length)
                                    {
				    case 0: // special case -- open all shutters
				      current_shutter = shutter_id("IRB1");
                                      send_shutter_open(current_shutter);
                                      current_shutter = shutter_id("IRB2");
                                      send_shutter_open(current_shutter);
                                      current_shutter = shutter_id("IRB3");
                                      send_shutter_open(current_shutter);
                                      current_shutter = shutter_id("IRB4");
                                      send_shutter_open(current_shutter);
                                      current_shutter = shutter_id("IRB5");
                                      send_shutter_open(current_shutter);
                                      current_shutter = shutter_id("IRB6");
                                      send_shutter_open(current_shutter);
				      printf("UX: opening all shutters\n");
				      break;

                                    case 1:
				      current_shutter = shutter_id("IRB1");
				      send_shutter_open(current_shutter);
				      current_shutter = shutter_id("IRB2");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB3");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB4");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB5");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB6");
				      send_shutter_close(current_shutter);
				      break;
				      
                                    case 2:
                                      current_shutter = shutter_id("IRB1");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB2");
				      send_shutter_open(current_shutter);
				      current_shutter = shutter_id("IRB3");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB4");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB5");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB6");
				      send_shutter_close(current_shutter);
				      break;
				      
                                    case 3:
				      current_shutter = shutter_id("IRB1");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB2");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB3");
				      send_shutter_open(current_shutter);
				      current_shutter = shutter_id("IRB4");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB5");
				      send_shutter_close(current_shutter);
				      current_shutter = shutter_id("IRB6");
				      send_shutter_close(current_shutter);

                                      break;

                                    case 4:
				      current_shutter = shutter_id("IRB1");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB2");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB3");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB4");
                                      send_shutter_open(current_shutter);
                                      current_shutter = shutter_id("IRB5");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB6");
                                      send_shutter_close(current_shutter);

                                      break;
				      
                                    case 5:
				      current_shutter = shutter_id("IRB1");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB2");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB3");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB4");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB5");
                                      send_shutter_open(current_shutter);
                                      current_shutter = shutter_id("IRB6");
                                      send_shutter_close(current_shutter);

                                      break;
				      
                                    case 6:
				      current_shutter = shutter_id("IRB1");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB2");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB3");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB4");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB5");
                                      send_shutter_close(current_shutter);
                                      current_shutter = shutter_id("IRB6");
                                      send_shutter_open(current_shutter);

                                      break;
                                    }  
				  if(message.length>0) 
				    printf("UX: closing all shutters except IRB %d\n", message.length);
				}
			      else
				{
                                  printf("Bad shutter number\n");
                                }
                            }
                          // close(fd);                                                                                                                                                                                                      
                          break;


			case FX: // Send out FluxSeries upon request of IRCCam
			  // Recall: in the current (bad) paradigm, the python code asks for a certain number of bytes and
			  // so we hope all this matches what we want to send out.
			  
			  //Length of nessage = 4 chars (int). this could differ in other architecture
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      // allocate data buffer
			      message.data = (unsigned char *) realloc(message.data, message.length);
			      memcpy((unsigned char *) message.data, daqOut.fluxSeries, message.length);
			      
			      // send data through socket
			      n = write(newsockfd, (unsigned char *) message.data, message.length);
			      
			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");
			    }
			  break;


			case PX: // PHASE vs OPD
			  // Recall: in the current (bad) paradigm, the python code asks for a certain number of bytes and
			  // so we hope all this matches what we want to send out.
			  
			  //Length of nessage = 4 chars (int). this could differ in other architecture
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      // allocate data buffer
			      message.data = (unsigned char *) realloc(message.data, message.length);
			      memcpy((unsigned char *) message.data, ft_phi_transfer, message.length);
			      
			      // send data through socket
			      n = write(newsockfd, (unsigned char *) message.data, message.length);
			      
			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");
			    }
			  break;


			case AM: // Send out the amplitude upon request of FTGUI                                                                                                                           
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
                          if ( n < 0 )
                            {
                              error(ERROR,"ERROR reading message length");
                              err = 1;
                            }
                          else
                            {
			      //			      printf("AM: Amplitude vs OPDs requested\n");
                              // allocate data buffer                                                                                                                                                     
                              message.data = (unsigned char *) realloc(message.data, message.length);
			      if(ditherCount_transfer > 1)
				{
				  temp = sizeof(int) + MAXCOMBOS * sizeof(float) + 3 * daq.numberOfFrames * MAXCOMBOS / 2 * sizeof(float); 
				  memcpy( message.data, &temp, sizeof(int)); // 
				  temp = sizeof(int);
				  memcpy( &message.data[temp] , estimated_delay_transfer, MAXCOMBOS * sizeof(float));
				  temp += MAXCOMBOS * sizeof(float);
				  memcpy( &message.data[temp] , gdt_filtered_amp_transfer, daq.numberOfFrames * MAXCOMBOS / 2 * sizeof(float));
                                  temp += daq.numberOfFrames * MAXCOMBOS / 2 * sizeof(float);
				  memcpy( &message.data[temp] , gdt_opds_transfer, daq.numberOfFrames * MAXCOMBOS / 2 * sizeof(float));
				  temp += daq.numberOfFrames * MAXCOMBOS / 2 * sizeof(float);
				  memcpy( &message.data[temp], gdt_amp_transfer, daq.numberOfFrames * MAXCOMBOS / 2 * sizeof(float));
				}
			      else
				{
				  printf("AM: no data sent as no data yet\n");
				  memset( message.data, 0, message.length);
				}
			      // send data through socket                                                                                                                                                 
                              n = write(newsockfd, (unsigned char *) message.data, message.length);

                              if ( n < 0 )
                                error(ERROR,"ERROR writing to socket");
                            }
                          // close(fd);                                                                                                                                                                   
                          break;


			case HF: // Send out the flux history upon request of champFT.py
			  // Recall: in the current (bad) paradigm, the python code asks for a certain number of bytes and                                                                       
			  // so we hope all this matches what we want to send out.                                                                                                               
			  
			  //Length of nessage = 4 chars (int). this could differ in other architecture                                                                                           

			  // send length of message, current history position, and data since the last position through socket
			  //			  printf("ditherCount_transfer %d\n", ditherCount_transfer);

			  // reading the index of the latest frame received, using message length
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      temp = slice_history_transfer( flux_history, &message, ditherCount_transfer, MAXCOMBOS , FTMEMORY );

			      // send data through socket                                                                                                                                       
			      n = write(newsockfd, (unsigned char *) message.data, temp + sizeof(int) );

			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");

			    }

			  break;


			case HO: // FT offsets history
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      temp = slice_history_transfer( ft_offsets_history , &message, scanCount_transfer, MAXCOMBOS / 2, FTMEMORY * 2 );

			      // send data through socket                                                                                                                                       
			      n = write(newsockfd, (unsigned char *) message.data, temp + sizeof(int) );

			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");

			    }

			  break;

			case HN: // history of the averages over a scan of ft num values
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      temp = slice_history_transfer( ft_num_history, &message, scanCount_transfer, MAXCOMBOS / 2 , FTMEMORY * 2 );
			      // send data through socket                                                                                                                                       
			      n = write(newsockfd, (unsigned char *) message.data, temp + sizeof(int) );
			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");
			    }
			  break;

			case HP: 
			  //			  printf("HP: Mean phase history\n");
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      temp = slice_history_transfer( ft_phi_mean_history, &message, scanCount_transfer, MAXCOMBOS / 2, FTMEMORY  );
			      // send data through socket                                                                                                                                       
			      n = write(newsockfd, (unsigned char *) message.data, temp + sizeof(int) );

			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");

			    }

			  break;

			case HD:
			  //			  printf("HD: Delay history\n");
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      temp = slice_history_transfer( gdt_delay_history, &message, ditherCount_transfer, MAXCOMBOS / 2 , FTMEMORY * 2 );
			      // send data through socket                                                                                                                                       
			      n = write(newsockfd, (unsigned char *) message.data, temp + sizeof(int) );
			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");
			    }
			  break;


			case SP: // Send out the pyconfig structure out the socket.
			  // Again -- the original request must contain the correct message length
			  
			  //outside service request configuration
			  //Length of nessage = 4 chars (int). this could differ in other architecture
			  
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      // allocate data buffer
			      message.data = (unsigned char *) realloc(message.data, message.length);
			      memcpy((unsigned char *) message.data, &pyconfig, message.length);
			      // send data through socket
			      
			      n = write(newsockfd, (unsigned char *) message.data, message.length);
			      
			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");
			    }
			  // close(fd);
			  break;
			  
			case RC:
			  //outside service request configuration
			  //Length of nessage = 4 chars (int). this could differ in other architecture
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      
			      // allocate data buffer
			      message.data = (unsigned char *) realloc(message.data, message.length);
			      memcpy((unsigned char *) message.data, &daq, message.length);
			      // send data through socket
			      
			      n = write(newsockfd, (unsigned char *) message.data, message.length);
			      
			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");
			    }
			  // close(fd);
			  break;


			case NF:
			  // send configuration (atm: numberofframes) to ftgui
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      
			      // allocate data buffer
			      message.data = (unsigned char *) realloc(message.data, message.length);
			      memcpy((unsigned char *) message.data, &daq.numberOfFrames, message.length);
			      // send data through socket
			      
			      n = write(newsockfd, (unsigned char *) message.data, message.length);
			      
			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");
			    }
			  // close(fd);
			  break;


			  
			case PS:
			  
			  //Length of nessage = 4 chars (int). this could differ in other architecture
			  // read data length
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  // printf("PS Request length = %d\n",message.length);
			  if ( n < 0 )
			    {
						error(ERROR,"ERROR reading message length");
						err = 1;
			    }
			  else
			    {
			      
						// allocate data buffer
			      message.data = (unsigned char *) realloc(message.data, message.length);
			      //          memcpy((unsigned char *) message.data , daqOut.pSpec2D, message.length);
			      
			      // send data through socket
			      n = write(newsockfd, (unsigned char *) message.data, message.length);
			      
			      if ( n < 0 )
							error(ERROR,"ERROR writing to socket");
			    }
			  // close(fd);
			  break;

				case XX:
					// Sets continuous readout to be FALSE
					daq.continuousReadout = FALSE;
					printf("Stopping Continuous Readout at next opportunity...\n");
					while (exc.continuousReadout)
					{
						usleep(100);
					}
					// ok, after getting signal from sem_task, then shut down trigger.
					printf("Stopping NIDAQ_modtrig\n");
					//JDM had the following lines commented
					NIDAQ_modtrig(MODTRIG_STOP, 1, 1, 1, 1);
					usleep(100000);
					NIDAQ_modtrig(MODTRIG_RESET, 1, 1, 1, 1);
					NIDAQ_modtrig(MODTRIG_START, 1, 1, 1, 1);
					NIDAQ_modtrig(MODTRIG_STOP, 1, 1, 1, 1);
					usleep(3000000);
					exc.resetBkgnd = TRUE;
                                        exc.useBkgnd = FALSE;
					break;

				case SX:
					/* Do not execute if rt task running */
					printf("Received Start Exposure SX command %d\n", exc.daqRunning);
					sleep(2);
					if ( exc.daqRunning )
						break;
					/* Set frames per buffer */
					sleep(2);
					//JDM will try Scott STreit's new verison
					exc.framesPerBuffer = nFramesPerBuffer(mem_fd, BUFFER_SIZE, daq.rows, daq.cols * daq.nreads, BITS_PER_PIXEL);

					printf("Setting FPB to %d\n", (int) exc.framesPerBuffer);
					if ( doCommand1(pci_fd, TIM_ID, FPB, exc.framesPerBuffer, DON) == _ERROR )
					{
						printf("\tERROR: could not set frames-per-buffer\n");
					}

					sleep(1);
					/* Set the number of frames */
					printf("Setting number of frames to %d\n", (int) daq.numberOfFrames);
					//DHB "SNF" was "SNC" for MIRC in the following line
					//but parallel changes were made in the dsp code by Leach
					//so it's effectively the same call.
					if ( doCommand1(pci_fd, TIM_ID, SNF, daq.numberOfFrames, DON) == _ERROR )
					{
						printf("\tERROR: could not set number of frames\n");
					}
					sleep(1);
					/* Set the exposure time */
					printf("Setting exposure time (ms): %d\n", daq.exptime);
					if ( doCommand1(pci_fd, TIM_ID, SET, daq.exptime, DON) == _ERROR )
					{
						printf("\tERROR: Setting exposure time failed -> 0x%X\n", getError());
					}

					/* start the exposure.  the trigger will sync everything from here */
					printf("Starting exposure\n");
					if ( (doCommand(pci_fd, TIM_ID, SEX, DON) == _ERROR) )
					{
						printf("\tERROR: Starting exposure failed -> 0x%X\n", getError());
						exc.cmd = C_IDLE;
					}

					printf("Starting modtrig... ");
					printf("Parameters Freq:%f Amp:%f Dither_Smooth: %f Dither_delay: %f\n",
					       ftdata.dither_freq, ftdata.dither_amp, ftdata.dither_smoothfrac, ftdata.dither_delay);
					NIDAQ_modtrig(MODTRIG_START, ftdata.dither_freq, ftdata.dither_amp, ftdata.dither_smoothfrac, ftdata.dither_delay);
					printf("Done\n");

					usleep(3000000);

					exc.continuousReadout = TRUE;
					exc.cmd = C_RUN;

					break;

				case SB:
					/* Do not execute if rt task running */
					if ( exc.daqRunning )
						break;

					printf("Setting PICNIC bias:\n");
					printf("VOFFSET: %6.3f V\n", daq.voffset);
					printf("VRESET: %6.3f V\n", daq.vreset);
					printf("VD: %6.3f V\n", daq.vd);
					printf("ICTL: %6.3f V\n", daq.ictl);

					if ( doCommand4(pci_fd, TIM_ID, SBN, 0, DAC2, VID, (int) (VOLTS2WORD * daq.voffset), DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
					}
					usleep(500000);
					if ( doCommand4(pci_fd, TIM_ID, SBN, 0, DAC3, VID, (int) (VOLTS2WORD * daq.vreset), DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
					}
					usleep(500000);
					if ( doCommand4(pci_fd, TIM_ID, SBN, 0, DAC4, VID, (int) (VOLTS2WORD * daq.vd), DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
					}
					usleep(500000);
					if ( doCommand4(pci_fd, TIM_ID, SBN, 0, DAC5, VID, (int) (VOLTS2WORD * daq.ictl), DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
					}
					break;

				case AQ:
					/* Do not execute if rt task running */
					if ( exc.daqRunning )
						break;
					//Length of nessage = 4 chars (int). this could differ in other architecture
					n = read(newsockfd, (unsigned int *) &message.length, 4);
					if ( n < 0 )
					{
						error(ERROR,"ERROR reading message length");
						err = 1;
					}
					else
					{
						message.data = (unsigned char *) realloc(message.data, message.length);
						n = read(newsockfd, message.data, message.length);
						if ( n < 0 )
						{
							error(ERROR,"ERROR reading data from socket");
							err = 1;
						}
						else
						{
							daq = *(daq_struct *) message.data;

							printf("bufferSize = %d\n", BUFFER_SIZE);
							printf("image_filename = %s\n", daq.image_filename);
							exc.pixelCount = daq.rows * daq.cols * daq.nreads;
							exc.byte_count_float = exc.pixelCount * sizeof(float);
							exc.byte_count_short = exc.pixelCount * sizeof(unsigned short);
							message.data = (unsigned char *) realloc(message.data, message.length);
							exc.endProg = FALSE;

							/* init the save counter */
							nsaved = daq.savenum;
							/* init scans left to scans per file */
							dithersleft = daq.savesPerFile;
							/* write the header first */
							write_header = TRUE;
							write_footer = FALSE;

							//spawn data acquisition task
							printf("daqRunning = %d\n", exc.daqRunning);
							printf("daq.continuousRead = %d\n", daq.continuousReadout);
							printf("daq.doImageSave = %d\n", daq.do_image_save);
							printf("daq message length, n  = %i %d\n", message.length, n);
							printf("daq.savenum %i, daq.numberCoherent %i\n", daq.savenum, daq.numberCoherent);
							/* Does not appear that dataBuffer is used anymore.. */
							/* dataBuffer = (unsigned short *)realloc(dataBuffer, daq.framesPerReset * exc.byte_count_short); */
							/* 	    if (dataBuffer == NULL){ */
							/* 	      printf("Cannot allocate dataBuffer \n"); */
							/* 	      exit (1); */
							/* 	    } */

							exc.cmd = C_INIT;
						}
					}
					break;

				case RP: // Receive new pyconfig structure from python and update exc values (if appropriate).

					//Length of nessage = 4 chars (int). this could differ in other architecture
					n = read(newsockfd, (unsigned int *) &message.length, 4);
					if ( n < 0 )
					{
						error(ERROR,"ERROR reading message length");
						err = 1;
					}
					else
					{
						message.data = (unsigned char *) realloc(message.data, message.length);
						n = read(newsockfd, message.data, message.length);
						if ( n < 0 )
						{
							error(ERROR,"ERROR reading data from socket");
							err = 1;
						}
						else
						{
							pyconfig = *(pythonconfig_struct *) message.data;
							// Update exc variables if need-be !!  [super kludge]
							for ( j = 0; j < NUMTELS ; j++ )
								exc.BeamOrder[j] = pyconfig.BeamOrder[j];
							printf("WARNING: UPDATING BEAM ORDER FROM PYCONFIG !!!!!\n");
							
							exc.RefTel = pyconfig.RefTel;
							exc.lambda = pyconfig.lambda;
							exc.bandwidth = pyconfig.bandwidth;
							
							for ( j = 0; j < BASELINES ; j++ )
							{
								exc.cpeak[j] = pyconfig.cpeak[j];
								exc.cpeak0[j] = pyconfig.cpeak0[j];
								exc.cpeak1[j] = pyconfig.cpeak1[j];
								exc.gdtThreshold[j] = pyconfig.gdtThreshold[j];
							}
							exc.bkgndNoiseCol = pyconfig.bkgndNoiseCol;
							
						}
					}
					break;

				case CN: // Change name of next file.
					//Length of nessage = 4 chars (int). this could differ in other architecture
					printf("CN: Request to Change Name\n");
					n = read(newsockfd, (unsigned int *) &message.length, 4);
					if ( n < 0 )
					{
						error(ERROR,"ERROR reading message length");
						err = 1;
					}
					else
					{
						message.data = (unsigned char *) realloc(message.data, message.length);
						n = read(newsockfd, message.data, message.length);
						if ( n < 0 )
						{
							error(ERROR,"ERROR reading data from socket");
							err = 1;
						}
						else
						{
							daq = *(daq_struct *) message.data;
							message.data = (unsigned char *) realloc(message.data, message.length);
							strcpy(exc.image_filename, daq.image_filename); /* update name */
							printf("Changing next image name to: %s\n", exc.image_filename);
						}
					}
					break;

				case SD: // Save Data. Actually this updates all daq parameters!!
					//Length of nessage = 4 chars (int). this could differ in other architecture
					
				  printf("SD: Saving data\n");
				  
				  n = read(newsockfd, (unsigned int *) &message.length, 4);
				  if ( n < 0 )
				    {
				      error(ERROR,"ERROR reading message length");
				      err = 1;
				    }
				  else
				    {
				      message.data = (unsigned char *) realloc(message.data, message.length);
				      n = read(newsockfd, message.data, message.length);
				      if ( n < 0 )
					{
					  error(ERROR,"ERROR reading data from socket");
					  err = 1;
					}
				      else
					{
					  daq = *(daq_struct *) message.data;
					  message.data = (unsigned char *) realloc(message.data, message.length);
					  printf("Saving Data: %d\n", daq.do_image_save);
					  printf("Number of files to save: %d\n", daq.savenum);
					  printf("Number of dithers per file: %d\n", daq.savesPerFile);
					  nsaved = daq.savenum;
					  dithersleft = daq.savesPerFile;
					  
					}
				    }
				  
					break;

				case SC:
					/* Do not execute if rt task running */
					if ( exc.daqRunning )
						break;

					/* -----------------------------------
					 POWER ON
					 ----------------------------------- */
					printf("Doing power on...");
					if ( doCommand(pci_fd, TIM_ID, PON, DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
						//return _ERROR;
					}
					else
						printf("done.\n");

					sleep(1);
					//printf("Extra error test power on -> 0x%X\n", getError());


					/* -----------------------------------
					 SET DIMENSIONS
					 ----------------------------------- */

					printf("Setting image columns %d ...", daq.cols);
					if ( doCommand2(pci_fd, TIM_ID, WRM, (Y | 1), daq.cols * daq.nreads, DON) == _ERROR )
					{
						printf("failed setting image columns -> 0x%X\n", getError());
						//return _ERROR;
					}
					else
						printf("done.\n");

					sleep(1);
					//printf("Extra error test after cols -> 0x%X\n", getError());

					printf("Setting image rows %d ...", daq.rows);
					if ( doCommand2(pci_fd, TIM_ID, WRM, (Y | 2), daq.rows, DON) == _ERROR )
					{
						printf("failed setting image rows -> 0x%X\n", getError());
						//return _ERROR;
					}
					else
						printf("done.\n");

					sleep(1);
					//printf("Extra error test after rows -> 0x%X\n", getError());


					/* --------------------------------------
					 READ THE PIXEL MASK FILE
					 -------------------------------------- */
					/* zero the pixel mask */
					for ( b = 0; b < MAXIMAGE ; b++ )
						pixelMap[b] = 0;

					/* get the beam positions */
					if ( ReadPixelMapFile(daq.rows, daq.cols, pixelMap) != 0 )
					{
						printf("ERROR reading pixel map file.  ");
						printf("Setting entire subarray mask to 1.\n");
						for ( b = 0; b < (daq.rows * daq.cols) ; b++ )
							pixelMap[b] = 1;
					}

					/* --------------------------------------
					 READ THE CONTROLLER PARAMETERS INFO
					 -------------------------------------- */
					configWord = doCommand(pci_fd, TIM_ID, RCC, UNDEFINED);

					/* --------------------------------------
					 SELECT SUBARRAY MODE
					 -------------------------------------- */

					sleep(1);
					//printf("Extra error test after RCC -> 0x%X\n", getError());

					printf("Setting subarray mode: row = %d col = %d... ", daq.rowOffset, daq.colOffset);
					/* Row and column offset are swapped due to a documentation error */
					if ( doCommand2(pci_fd, TIM_ID, SSA, daq.colOffset, daq.rowOffset, DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
					}
					else
						printf("done.\n");

					sleep(1);
					/* --------------------------------------
					 SET NUMBER OF READS PER PIXEL
					 -------------------------------------- */
					printf("Setting number of reads per pixel = %d...\n", daq.nreads);
					if ( doCommand1(pci_fd, TIM_ID, SNR, daq.nreads, DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
					}
					printf("Note: Number of skipped reads is set to %i\n", NUM_SKIP_READS);
					if ( NUM_SKIP_READS >= daq.nreads )
						printf("***WARNING: number of skipped reads is >= num reads!!!!!\n");
					sleep(1);
					/* --------------------------------------
					 SET TRIGGER MODE
					 -------------------------------------- */
					printf("Setting trigger mode = %d...\n", trigger);
					if ( doCommand1(pci_fd, TIM_ID, STM, trigger, DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
					}
					sleep(1);
					/* --------------------------------------
					 SET READOUT MODE TO CDS
					 -------------------------------------- */
					printf("Setting CDS mode = %d...\n", cds);
					if ( doCommand1(pci_fd, TIM_ID, CDS, cds, DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
					}

					sleep(1);
					//printf("Extra error test after setting CDS mode -> 0x%X\n", getError());

					/* --------------------------------------
					 SET NUMBER OF FRAMES PER RESET
					 -------------------------------------- */
					printf("Setting number of frames per reset = %d...\n", daq.framesPerReset);
					if ( doCommand1(pci_fd, TIM_ID, SFR, daq.framesPerReset, DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
					}
					sleep(1);
					/* --------------------------------------
					 SET READ SIGNAL TOGGLE
					 -------------------------------------- */
					printf("Enable READ signal toggle =  %d...\n", daq.readToggle);
					if ( doCommand1(pci_fd, TIM_ID, SRD, daq.readToggle, DON) == _ERROR )
					{
						printf("failed -> 0x%X\n", getError());
					}

					break;

					//JDM Add writing to ople server (test code)
				case OP:
					//Length of nessage = 4 chars (int). this could differ in other architecture
					n = read(newsockfd, (unsigned int *) &message.length, 4);
					if ( n < 0 )
					{
						error(ERROR,"ERROR reading message length");
						err = 1;
					}
					else
					{
						message.data = (unsigned char *) realloc(message.data, message.length);
						n = read(newsockfd, message.data, message.length);
						if ( n < 0 )
						{
							error(ERROR,"ERROR reading data from socket");
							err = 1;
						}
						else
						{
							memcpy(gdt.dlOffsets, message.data, message.length);
							for ( i = 0; i < NUMDLS ; i++ )
							{
								gdt.manualOffset[i] += gdt.dlOffsets[i];
								// printf("dl received %d %lf\n",i,gdt.dlOffsets[i]);
							}
							exc.manOffsetsReady = TRUE;
						}
					}
					break;

				case GI:
					//Length of nessage = 4 chars (int). this could differ in other architecture
					n = read(newsockfd, (unsigned int *) &message.length, 4);
					if ( n < 0 )
					{
						error(ERROR,"ERROR reading message length");
						err = 1;
					}
					else
					{
						message.data = (unsigned char *) realloc(message.data, message.length);
						n = read(newsockfd, message.data, message.length);
						if ( n < 0 )
						{
							error(ERROR,"ERROR reading data from socket");
							err = 1;
						}
						else
						{
							memcpy(&gdt.gdtLock, message.data, 4);
							memcpy(gdt.activeBaselines, message.data + 4, message.length - 4);
							printf("Locking status %d %d %d %d %d %d\n",
							       gdt.gdtLock[0], gdt.gdtLock[1],  gdt.gdtLock[2], 
							       gdt.gdtLock[3],  gdt.gdtLock[4], gdt.gdtLock[5]);
							printf("Receiving active baselines from client %i %i %i %i %i %i\n", 
							       gdt.activeBaselines[0], gdt.activeBaselines[1], gdt.activeBaselines[2],
							       gdt.activeBaselines[3], gdt.activeBaselines[4], gdt.activeBaselines[5]);
						}
					}
					break;

				case OF:
					//Length of nessage = 4 chars (int). this could differ in other architecture
					// read data length
					printf("Received Request OF..");
					n = read(newsockfd, (unsigned int *) &message.length, 4);
					// printf("PS Request length = %d\n",message.length);
					if ( n < 0 )
					{
						error(ERROR,"ERROR reading message length");
						printf("ERROR in OF");
						err = 1;
					}
					else
					{
						// allocate data buffer
						message.data = (unsigned char *) realloc(message.data, message.length);
						memcpy((unsigned char *) message.data, dloffsets, message.length);

						// send data through socket
						n = write(newsockfd, (unsigned char *) message.data, message.length);
						//	  printf("Sending DL Offsets to client %lf %lf %lf %lf %lf %lf\n",
						//dloffsets[0],dloffsets[1],dloffsets[2],dloffsets[3],dloffsets[4],dloffsets[5]);
						if ( n < 0 )
							error(ERROR,"ERROR writing to socket");
					}
					break;

				case FI:
					//Length of nessage = 4 chars (int). this could differ in other architecture
					// read data length
					n = read(newsockfd, (unsigned int *) &message.length, 4);
					// printf("PS Request length = %d\n",message.length);
					if ( n < 0 )
					{
						error(ERROR,"ERROR reading message length");
						err = 1;
					}
					else
					{

						// allocate data buffer
						message.data = (unsigned char *) realloc(message.data, message.length);
						memcpy((unsigned char *) message.data, exc.gdtOpd, message.length / 2);
						memcpy((unsigned char *) message.data + message.length / 2, exc.gdtWeight, message.length / 2);

						// send data through socket
						n = write(newsockfd, (unsigned char *) message.data, message.length);

						if ( n < 0 )
							error(ERROR,"ERROR writing to socket");
					}
					break;

					/* 	dloffsets[5]=dloffsets[5]+.0001; //increment dl by 100nm */
					/* 	set_manual_target(opleserver,5,dloffsets[5],0,0); */

					break;


			case TB:
			  if (daq.numBkgnd > 0) {
			    exc.takeBkgnd = TRUE;
			    nBkgnd=daq.numBkgnd;
			    printf("Taking %i backgrounds \n",nBkgnd);
			  } else {
			    printf("ERROR:  NOT taking backgrounds because it's set to %i\n",
				   daq.numBkgnd);
			  }
			  break;


				case RB:
					exc.resetBkgnd = TRUE;
					exc.useBkgnd = FALSE;
					printf("Reset Background Now Set for %i\n", exc.resetBkgnd);
					break;
					
				case UB:
					if ( exc.useBkgnd == TRUE )
					{
						exc.useBkgnd = FALSE;
					}
					else
					{
						exc.useBkgnd = TRUE;
					}
					printf("Use Background Now Set for %i\n", exc.useBkgnd);
					break;
					
				case FT: /* receive fringe tracker structure */
					/* need to add some things:
					 *
					 * npix_fringe (number of pixels to use in calculate fringe, default 4);
					 *
					 * */
					printf("Received Request FT..\n");

					// Load in the FT states into local memory since we have to
					// do a check for change-of-state in order to reset FT_offsets
					ftdata_old = ftdata;

					n = read(newsockfd, (unsigned int *) &message.length, 4);
					if ( n < 0 )
					{
						error(ERROR,"ERROR reading message length");
						err = 1;
					}
					else
					{
						message.data = (unsigned char *) realloc(message.data, message.length);
						n = read(newsockfd, message.data, message.length);
						ftdata = *(ft_struct *) message.data;
						//          message.data = (unsigned char *)realloc(message.data, message.length); // seems useless
						printf("%f %f %f %f %f %s\n", 
						       ftdata.dither_freq, ftdata.dither_amp, ftdata.opdamp,
						       ftdata.dither_smoothfrac, ftdata.dither_delay,
						       ftdata.opdmapsfile);
						
						for ( i = 0; i < 6 ; i++ )
						{
						  //printf("beam %i: %i %5.3f %4.2f %i %4.2f %4.2f\n",
						  //	       i, ftdata.active[i], ftdata.offset[i], ftdata.gain[i], ftdata.search_state[i],
						  //	       ftdata.search_step[i], ftdata.search_threshold[i]);
						  if ( (ftdata.active[i]) && (!ftdata_old.active[i]) )
						    ft_offsets[i] = 0.;
						  BC_Amp_Threshold[i] = ftdata.search_threshold[i];
						  BC_FT_Servo_Gains[i] = ftdata.gain[i];
						  printf("beam %i: %i %5.3f %4.2f %i %4.2f %4.2f\n",
							 i, ftdata.active[i], ftdata.offset[i], ftdata.gain[i], ftdata.search_state[i],
							 ftdata.search_step[i], ftdata.search_threshold[i]);
						}

					}

					break;

				case CO: /* ChampCommand */
					/* 6ints, sets states of each BC (999=no change) */
					n = read(newsockfd, (unsigned int *) &message.length, 4);
					if ( n < 0 )
					{
						error(ERROR,"ERROR reading message length");
						err = 1;
					}
					else
					{

						message.data = (unsigned char *) realloc(message.data, message.length);
						n = read(newsockfd, message.data, message.length);
						memcpy(socket_int6, message.data, message.length);
						printf("CO: Champ Command: %i %i %i %i %i %i\n", 
						       socket_int6[0], socket_int6[1], socket_int6[2], socket_int6[3], socket_int6[4], socket_int6[5]);
						for ( i = 0; i < 6 ; i++ )
						{
							if ( socket_int6[i] != 999 )
							  {
							    BC_FT_State_Last[i] = BC_FT_State[i];
							    BC_FT_State[i] = socket_int6[i];
							  }
						}
					}//else
					break;

                                case SR: /* Set Reference Offsets */
                                        /* 6ints, BC mask (1=use, 999=no change) 
					 6 doubles, ref offsets. */

                                        n = read(newsockfd, (unsigned int *) &message.length, 4);
                                        if ( n < 0 )
                                        {
                                                error(ERROR,"ERROR reading message length");
                                                err = 1;
                                        }
                                        else
                                        {

                                                message.data = (unsigned char *) realloc(message.data, message.length);
					        temp_ptr = message.data;
                                                n = read(newsockfd, message.data, message.length);
                                                // 6 ints + 6 doubles
						printf("SR: Champ Command: %i %i %i %i %i %i\n",
                                                       socket_int6[0], socket_int6[1], socket_int6[2], socket_int6[3], socket_int6[4], socket_int6[5]);
						
						for ( i = 0; i < 6 ; i++ )
						  {
						    socket_int6[i] = * (int * )temp_ptr;
						    temp_ptr+= sizeof(int);
						  }

						for ( i = 0; i < 6 ; i++ )
						  {
						    if ( socket_int6[i] != 999 )
						      BC_Ref_Offset[i] = * (double *)temp_ptr;
						      
						    temp_ptr+= sizeof(double);
						  }


						printf("SR: Champ Command: %i %i %i %i %i %i\n",
                                                       socket_int6[0], socket_int6[1], socket_int6[2], socket_int6[3], socket_int6[4], socket_int6[5]);
						printf("SR: received Ref Offset %f %f %f %f %f %f\n", BC_Ref_Offset[0], BC_Ref_Offset[1],
						       BC_Ref_Offset[2],  BC_Ref_Offset[3], BC_Ref_Offset[4], BC_Ref_Offset[5]);



                                        }//else
                                        break;


				case AT:
					/* Set Amplitude Thresholds */

					/* receive 6 floats at BC Amp Thresholds */
					printf("AT: Receiving Amplitude Thresholds");

                                        n = read(newsockfd, (unsigned int *) &message.length, 4);
                                        if ( n < 0 )
                                        {
                                                error(ERROR,"ERROR reading message length");
                                                err = 1;
                                        }
                                        else
                                        {

                                                message.data = (unsigned char *) realloc(message.data, message.length);
                                                n = read(newsockfd, message.data, message.length);
                                                memcpy(BC_Amp_Threshold, message.data, message.length);
                                                printf("AT: Amplitude Thresholds %f %f %f %f %f %f\n",
                                                       BC_Amp_Threshold[0], BC_Amp_Threshold[1], BC_Amp_Threshold[2], BC_Amp_Threshold[3], 
							BC_Amp_Threshold[4], BC_Amp_Threshold[5]);
						
						for(n=0; n< MAXCOMBOS /2; n++) 
						  {
						    BC_Amp_Threshold_Mean[n] = BC_FT_Auto_Threshold_Mean[n];
						    BC_Amp_Threshold_RMS[n] = BC_FT_Auto_Threshold_RMS[n];
						  }


                                        }//else
                                        break;

					

				case CS:
					/* Champ Status */

					/* send out, BC_Ref_Offset[6] 6d
					 BC_FT_Offset[6]  6d
					 BC_FT_State      6i
					 */

					// First send BC_Ref_Offset
					message.length = 6* sizeof( double );
					message.data = (unsigned char *) realloc(message.data, message.length);
					memcpy((unsigned char *) message.data, BC_Ref_Offset, message.length);
					n = write(newsockfd, (unsigned char *) message.data, message.length);
					if (n < 0)
					{
						error(ERROR,"ERROR writing to socket");
					}

					// First send BC_FT_Offset
					message.length = 6 * sizeof( double );
					message.data = (unsigned char *) realloc(message.data, message.length);
					memcpy((unsigned char *) message.data, BC_FT_Offset, message.length);
					n = write(newsockfd, (unsigned char *) message.data, message.length);
					if (n < 0) error(ERROR,"ERROR writing to socket");

					// First send BC_FT_State
					message.length = 6 * sizeof( unsigned char );
					message.data = (unsigned char *) realloc(message.data, message.length);
					memcpy((unsigned char *) message.data, BC_FT_State, message.length);
					n = write(newsockfd, (unsigned char *) message.data, message.length);
					//					printf("BC_FT_State %d %d %d %d %d %d \n", BC_FT_State[0], BC_FT_State[1],BC_FT_State[2],BC_FT_State[3],BC_FT_State[4],BC_FT_State[5]);
					if (n < 0) error(ERROR,"ERROR writing to socket");
					break;

                                case PA:
                                  /* Paddle */
                                  /* 1i, integer for which BC, 
                                     1d, double for teh relative offset to add to BC Ref_Offset
                                  */

                                  printf("PA: Paddle me\n");
                                  n = read(newsockfd, (unsigned int *) &message.length, 4);
                                  if ( n < 0 )
                                    {
                                      error(ERROR,"ERROR reading message length");
                                      err = 1;
                                    }
                                  else
                                    {
                                      message.data = (unsigned char *) realloc(message.data, message.length);
                                      temp_ptr = message.data;
                                      n = read(newsockfd, message.data, message.length);
				      
 				      // paddle integer [which BS]
				      paddle_i=* (int *) temp_ptr;
 				      temp_ptr+=sizeof(int);
				      paddle_offset = *(double * ) temp_ptr;
				      printf("PA: Paddlling %i by %f microns\n",paddle_i,paddle_offset); 
				      BC_PADDLE_Index = paddle_i; 
				      BC_PADDLE_Offset = paddle_offset;
				      BC_PADDLE_State = 1;
				    }
				  break;
					

				case SS:
				  /* Set Search Parameters */
				  /* send out: 
				     BC_Search_Mode[6]   6i  paddle, spiral, spiral
				     BC_Search_Speed[6]  6f  mu/sec
				     BC_Spiral_Step[6]   6d  millimeters
				  */

				  printf("SS: Sending Search Parameters.");
				  n = read(newsockfd, (unsigned int *) &message.length, 4);
				  if ( n < 0 )
				    {
				      error(ERROR,"ERROR reading message length");
				      err = 1;
				    }
				  else
				    {
				      message.data = (unsigned char *) realloc(message.data, message.length);
				      temp_ptr = message.data;
				      n = read(newsockfd, message.data, message.length);
				      //memcpy(socket_int6, message.data, message.length);
				      //          socket_int6 = message.data;                                                                                                                           
				      //printf("SS: Search Modes: %i %i %i %i %i %i\n", 
				      // socket_int6[0], socket_int6[1], socket_int6[2], socket_int6[3], socket_int6[4], socket_int6[5]);
                                      
				      for ( i = 0; i < 6 ; i++ )
					{
					   BC_Search_Mode[i] = * (int * )temp_ptr;
                                           temp_ptr+= sizeof(int);
					}
				      for ( i = 0; i < 6 ; i++ )
                                        {
					  BC_Search_Speed[i] = * (float *)temp_ptr;
					  BC_Sweep_Speed[i] = BC_Search_Speed[i];
					  temp_ptr+= sizeof(float);
                                        }
				      for ( i = 0; i < 6 ; i++ )
                                        {
					  BC_Spiral_Step[i] = * (double *)temp_ptr;
					  BC_Sweep_Step[i] = BC_Spiral_Step[i] ;
					  temp_ptr+= sizeof(double);
                                        }
				      printf("SS: Search Modes: %i %i %i %i %i %i\n", 
					     BC_Search_Mode[0],  BC_Search_Mode[1], BC_Search_Mode[2], 
					     BC_Search_Mode[3],BC_Search_Mode[4],  BC_Search_Mode[5]);
                                      printf("SS: Search and Sweep Speed: %f %f %f %f %f %f\n",
					     BC_Search_Speed[0],  BC_Search_Speed[1], BC_Search_Speed[2], 
					     BC_Search_Speed[3],BC_Search_Speed[4],  BC_Search_Speed[5]);
                                      printf("SS: Spiral and Sweep Step: %lf %lf %lf %lf %lf %lf\n",
					     BC_Spiral_Step[0],  BC_Spiral_Step[1], BC_Spiral_Step[2], 
					     BC_Spiral_Step[3],BC_Spiral_Step[4],  BC_Spiral_Step[5]);

				    }//else                                                            
				  break;
				  
				case BO:
				  /* Set Beam order        */
				  /* get     : 
				     BeamOrder[7]
				     --> exc.BeamOrder[6]   6i
				     --> exc.RefTel   i
				  */

				  printf("BO: Receiving Beam Order Parameters.");
				  n = read(newsockfd, (unsigned int *) &message.length, 4);
				  if ( n < 0 )
				    {
				      error(ERROR,"ERROR reading message length");
				      err = 1;
				    }
				  else
				    {
				      message.data = (unsigned char *) realloc(message.data, message.length);
				      temp_ptr = message.data;
				      n = read(newsockfd, message.data, message.length);
                                      
				      for ( i = 0; i < 6 ; i++ )
					{
					   exc.BeamOrder[i] = * (int * )temp_ptr;
                                           temp_ptr+= sizeof(int);
					}

				      exc.RefTel =  * (int * )temp_ptr;

                                      printf("BO: BeamOrder: %d %d %d %d %d %d Ref: %d\n",
					     exc.BeamOrder[0],  exc.BeamOrder[1], exc.BeamOrder[2], 
					     exc.BeamOrder[3],  exc.BeamOrder[4], exc.BeamOrder[5], exc.RefTel);

				    }
                                                                                                                                      
				  
				  break;

                                case CP:
                                  /* Set Champ servo Parameters */
                                  /* receive information only
					BC_FT_Servo_Gains[6]		6f
					BC_FT_StoSL_SNR_Threshold[6]	6f
                                        BC_FT_SLtoL_SNR_Threshold[6]    6f
                                        BC_FT_LtoSL_SNR_Threshold[6]    6f
                                        BC_FT_SLtoS_SNR_Threshold[6]    6f
                                  */

                                  printf("CP: Set Servo Parameters\n");
                                  n = read(newsockfd, (unsigned int *) &message.length, 4);
                                  if ( n < 0 )
                                    {
                                      error(ERROR,"ERROR reading message length");
                                      err = 1;
                                    }
                                  else
                                    {
                                      message.data = (unsigned char *) realloc(message.data, message.length);
                                      temp_ptr = message.data;
                                      n = read(newsockfd, message.data, message.length);
                                      memcpy(temp_ptr, message.data, message.length);
                                      //          socket_int6 = message.data;                                                                                                                           
                                      //printf("SS: Search Modes: %i %i %i %i %i %i\n", 
                                      // socket_int6[0], socket_int6[1], socket_int6[2], socket_int6[3], socket_int6[4], socket_int6[5]);

                                      for ( i = 0; i < 6 ; i++ )
                                        {
                                           BC_FT_Servo_Gains[i] = * (float * )temp_ptr;
                                           temp_ptr+= sizeof(float);
                                        }
                                      for ( i = 0; i < 6 ; i++ )
                                        {
					  BC_FT_GDT_Gains[i] = * (float * )temp_ptr;
					  temp_ptr+= sizeof(float);
                                        }

                                      for ( i = 0; i < 6 ; i++ )
                                        {
                                          BC_FT_StoSL_SNR_Threshold[i] = * (float *)temp_ptr;
					  BC_Sweep_SNR_Threshold[i] = BC_FT_StoSL_SNR_Threshold[i] ;
                                          temp_ptr+= sizeof(float);
                                        }
                                      for ( i = 0; i < 6 ; i++ )
                                        {
                                          BC_FT_SLtoL_SNR_Threshold[i] = * (float *)temp_ptr;
                                          temp_ptr+= sizeof(float);
                                        }
                                      for ( i = 0; i < 6 ; i++ )
                                        {
                                          BC_FT_LtoSL_SNR_Threshold[i] = * (float *)temp_ptr;
                                          temp_ptr+= sizeof(float);
                                        }
                                     for ( i = 0; i < 6 ; i++ )
                                        {
                                          BC_FT_SLtoS_SNR_Threshold[i] = * (float *)temp_ptr;
                                          temp_ptr+= sizeof(float);
                                        }
                                     for ( i = 0; i < 6 ; i++ )
				       {
					 if( (* (int *)temp_ptr ) != NACCUMULATIONS[i])
					   { 
					     NACCUMULATIONS[i] = * (int *)temp_ptr;
					     NACCUMULATIONS_CHANGED[i] = TRUE;
					   }
					 temp_ptr+= sizeof(int);
				       }

                                      printf("CP: BC_FT_Servo_Gains: %f %f %f %f %f %f\n",
                                             BC_FT_Servo_Gains[0],  BC_FT_Servo_Gains[1], BC_FT_Servo_Gains[2],
                                             BC_FT_Servo_Gains[3],BC_FT_Servo_Gains[4],  BC_FT_Servo_Gains[5]);
                                      printf("CP: BC_FT_GDT_Gains: %f %f %f %f %f %f\n",
                                             BC_FT_GDT_Gains[0],  BC_FT_GDT_Gains[1], BC_FT_GDT_Gains[2],
                                             BC_FT_GDT_Gains[3],BC_FT_GDT_Gains[4],  BC_FT_GDT_Gains[5]);

                                      printf("BC_FT_StoSL_SNR_Threshold and Sweep_SNR_Threshold: %f %f %f %f %f %f\n",
                                             BC_FT_StoSL_SNR_Threshold[0], BC_FT_StoSL_SNR_Threshold[1], BC_FT_StoSL_SNR_Threshold[2], 
                                             BC_FT_StoSL_SNR_Threshold[3], BC_FT_StoSL_SNR_Threshold[4], BC_FT_StoSL_SNR_Threshold[5]);
                                      printf("BC_FT_SLtoL_SNR_Threshold: %f %f %f %f %f %f\n",
                                             BC_FT_SLtoL_SNR_Threshold[0], BC_FT_SLtoL_SNR_Threshold[1], BC_FT_SLtoL_SNR_Threshold[2],
                                             BC_FT_SLtoL_SNR_Threshold[3], BC_FT_SLtoL_SNR_Threshold[4], BC_FT_SLtoL_SNR_Threshold[5]);
                                      printf("BC_FT_StoSL_SNR_Threshold: %f %f %f %f %f %f\n",
                                             BC_FT_LtoSL_SNR_Threshold[0], BC_FT_LtoSL_SNR_Threshold[1], BC_FT_LtoSL_SNR_Threshold[2],
                                             BC_FT_LtoSL_SNR_Threshold[3], BC_FT_LtoSL_SNR_Threshold[4], BC_FT_LtoSL_SNR_Threshold[5]);
                                      printf("BC_FT_StoSL_SNR_Threshold: %f %f %f %f %f %f\n",
                                             BC_FT_SLtoS_SNR_Threshold[0], BC_FT_SLtoS_SNR_Threshold[1], BC_FT_SLtoS_SNR_Threshold[2],
                                             BC_FT_SLtoS_SNR_Threshold[3], BC_FT_SLtoS_SNR_Threshold[4], BC_FT_SLtoS_SNR_Threshold[5]);
				      printf("NACCUMULATIONS %i %i %i %i %i %i\n",
					     NACCUMULATIONS[0], NACCUMULATIONS[1], NACCUMULATIONS[2], NACCUMULATIONS[3], NACCUMULATIONS[4],
					     NACCUMULATIONS[5]);

                                    }//else                                                            
				    break;

			case CT:
			  /* Calculate Amplitude Thresholds */
			  /* which fringe  	1i		INPUT
			     Report Threshold	1f		OUPUT 
			  */
			  
			  //Length of nessage = 4 chars (int). this could differ in other architecture
			  // This is not really the length but rather is the fringe to get calc amp
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      // allocate data buffer
			      message.data = (unsigned char *) realloc(message.data, sizeof(float));
			      
			      // FB: later we will need to deal with BIAS					
			      j = message.length;
			      if (j < 0 || j > 5) {
				printf("BC index value out of range. must be 0-5\n");
				break;
			      }
			      
			      //FB: ideally, need a case when autoThresholdcoutner< AUTONUM
			      BC_FT_Auto_Threshold_Mean[j] = 0.0;
			      for (i=0; i < AUTONUM; i++)			       
				  BC_FT_Auto_Threshold_Mean[j] += autoThreshold[j][i]; 			      
			      BC_FT_Auto_Threshold_Mean[j] /= AUTONUM;
			      for (i=0; i < AUTONUM; i++)
				BC_FT_Auto_Threshold_RMS[j] += 
				  (autoThreshold[j][i]-BC_FT_Auto_Threshold_Mean[j])*(autoThreshold[j][i]-BC_FT_Auto_Threshold_Mean[j]);

			      BC_FT_Auto_Threshold_RMS[j] = sqrt(BC_FT_Auto_Threshold_RMS[j]/(AUTONUM-1)) ;			     

			     // Round up
			      // BC_FT_Auto_Threshold_Mean[j] = ceil(BC_FT_Auto_Threshold_Mean[j]) ;
			      // BC_FT_Auto_Threshold_RMS[j]  = ceil(BC_FT_Auto_Threshold_RMS[j]) ;
			      printf("CT: Calculate Threshold for BC %i : Mean: %f RMS: %f\n",j, BC_FT_Auto_Threshold_Mean[j], BC_FT_Auto_Threshold_RMS[j]);			     			      


			      memcpy( (unsigned char *) message.data, &BC_FT_Auto_Threshold_Mean[j] , sizeof(float) );
			      // send data through socket
			      n = write(newsockfd, (unsigned char *) message.data, sizeof(float));
			      
			      if ( n < 0 )
				error(ERROR,"ERROR writing to socket");
			    }
			  break;
			  
			case IR:
			  /* Initialize Reference. Move FTdata ref -> bc ref offset */
			  /* which fringe         1i              INPUT
			  */

			  //Length of nessage = 4 chars (int). this could differ in other architecture                                                                                              
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      // allocate data buffer                                                                                                                                           
			      message.data = (unsigned char *) realloc(message.data, sizeof(float));
			      j=message.length;
			      printf("IR: Init Reference for BC %i \n",j);

			      if (j < 0 || j > 5)
				{
				  printf("index value out of range. must be 0-5\n");
				  break;
				}
			      BC_INIT_Ref_Index = j;
			      BC_INIT_Ref_State = 1;
			    }
			  break;
                    

			case RR:
			  /* Reset Reference. Move back  bc ref offset -> FTdata ref */
			  /* which fringe         1i              INPUT
			  */
			  // FB, added July 2011, still TBD
			  //Length of nessage = 4 chars (int). this could differ in other architecture                                                                                              
			  n = read(newsockfd, (unsigned int *) &message.length, 4);
			  if ( n < 0 )
			    {
			      error(ERROR,"ERROR reading message length");
			      err = 1;
			    }
			  else
			    {
			      // allocate data buffer                                                                                                                                           
			      message.data = (unsigned char *) realloc(message.data, sizeof(float));
			      j=message.length;
			      printf("RR: Reset Reference for BC %i\n",j);

			      if (j < 0 || j > 5)
				{
				  printf("index value out of range. must be 0-5\n");
				  break;
				}
			      BC_INIT_Ref_Index = j;
			      BC_INIT_Ref_State = 2;
			    }
			  break;
                    
			default:
			  // do nothing
			  break;
			}
			
		}
		if (!err)
		{
			n = write(newsockfd,"OK",2);
			if (n < 0) error(ERROR,"ERROR writing to socket");
		}
		else
		{
			n = write(newsockfd,"KO",2);
			if (n < 0) error(ERROR,"ERROR writing to socket");
		}

		//printf("Here is the message: %i\n",message.length);
		//n = write(newsockfd,"I got your message",18);
		//if (n < 0) error(ERROR",ERROR writing to socket");
		close(newsockfd);
	} /* end BIG while */

	// Free image memory
	free(message.data);
	close(sockfd);
	close(vme_sd);
	if (mem_fd != NULL)
		munmap((char *)mem_fd, BUFFER_SIZE);
	close(pci_fd);
	fflush(NULL);
	fprintf(stdout,"Stopping CHAMP/CHARA server\n");
	free(mess.data);
	stop_chara_server();
	fprintf(stdout,"Exit - ok\n");
	return 0;
}

/************************************************************************/
/*  using reverse_byte_order() from control/libs/rwfits/fits_util.c     */
/************************************************************************/
/* reverse_byte_order()                                                 */
/*                                                                      */
/* Reverses the byte order from the pointer location including the      */
/* given number of bytes.                                               */
/************************************************************************/

void reverse_byte_order(void *p, int nbytes)
{
	unsigned char *c1, *c2;
	int i;
	unsigned char *np;

	/* Allocate enough memory to do this */

#ifdef DEBUG2
	printf("nbytes = %d\n",nbytes);
	fflush(stdout);
#endif
	np = (unsigned char *) malloc(nbytes * sizeof(unsigned char));
	// mem_test(np);  // test to see if out of memory -- not needed!!

	/* Make a copy of the bytes */

	for ( c1 = p, c2 = np, i = 0; i < nbytes ; i++, c1++, c2++ )
	{
		*c2 = *c1;
	}

	/* Now copy it back in reverse order */

	for ( c1 = p, c2 = np + nbytes - 1, i = 0; i < nbytes ; i++, c1++, c2-- )
	{
		*c1 = *c2;
	}

#ifdef DEBUG2
	for(c1 = p, c2 = np, i=0; i<nbytes; i++,c1++,c2++)
	{
		printf("%d %d\n",*c2,*c1);
	}
	printf("\n");
	fflush(stdout);
#endif

	/* Release the temp memory space */

	free(np);

} /* reverse_byte_order() */

/********************************************************************************/
/*  open_connect_socket()        taken from /control/libs/charaui/src/socket.c  */
/************************************************************************/
/* open_connect_socket()                                                */
/*                                                                      */
/* Attempts to open up a socket port for connection. Requires a         */
/* machine name and a port number. Returns the socket file descriptor   */
/* if successful or a -1 if it fails.                                   */
/* The corresponding function close_connect_socket() is a simple        */
/* macro for close().                                                   */
/* This may not work so well for STANDALONE machines...                 */
/************************************************************************/

int open_connect_socket(char *machine, int port)
{
	struct sockaddr_in server; /* Server address */
	struct hostent *hp; /* Pointer to host data */
	int sock; /* Socket number */
	int buf_size; /* Size of buffer */

	/* removing error checking!! 2006Mar30 JDM */
	/* First see if we can identify the host. */

	if ( (hp = gethostbyname(machine)) == 0 )
	{
		printf("Host %s unknown.", machine);
		return -1;
	}

	/* We can not connect to ourselves. */

	/*         if (strcmp(hp->h_name,myname) == 0 && port == myport) */
	/*         { */
	/*                 return (WARNING,"We can not connect to ourselves.\n"); */
	/*         } */

	/* Create a socket */

	if ( (sock = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
	{
		printf("Failed to create socket.\n");
		return -1;
	}

	/* Force buffer sizes to SOCKET_BUFFER */

	buf_size = SOCKET_BUFFER;

	if ( setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *) &buf_size, sizeof(buf_size)) < 0 )
	{
		close(sock);
		printf("Failed to change send buffer size to %d\n", buf_size);
		return -1;
	}
	if ( setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (char *) &buf_size, sizeof(buf_size)) < 0 )
	{
		close(sock);
		printf("Failed to change receive buffer size to %d\n", buf_size);
		return -1;
	}

	/* Create server address */

	server.sin_family = AF_INET;
	strncpy((char *) &server.sin_addr, (char *) hp->h_addr, hp->h_length);
	server.sin_port = htons(port);

	/* Let's try and connect now.... */

	if ( connect(sock, (struct sockaddr *) &server, sizeof(server)) < 0 )
	{
		close(sock);
		printf("Failed to connect to %s port %d.", machine, port);
		return -1;
	}

	/* All is well it seems */

	return sock;

} /* open_connect_socket() */

/************************************************************************/
/* char_waiting()                                                       */
/*                                                                      */
/* Unix version of kbhit like function. Returns 1 if a character        */
/* waiting at the port, 0 otherwise.                                    */
/* Uses the select(2) routine to do it's work.                          */
/* From background.c and stripped .                                     */
/************************************************************************/

//int char_waiting(int fd)
//{
	/* Locals */

//	fd_set rfds; /* File discriptor set */
//	struct timeval tv; /* Time to wait */

	/* Set tings up */

//	FD_ZERO(&rfds); /* Zero the file discriptor set */
//	FD_SET(fd, &rfds); /* Add fd to the set */
//	tv.tv_sec = 0; /* Wait no more than 1 usec */
//	tv.tv_usec = 1;

//	if ( select(FD_SETSIZE, &rfds, NULL, NULL, &tv) < 0 )
//	{
		/* Something is wrong. */

//	perror("char_waiting ");
//		return 0;
//}

//if ( FD_ISSET(fd, &rfds) ) /* Was there something? */
//{
//	return 1;
///}

	/* Must have been nothing there */

	//return 0;

//} /* char_waiting() */















/* the pixelmap file will match the way the image looks. this means the nrows (y) are flipped!
 *  so this will need fixed up here */

int ReadPixelMapFile(int rows, int cols, int *pixelMap)
{
	int i, j, r, c, n;
	FILE *fp;

	if ( (fp = fopen(PIXELMASK, "r")) == NULL )
	{
		printf("ERROR:  can't open pixel mask file for reading\n");
		return 1;
	}
	else
	{
		printf("Opened pixel mask file for reading\n");
	}

	i = 0;
	while (!feof(fp))
	{
		r = i / cols;
		c = i - (r * cols);
		j = (rows - 1 - r) * cols + c;
		fscanf(fp, "%i", &n);
		if ( n < 0 || n > MAXCOMBOS )
		{
			printf("Illegal value for combination identifier: %i\n", n);
			pixelMap[j] = 0;
		}
		else
		{
			pixelMap[j] = n;
		}
		//printf("i,n, c,r,j %i %i %i %i %i\n",i,n,c,r,j);
		/* JDM: I should create a list of pixels and average locations in array */

		i++;
	}

	fclose(fp);

	printf("Number pixels read by ReadPixelMapFile: %i\n", i - 1);
	if ( (rows * cols) != (i - 1) )
	{
		printf("WARNING:  size of pixel map is NOT equal to subarray size\n");
	}

	return 0;
}


// 2009Aug18 JDM Adding in all National Instruments stuff here.
//
/*********************************************************************
 *
 * ANSI C Example program:
 *    contGeneration.c
 *
 * Example Category:
 *    AO
 *
 * Description:
 *    This example demonstrates how to output a continuous periodic
 *    waveform using an internal sample clock.
 *
 * Instructions for Running:
 *    1. Select the Physical Channel to correspond to where your signal
 *       is output on the DAQ device.
 *    2. Enter the Minimum and Maximum Voltage Ranges.
 *    3. Enter the desired rate for the generation. The onboard sample
 *       clock will operate at this rate.
 *    4. Select the desired waveform type.
 *    5. The rest of the parameters in the Waveform Information section
 *       will affect the way the waveform is created, before it's sent
 *       to the analog output of the board. Select the number of samples
 *       per cycle and the total number of cycles to be used as waveform
 *       data.
 *
 * Steps:
 *    1. Create a task.
 *    2. Create an Analog Output Voltage channel.
 *    3. Define the update Rate for the Voltage generation. Additionally,
 *       define the sample mode to be continuous.
 *    4. Write the waveform to the output buffer.
 *    5. Call the Start function.
 *    6. Loop continuously for 10 seconds. Check for errors every
 *       100 ms using the IsTaskDone function.
 *    7. Call the Clear Task function to clear the Task.
 *    8. Display an error if any.
 *
 * I/O Connections Overview:
 *    Make sure your signal output terminal matches the Physical Channel
 *    I/O Control. For further connection information, refer to your
 *    hardware reference manual.
 *
 * Recommended use:
 *    Call Configure, Write and Start functions.
 *    Call IsDone function in a loop.
 *    Call Stop function at the end.
 *
 *********************************************************************/

int NIDAQ_modtrig(int command, float dither_freq, float dither_amplitude, float dither_smoothfrac, float dither_delay)
{
	int32 i, c;
	char errBuff[2048] = { '\0' };
	int32 err_modtrig = 0;

	// Channel parameters
	char chan[] = "Dev1/ao0:3";
	float64 min = -10.0;
	float64 max = 10.0;

	// Timing parameters
	float64 sample_rate;

	// Data write parameters
	int32 data_size = NIDAQ_BUFFERSIZE * NIDAQ_NUMCHANNELS;
	float64 waveform[NIDAQ_BUFFERSIZE], waveform1[NIDAQ_BUFFERSIZE], waveform2[NIDAQ_BUFFERSIZE];
	uInt8 ttl[NIDAQ_BUFFERSIZE];
	float64 data[data_size];
	int32 pointsWritten;
	float64 timeout = 10.0;
	//FILE	*fp;

	// Clock parameters
	float64 clock_freq;

	// Trigger parameters
	float pulseWidth;
	float64 trigger_delay;
	float64 dutyCycle;
	int32 lowTicks, highTicks;

	// Misc
	float freq;
	int32 samples_per_chan = 0;
	int32 smooth_width;

	switch (command)
	{

		case MODTRIG_START:

			/* get the frequency */
			freq = dither_freq;
			printf("Dither frequency: %0.1f Hz\n", freq);

			/* init the vectors */
			for ( i = 0; i < NIDAQ_BUFFERSIZE ; i++ )
			{
				waveform[i] = 0;
				waveform1[i] = 0;
				waveform2[i] = 0;
			}
			for ( i = 0; i < data_size ; i++ )
			{
				data[i] = 0;
			}
			printf("Data size: %ld\n", data_size);

			/* set the number of samples per channel */
			samples_per_chan = NIDAQ_BUFFERSIZE;
			printf("Number of samples per channel: %f\n", (float) samples_per_chan);

			/* make a triangle wave */
			/* JDM puts in an empirical phase offset factor, assuming 0.05 smoothfrac  */
			/* previous suggestion is 2*pi*dither_smoothfrac */

			if(DAQ_WAVEFORM == 1)
			  NIDAQ_TriangleWave(samples_per_chan, dither_amplitude, dither_delay * 2. * PI, waveform);
			else 
			  NIDAQ_SawToothWave(samples_per_chan, dither_amplitude, dither_delay * 2. * PI, waveform);

			/* smooth it by SMOOTH_FRAC */
			smooth_width = (int) (samples_per_chan * dither_smoothfrac);

			printf("Smoothing boxcar width per cycle: %ld (%.0f%%)\n", smooth_width, dither_smoothfrac * 100.);
			
			if (DAQ_WAVEFORM == 1)
			  NIDAQ_BoxcarSmooth(samples_per_chan,waveform,smooth_width,waveform2); // Triangle
			else
			  {
			    // if SawTooth must smooth twice by half as much.
			    NIDAQ_BoxcarSmooth(samples_per_chan, waveform, smooth_width / 2, waveform1);
			    NIDAQ_BoxcarSmooth(samples_per_chan, waveform1, smooth_width / 2, waveform2);
			  }

			/* what sample rate (in samples/second) do we need */
			/* to get the freq we want? */
			sample_rate = freq * samples_per_chan;
			printf("Sample rate: %f samples/ms\n", sample_rate / 1000);

			/* the TTL pulse */
			NIDAQ_Trigger(samples_per_chan, sample_rate / 1000, ttl);

			/* write the data (grouped by channel) for multiple channels */
			for ( i = 0; i < samples_per_chan ; i++ )
			{
				for ( c = 0; c < NIDAQ_NUMAOCHANNELS ; c++ )
					data[i + c * samples_per_chan] = waveform2[i];
				data[i + 3* samples_per_chan ] = ttl[i];
			}

			/* write the data to a file for posterity */
			//	fp = fopen("waveform.dat", "w");
			//        for (i=0; i<data_size; i++){
			//		fprintf(fp,"%ld %f\n",i,data[i]);
			//	}
			//	fclose(fp);


			/*
			 * MODULATION signal
			 */

			/* Create the analog out task */
			DAQmxErrChk(DAQmxCreateTask("modulation", &NIDAQ_mod_taskHandle));

			/* Create the channel(s) and add them to the task */
			DAQmxErrChk(DAQmxCreateAOVoltageChan(NIDAQ_mod_taskHandle, chan, "", min, max, DAQmx_Val_Volts, NULL));

			/* Set the source and rate of the clock and the number of samples */
			printf("Set the clock source and rate... ");
			DAQmxErrChk(DAQmxCfgSampClkTiming(NIDAQ_mod_taskHandle, "Ctr0InternalOutput", sample_rate, DAQmx_Val_Rising, DAQmx_Val_ContSamps, samples_per_chan));
			printf("Done\n");

			/* Write the data to the buffer */
			printf("Writing waveform data to the buffer... ");
			DAQmxErrChk(DAQmxWriteAnalogF64(NIDAQ_mod_taskHandle, samples_per_chan, 0, // auto-start set to false
					timeout, DAQmx_Val_GroupByChannel, data, &pointsWritten, NULL));
			printf("Done\n");
			printf("Number of points written per channel:  %ld\n", pointsWritten);

			/* sync the the mod signal with the trigger */
			DAQmxErrChk(DAQmxCfgDigEdgeStartTrig(NIDAQ_mod_taskHandle, "/Dev1/Ctr0InternalOutput", DAQmx_Val_Rising));

			/* start the task */
			DAQmxErrChk(DAQmxStartTask(NIDAQ_mod_taskHandle));

			/*
			 * CLOCK
			 */

			clock_freq = freq * samples_per_chan;
			printf("Clock frequency: %f Hz\n", clock_freq);

			/* create the clock task */
			DAQmxErrChk(DAQmxCreateTask("clock", &NIDAQ_clock_taskHandle));

			/* setup the clock */
			DAQmxErrChk(DAQmxCreateCOPulseChanFreq(NIDAQ_clock_taskHandle, "Dev1/ctr0", "", DAQmx_Val_Hz, DAQmx_Val_Low, 0.0, clock_freq, 0.5));

			/* implicit timing */
			DAQmxErrChk(DAQmxCfgImplicitTiming(NIDAQ_clock_taskHandle, DAQmx_Val_ContSamps, 0));

			/*
			 * TRIGGER
			 */
			pulseWidth = 0.5; //in milliseconds
			if ( (dutyCycle = (pulseWidth / 1000.) * clock_freq) < 2 )
			{
				dutyCycle = 2;
				printf("WARNING:  dutyCycle too low, setting to %f (%f ms)\n", dutyCycle, dutyCycle * 1000 / clock_freq);
			}
			else
			{
				printf("dutyCycle: %f\n", dutyCycle);
			}

			/* phase offset between trigger and modulation signal */
			trigger_delay = samples_per_chan / 4. - smooth_width / 2;

			highTicks = dutyCycle;
			lowTicks = samples_per_chan / 2 - highTicks;

			/* create the task */
			DAQmxErrChk(DAQmxCreateTask("trigger", &NIDAQ_trigger_taskHandle));

			/* setup the trigger in terms of clock ticks */
			DAQmxErrChk(DAQmxCreateCOPulseChanTicks(NIDAQ_trigger_taskHandle, "Dev1/ctr1", "", "Ctr0InternalOutput", DAQmx_Val_Low,
					//trigger_delay,
					0, lowTicks, highTicks));

			DAQmxErrChk(DAQmxCfgImplicitTiming(NIDAQ_trigger_taskHandle, DAQmx_Val_ContSamps, 0));

			/* start the trigger when the clock starts */
			DAQmxErrChk(DAQmxCfgDigEdgeStartTrig(NIDAQ_trigger_taskHandle, "Ctr0InternalOutput", DAQmx_Val_Rising));

			DAQmxErrChk(DAQmxStartTask(NIDAQ_trigger_taskHandle));

			/* start the clock, which starts the mod and trig */
			DAQmxErrChk(DAQmxStartTask(NIDAQ_clock_taskHandle));

			printf("Done with start trigger\n");

			break; /* end START case*/

		case MODTRIG_STOP:

			/* if the task has let go, stop and clear it */
			if ( NIDAQ_mod_taskHandle != 0 )
			{
				printf("Stopping and Clearing Mod Task... ");
				DAQmxErrChk(DAQmxStopTask(NIDAQ_mod_taskHandle));
				DAQmxErrChk(DAQmxClearTask(NIDAQ_mod_taskHandle));
				printf("Done\n");
			}

			if ( NIDAQ_trigger_taskHandle != 0 )
			{
				printf("Stopping and Clearing Trigger Task... ");
				DAQmxErrChk(DAQmxStopTask(NIDAQ_trigger_taskHandle));
				DAQmxErrChk(DAQmxClearTask(NIDAQ_trigger_taskHandle));
				printf("Done\n");
			}

			if ( NIDAQ_clock_taskHandle != 0 )
			{
				printf("Stopping and Clearing clock Task... ");
				DAQmxErrChk(DAQmxStopTask(NIDAQ_clock_taskHandle));
				DAQmxErrChk(DAQmxClearTask(NIDAQ_clock_taskHandle));
				printf("Done\n");
			}

			break; /* end STOP case*/

		case MODTRIG_RESET:

			/* Reset the device */
			printf("Resetting NI device... ");
			DAQmxErrChk(DAQmxResetDevice("Dev1"));
			printf("Done\n");

			break; /* end RESET case*/

	} /* end switch */

 ErrLabel:
	/* if there was an error, get a more user-friendly message */
	if ( DAQmxFailed(err_modtrig) )
	{
		printf("\n\nERROR ERROR ERROR\n\n");
		DAQmxGetExtendedErrorInfo(errBuff, 2048);
	}

	/* report the error if there was one */
	if ( DAQmxFailed(err_modtrig) )
		printf("DAQmx Error: %s\n", errBuff);

	return 0;
}

/* function to create a triangle waveform */
int NIDAQ_TriangleWave(int number, float amplitude, float phase_offset, float64 *data)
{
	int i;
	float64 phase;

	for ( i = 0; i < number ; i++ )
	{
		phase = 2.0 * PI * (float) i / (float) number + phase_offset;
		// data amp is centered on 5 (middle of DAQ range
		*data++ = amplitude * (asin(sin(phase+0.5*PI)) / PI) + 5.;
	}

	return 0;
}

/* function to create a triangle waveform */
int NIDAQ_SawToothWave(int number, float amplitude, float offset, float64 *data)
{
	// offset is in fractions of sawtooth
	int i;
	float64 phase;
	float64 intpart;

	for ( i = 0; i < number ; i++ )
	{
		phase = 2.0 * (float) i / (float) number + offset;
		// data amp is centered on 5 (middle of DAQ range
		*data++ = amplitude * (modf(phase, &intpart) - .5) + 5.;
		//		printf("SawTooth bin, amp:%f6.2 %f6.2\n",phase,amplitude*(modf(phase,&intpart)-.5)+5.);
	}
	return 0;
}

/* function to create a triangle waveform */
int NIDAQ_Trigger(int number, int width, uInt8 *data)
{
	int i;

	for ( i = 0; i < number ; i++ )
	{
		if ( (i > 0) && (i < width) )
		{
			*data++ = 1;
		}
		else
		{
			*data++ = 0;
		}
	}

	return 0;
}

/* boxcar smoothes any waveform */
// JDM will smooth around cycliclally [worked ok before too!]
int NIDAQ_BoxcarSmooth(int number, float64 *data, int width, float64 *data_sm)
{
	float64 sum;
	int i, j, index;

	for ( i = 0; i < number ; i++ )
	{
		sum = 0.0;
		for ( j = 0; j < width - 1 ; j++ )
		{
			index = i + j - width / 2;
			if ( index >= number )
				index = index - number;
			if ( index < 0 )
				index = index + number;
			sum += data[index];
		}
		sum /= width;
		*data_sm++ = sum;
	}

	return 0;
}

/*  routine to make sending VME commands more transparent. */
// this routine could be sped up immensely!! lots of loops JDM
void send_champ2vme(int *BeamOrder, int RefTel, int nbeams, double *BC_Ref_Offset, double *BC_FT_Offset)
{
	int n;
	//double lab_dl_values[6];
	double dummy_wts[6] = { 2., 2., 2., 2., 2., 2. };
	double combined_OPD0[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
	double combined_OPD1[6] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
	double refOffset;
	// nbeams == number of beams using. need to match champ_bs4.

	// it is CRITICAL that the OFFSETS HERE are always internally consistent with a DELAY solution.
	// this program will give each offset equal weight so this will lead to bad things if
	// there is an internal inconsistency.. basically, the program needs to enforce the closure
	// relations EVERY TIME there is a change to REF OFFSET or FT_OFFSET
	//

	// loop through all combiners
	for ( n = 0; n < nbeams ; n++ )
	{
		combined_OPD0[n] = BC_Ref_Offset[n] + BC_FT_Offset[n];
	}

	if ( nbeams == 4 )
	  champ_bs4(dummy_wts, combined_OPD0, combined_OPD1, vme_beam_offsets);
	if ( nbeams == 6 )
	  champ_bs6(dummy_wts, combined_OPD0, combined_OPD1, vme_beam_offsets);
	
	if ( 0 )
	{//DEBUG
		printf("dummy_wts, combined_OPD0, combined_OPD1, vme_beam_offsets\n");
		printf("%f %f %f %f %f %f \n", dummy_wts[0], dummy_wts[1], dummy_wts[2], dummy_wts[3], dummy_wts[4], dummy_wts[5]);
		printf("%f %f %f %f %f %f \n", combined_OPD0[0], combined_OPD0[1], combined_OPD0[2], combined_OPD0[3], combined_OPD0[4], combined_OPD0[5]);
		printf("%f %f %f %f %f %f \n", combined_OPD1[0], combined_OPD1[1], combined_OPD1[2], combined_OPD1[3], combined_OPD1[4], combined_OPD1[5]);
		printf("%f %f %f %f %f %f \n", vme_beam_offsets[0], vme_beam_offsets[1], vme_beam_offsets[2], vme_beam_offsets[3], vme_beam_offsets[4], vme_beam_offsets[5]);
	}
	// only uses first 4 elements
	for ( n = 0; n < nbeams ; n++ )
	{
		vme_dl_offsets[BeamOrder[n]] = (double) vme_beam_offsets[n] * (double) 1e-6;
	}

	refOffset = vme_dl_offsets[RefTel];
	// now subtract from REFTEL
	for ( n = 0; n < nbeams ; n++ )
	{
		vme_dl_offsets[BeamOrder[n]] = vme_dl_offsets[BeamOrder[n]] - refOffset;
		vme_dl_offsets_byteswap[BeamOrder[n]] = vme_dl_offsets[BeamOrder[n]];
		byte_swap(vme_dl_offsets_byteswap[BeamOrder[n]]);
	}
	write(vme_sd, &vme_dl_offsets_byteswap, sizeof(vme_dl_offsets_byteswap));

	if ( 0 )
	{
		printf("champ2vme time: %f dls: %f %f %f %f %f %f \n", (double) (rt_timer_read() / ((double) 1000000)), vme_dl_offsets[0], vme_dl_offsets[1], vme_dl_offsets[2],
				vme_dl_offsets[3], vme_dl_offsets[4], vme_dl_offsets[5]);
	}

}


int slice_history_transfer( float* data_to_transfer, message_struct* message, unsigned int current_position, unsigned int wrap, unsigned int history_size  )
// We transfer all from the last point (not included) to the new one (included)                                                                                                     
// The data to transfer is ( i % history_size) * wrap + n 
{
  int temp, temp1, temp2; 
  if( (current_position - message->length + 1) >= history_size) // catchup as we can...
    {
      message->length = history_size;
      temp = wrap * message->length * sizeof(float) + sizeof(int) ;
      message->data = (unsigned char *) realloc(message->data, temp +  sizeof(int) );
      memcpy( message->data, &temp, sizeof(int) ); // size of the incoming transfer 
      memcpy( &message->data[ sizeof(int) ], &current_position, sizeof(int) ); // current frame
      memcpy( &message->data[ 2 * sizeof(int) ], &data_to_transfer[0], temp - sizeof(int) );
    }
  else
    {
      temp1 = current_position  % history_size; 
      message->length = message->length % history_size; 

      if( temp1 > message->length) // in general we will be executing this  9>5
	{
	  temp = wrap * ( ( temp1 - message->length ) * sizeof(float) ) + sizeof(int) ; // 9-5 = 4 (6, 7, 8, 9)
	  message->data = (unsigned char *) realloc(message->data, temp +  sizeof(int) );
	  memcpy( message->data, &temp, sizeof(int) ); // size of the incoming transfer                                                                                  
	  memcpy( &message->data[ sizeof(int) ], &current_position, sizeof(int) ); // current frame                                 	  
	  memcpy( &message->data[ 2 * sizeof(int) ], &data_to_transfer[ ( message->length + 1 ) * wrap ], temp - sizeof(int) ); // start at 6
	}
      else // wrapping  temp1 = message->length or temp1 < message->length    temp = 5  mess = 6 FTMEM = 10   7, 8, 9  + 0, 1, 2, 3, 4, 5 = 3 + 6 = FTMEM - mess + temp 
	{
	  temp = wrap * ( ( history_size + temp1 - message->length ) * sizeof(float) ) + sizeof(int)  ; 
	  message->data = (unsigned char *) realloc(message->data, temp  +  sizeof(int));
	  memcpy( message->data, &temp, sizeof(int)); // size of incoming transfer                                                                                       
	  memcpy( &message->data[ sizeof(int) ], &current_position  , sizeof(int) ); // current frame                                                                                
	  temp2 = ( history_size - 1 - message->length ) * sizeof(float) * wrap ; // 
	  memcpy( &message->data[ 2 * sizeof(int) ], &data_to_transfer[ ( message->length + 1 ) * wrap  ], temp2 );
	  memcpy( &message->data[ 2 * sizeof(int) + temp2 ], &data_to_transfer[ 0 ], (temp1 + 1 ) * wrap * sizeof(float));
	}
    }
  return temp;
}


int initialize_shutters(void)
{
  int ret = 0;
  struct smessage mess;
    time_t	 start;
  
  /* Initialise mesage jobs */
  
  printf("Initiliazing CHARA jobs\n");
  init_jobs();
  
  /* Open the sockets */
  
  if ((shutter_server = open_server_by_name("shutters")) == -1)
    {
      fprintf(stderr,"Failed to open process shutters");
      exit(-1);
    }
  
  /* Setup up local message servers */
  
  if (!setup_messages()) 
    {
      fprintf(stderr,"Could not setup messages.");
    }
  
  /* Send out requests for machine name and port names */
  
  *(shutter_conf.name) = 0;
  mess.type = SHUTTER_CONF;
  mess.length = 0;
  mess.data = NULL;
  
  if (!send_message(shutter_server, &mess))
    {
      fprintf(stderr,"Failed to send SHUTTER_CONF.");
      ret = 1;
    }
  
  /* Now let's wait a while and see if we get an answer. */

  
  start = time(NULL);
  while(time(NULL) < start+TIME_OUT && *(shutter_conf.name) == 0)
	{
	  process_server_socket(shutter_server);
	}
  
  if (*(shutter_conf.name) == 0)
    {
      fprintf(stderr,"Timed out waiting for response from server.\n");
      ret = 1;
    }
  

  //  while(1)
  //  process_server_socket(shutter_server); // need to run this from time to time
  
  return ret;
}



/************************************************************************/
/* setup_messages()							*/
/*									*/
/* Guess what this does.						*/
/************************************************************************/

int setup_messages(void)
{
        if (!add_message_job(shutter_server, NOOP, message_noop)) return FALSE;
        if (!add_message_job(shutter_server, MESSAGE, message_message)) return FALSE;
        if (!add_message_job(shutter_server, PING, message_ping)) return FALSE;
        if (!add_message_job(shutter_server, SERVER_STOPPING, 
		message_server_stopping)) return FALSE;
        if (!add_message_job(shutter_server, STATUS_SHUTTER, message_shutter_status)) 
		return FALSE;
        if (!add_message_job(shutter_server, SHUTTER_CONF, message_shutter_conf)) 
		return FALSE;

	return TRUE;

} /* setup_messages() */

/************************************************************************/
/* message_shutter_status()						*/
/*									*/
/************************************************************************/

int message_shutter_status(int server, struct smessage *mess)
{
	struct sshutter_status *status;

        if (mess->length != sizeof(*status))
        {
		fprintf(stderr,
			"Receiced STATUS_SHUTTER with wrong amount of data.");
		return FALSE;
        }

	status = (struct sshutter_status *)mess->data;

	printf("Received shutter status\n");

	if (status->status == INSANE)
	{
		fprintf(stderr,
			"ERROR\nShutter %s is reported as INSANE!",
			shutter_conf.shutter_names[status->shutter]);
	}

        return TRUE;

} /* message_shutter_status() */

/************************************************************************/
/* message_shutter_conf()						*/
/*									*/
/************************************************************************/

int message_shutter_conf(int server, struct smessage *mess)
{
	int	i;
	unsigned char	*p1, *p2;

	/* OK, we just copy the thing we got in the message */

	if (mess->length != sizeof(shutter_conf))
	{
	    fprintf(stderr,"Wrong number of bytes in configuration.");
	    return FALSE;
	}

	printf("Received shutter configuration\n");

	for(i=0, p1 = (unsigned char *)&shutter_conf, p2 = mess->data;
		i < sizeof(shutter_conf); i++, p1++, p2++) *p1 = *p2;

	return TRUE;

} /* message_shutter_conf() */

/************************************************************************/
/* send_shutter_open()							*/
/* 									*/
/* Send the OPEN_SHUTTER message to the server.				*/
/* Returns TRUE or FALSE.						*/
/************************************************************************/

int send_shutter_open(unsigned char shutter)
{
	struct smessage	mess;

	/* Make sure everything is OK */

	if (!server_socket_open(shutter_server)) return FALSE;

	if (shutter >= NUMBER_SHUTTERS) return FALSE;

	/* Construct command */

	mess.type = OPEN_SHUTTER;
	mess.length = sizeof(shutter);
	mess.data = &shutter;

	if (!send_message(shutter_server, &mess))
		fprintf(stderr,"Failed to send OPEN_SHUTTER message.");

	return TRUE;

} /* send_shutter_open() */

/************************************************************************/
/* send_shutter_close()							*/
/* 									*/
/* Send the CLOSE_SHUTTER message to the server.			*/
/* Returns an TRUE or FALSE.						*/
/************************************************************************/

int send_shutter_close(unsigned char shutter)
{
	struct smessage	mess;

	/* Make sure everything is OK */

	if (!server_socket_open(shutter_server)) return FALSE;

	if (shutter >= NUMBER_SHUTTERS) return FALSE;

	/* Construct command */

	mess.type = CLOSE_SHUTTER;
	mess.length = sizeof(shutter);
	mess.data = &shutter;

	if (!send_message(shutter_server, &mess))
		fprintf(stderr,"Failed to send CLOSE_SHUTTER message.");

	return TRUE;

} /* send_shutter_close() */

/************************************************************************/
/* send_shutter_status()						*/
/* 									*/
/* Send the STATUS_SHUTTER message to the server.			*/
/* Returns an TRUE or FALSE.						*/
/************************************************************************/

int send_shutter_status(unsigned char shutter)
{
	struct smessage	mess;
	struct sshutter_status status;

	/* Make sure everything is OK */

	if (!server_socket_open(shutter_server)) return FALSE;

	if (shutter >= NUMBER_SHUTTERS) return FALSE;

	/* Construct command */

	status.shutter = shutter;
	status.status  = 0;
	mess.type = STATUS_SHUTTER;
	mess.length = sizeof(status);
	mess.data = (unsigned char *)&status;

	if (!send_message(shutter_server, &mess))
		fprintf(stderr,"Failed to send STATUS_SHUTTER message.");

	return TRUE;

} /* send_shutter_status() */

/************************************************************************/
/* shutter_id()                                                         */
/*                                                                      */
/* Given the string name for a shutter as an argument returns the       */
/* shutter number for that shutter, or -1 if no name like that is       */
/* found.                                                               */
/************************************************************************/

int shutter_id(char *name)
{
        int     id;

        for (id=0; id<NUMBER_SHUTTERS; id++)
        {
                if (strcmp(name,shutter_conf.shutter_names[id]) == 0)
                {
                        return id;
                }
        }

        return -1;

} /* shutter_id() */

/************************************************************************/
/* message_noop()							*/
/*									*/
/* A message to do nothing.						*/
/************************************************************************/

int message_noop(int server, struct smessage *mess)
{
	printf("\nI just received a NOOP message.\n");
	return TRUE;
}

/************************************************************************/
/* message_message()							*/
/*									*/
/* A message to display a message.					*/
/************************************************************************/

int message_message(int server, struct smessage *mess)
{
        char *s,*p1,*p2;
        int     i;

        if (mess->length == 0)
        {
		return TRUE;
        }
        else
        {
                if ((s = malloc(mess->length+1)) == NULL)
                {
			fprintf(stderr,
				"Ran out of memory getting message string.\n");
			exit(-7);
                }

                for(i=0, p1=s, p2=(char *)mess->data;
                    i < mess->length; i++, p1++, p2++) *p1 = *p2;
                *p1 = 0;

		printf("\nMessage from server: %s\n", s);

                free(s);
        }

	return TRUE;

} /* message_message() */

/************************************************************************/
/* message_ping()							*/
/*									*/
/* If we get pinged by server, although why I do not know.		*/
/************************************************************************/

int message_ping(int server, struct smessage *mess)
{
        fprintf(stderr, "Example Client is alive.");
	return TRUE;

} /* message_ping() */

/************************************************************************/
/* message_server_stopping()						*/
/*									*/
/* What do we do if we get told the server has died?			*/
/************************************************************************/

int message_server_stopping(int server, struct smessage *mess)
{
	printf("\n\nServer is stopping.\n");
	close_server_socket(server);
	server = -1;
	return TRUE;

} /* message_server_stopping() */









/*                                                                                                                                                                                                                                           
 // note to test to make tcp/ip faster                                                                                                                                                                                                       
 int  nagle = 1;                                                                                                                                                                                                                             
 if (setsockopt(message_socket, IPPROTO_TCP, TCP_NODELAY,                                                                                                                                                                                    
 (char *)&nagle, sizeof(nagle) ))                                                                                                                                                                                                            
 {                                                                                                                                                                                                                                           
 close(message_socket);                                                                                                                                                                                                                      
 message_socket = -1;                                                                                                                                                                                                                        
 continue;                                                                                                                                                                                                                                   
 }                                                                                                                                                                                                                                           
*/
